/* 00bsp.cdf - BSP configuration file */

/*
 * Copyright (c) 2014-2017 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

/*
modification history
--------------------
19jun17,mjn  text updates (F8862)
07nov16,c_l  make INCLUDE_I8253_AUX_CLK using correct timer name (V7PRO-3382)
14sep15,jlv  add fast warm boot support. (F3387)
17aug15,ten  V7PRO-2066 - Added more information to SYS_MODEL, remove "no boot op".
06mar15,dot  V7PRO-1830 - vxMpTablePrint routine contains text not enclosed in  
             comments. Added multiple defautls to MP_APIC_DATA_FUNCTION 
             parameter based on usr boot option. 
10sep14,rec  Add INCLUDE_GEI_MSI_DISABLE for HV
18dec14,j_l  US48352 - Fix orphaned components
18nov14,scm  V7PRO-1413 System browser shows erratic data on itl_64_vx7 BSP
02jun14,jmz  US24640 - Remove unneeded HPET configuration options
08may14,scm  US32988/TA40489 - Test/Debug/Validate IA GEN2 drivers, IPIs...
06may14,jmz  US35979 - Move SIO configuration to 20IaDrvCfg.cdf
             US22498/TA49584 - remove references to MC146818...
01may14,j_l  US37558 - Create IA configuration configlettes for
                       VxBus GEN2 drivers
29apr14,j_l  US37556 - Convert *MPT* names to *IADT*
14apr14,scm  US24642/TA47728 Port I8253 Timer Driver to Gen2...
04apr14,scm  TA46620 Port IA Timestamp Timer Driver to Gen2...
21mar14,scm  TA42436 Port PC_CONSOLE to vxbus gen2 for IA
27mar14,j_l  US30676 - remove config.h
26mar14,jmz  temporarily remove HPET dependence on timestamp driver
21mar14,scm  TA42436 Port PC_CONSOLE to vxbus gen2 for IA
12mar14,jmz  Set default sysClk to Local APIC Timer
06mar14,j_l  replace PAGE_SIZE_4KB with 4096 constant,
             prjParams.h no longer includes external header files.
04mar14,scm  TA42438 - add 16550 support for IA
04feb14,scm  US20759 - remove unsupported interrupt modes...
03feb14,j_l  US23032 - IA BSP with VxBus GEN2 core library
31jan14,j_l  initial creation based on itl_64 version 05sep13
*/

/*
DESCRIPTION
This file overrides generic BSP components in comps/vxWorks/00bsp.cdf with

itl_64_vx7 BSP-specific versions of components and parameters defined in
the generic CDF file.
*/


/*******************************************************************************
*
* Memory definitions
*
*/

Parameter LOCAL_MEM_SIZE {
    NAME        Local memory size
    SYNOPSIS    This parameter specifies the fixed (static) memory size.
    DEFAULT     (SYSTEM_RAM_SIZE - \
                 ((LOCAL_MEM_LOCAL_ADRS) & (LOCAL_MEM_PHYS_ADRS_MASK)))
}

Parameter LOCAL_MEM_LOCAL_ADRS {
    NAME        Virtual memory base address
    DEFAULT     0xffffffff80100000
}

Parameter RAM_HIGH_ADRS {
    NAME        Bootrom copy region
    DEFAULT     (INCLUDE_BOOT_RAM_IMAGE)::(0xffffffff96408000) \
                (INCLUDE_BOOT_APP)::(0x00608000) \
                0xffffffff90008000
}

Parameter RAM_LOW_ADRS {
    NAME        Runtime kernel load address
    DEFAULT     (INCLUDE_BOOT_RAM_IMAGE)::(0xffffffff95008000) \
                (INCLUDE_BOOT_APP)::(0xffffffff90008000) \
                0xffffffff80408000
}

Parameter RESERVED {
    NAME        Reserved
    SYNOPSIS    Reserved.
    DEFAULT     0
}

Parameter ROM_TEXT_ADRS {
    NAME        ROM text address
    SYNOPSIS    This parameter defines the ROM text address.
    DEFAULT     (ROM_BASE_ADRS)
}

Parameter ROM_SIZE {
    NAME        ROM size
    SYNOPSIS    This parameter specifies the ROM size.
    DEFAULT     (INCLUDE_MULTI_STAGE_BOOT)::(0x00200000) \
                (0x00090000)
}

Parameter ROM_BASE_ADRS {
    NAME        ROM base address
    SYNOPSIS    This parameter defines the ROM base address.
    DEFAULT     (INCLUDE_MULTI_STAGE_BOOT)::(0x00408000) \
                (INCLUDE_WARM_BOOT)::(0x00408000) \
                (0x00008000)
}

Parameter FIRST_TEXT_ADRS {
    NAME        First stage text address
    SYNOPSIS    This parameter defines the first stage text address of bootrom.
    _CFG_PARAMS INCLUDE_MEMORY_CONFIG
    DEFAULT     0x00008000
}

Parameter WARMBOOT_COPY_ADRS {
    NAME        Bootrom stub code copy destination address
    SYNOPSIS    This parameter specifies the Bootrom stub code copy destination address.
    _CFG_PARAMS INCLUDE_MEMORY_CONFIG
    DEFAULT     0x500
}

Parameter ROM_STACK {
    NAME        Bootrom initial stack pointer
    SYNOPSIS    This parameter defines the bootrom initial stack pointer.
    _CFG_PARAMS INCLUDE_MEMORY_CONFIG
    DEFAULT     0x7000
}

Parameter ROM_WARM_HIGH {
    NAME        Warm reboot bootrom address offset
    SYNOPSIS    This parameter specifies the warm reboot bootrom address offset.
    _CFG_PARAMS INCLUDE_MEMORY_CONFIG
    DEFAULT     0x10
}

Parameter GDT_BASE_OFFSET {
    NAME        GDT offset
    SYNOPSIS    This parameter specifies the base GDT offset.
    _CFG_PARAMS INCLUDE_MEMORY_CONFIG
    DEFAULT     0x1000
}

Parameter SM_ANCHOR_OFFSET {
    NAME        GDT offset
    SYNOPSIS    This parameter defines the anchor SM offset.
    _CFG_PARAMS INCLUDE_MEMORY_CONFIG
    DEFAULT     0x1100
}

Parameter BOOT_LINE_OFFSET {
    NAME        Boot line offset
    SYNOPSIS    This parameter defines the boot line offset.
    DEFAULT     0x1200
}

Parameter EXC_MSG_OFFSET {
    NAME        Exception message offset
    SYNOPSIS    This parameter specifies the exception message offset.
    DEFAULT     0x1300
}

Parameter MPAPIC_DATA_OFFSET {
    NAME        MPAPIC data offset
    SYNOPSIS    This parameter specifies the MPAPIC data offset.
    _CFG_PARAMS INCLUDE_MEMORY_CONFIG
    DEFAULT     0x2000
}

/*
 * This value is defined here to reserve
 * the memory offset in low memory.
 * It is codependent with the same value
 * defined in vxBiosE820LibSP.h.  Both values
 *  must match.
 */

Parameter BIOS_E820_DATA_OFFSET {
    NAME        E820 data offset
    SYNOPSIS    This parameter specifies the E820 data offset.
    _CFG_PARAMS INCLUDE_MEMORY_CONFIG
    DEFAULT     0x4000
}

Parameter BOOT_MULTI_STAGE_DATA_OFFSET {
    NAME        Multistage data offset
    SYNOPSIS    This paramadter specifies the multistage data offset.
    _CFG_PARAMS INCLUDE_MEMORY_CONFIG
    DEFAULT     0x5000
}

/*******************************************************************************
*
* System Clock, Auxiliary Clock and Timestamp Component and Parameters
*
*/

Component INCLUDE_TIMESTAMP {
    NAME              Timestamp counter
    SYNOPSIS          This component includes the timestamp counter.
    REQUIRES          SELECT_TIMESTAMP_COUNTER
    _CHILDREN         FOLDER_CLOCK
}

Selection SELECT_TIMESTAMP_COUNTER {
    NAME              Timestamp counter selector
    COUNT             1-1
    CHILDREN          INCLUDE_TSC_TIMESTAMP \
                      INCLUDE_HPET_TIMESTAMP
    _CHILDREN         FOLDER_CLOCK
#ifdef _WRS_CONFIG_SMP
    DEFAULTS          INCLUDE_HPET_TIMESTAMP
#else
    DEFAULTS          INCLUDE_TSC_TIMESTAMP
#endif /* _WRS_CONFIG_SMP */
}

Parameter TIMESTAMP_TIMER_NAME {
    NAME              Force configured timestamp timer clock use
    SYNOPSIS          This parameter includes a function to override the VxBus timer selection code.
    TYPE              string
    DEFAULT           (INCLUDE_TIMESTAMP)::("Intel Timestamp Timer") \
                      (0)
}

Component INCLUDE_SYS_CLK  {
    NAME              System clock
    SYNOPSIS          This component includes the system clock.
    REQUIRES          SELECT_SYS_CLK
    _CHILDREN         FOLDER_CLOCK
}

Parameter SYS_CLK_RATE_MAX  {
    NAME              Maximum system clock rate
    SYNOPSIS          This parameter specifies the maximum system clock rate.
    TYPE              uint
    DEFAULT           (INCLUDE_HPET_SYS_CLK)::(5001) \
                      (5000)
}

Parameter SYS_CLK_RATE_MIN  {
    NAME              Minimum system clock rate
    SYNOPSIS          This parameter specifies the minimum system clock rate.
    TYPE              uint
    DEFAULT           (19)
}

Parameter SYS_CLK_RATE {
    NAME              System clock rate
    SYNOPSIS          This parameter specifies the default system clock rate in number of ticks per second.
    TYPE              uint
    DEFAULT           (60)
}

Component INCLUDE_AUX_CLK  {
    NAME              Auxiliary clock
    SYNOPSIS          This component includes the auxiliary clock component.
    REQUIRES          SELECT_AUX_CLK
    _CHILDREN         FOLDER_CLOCK
}

Parameter AUX_CLK_RATE_MAX  {
    NAME              Maximum auxiliary clock rate
    SYNOPSIS          This parameter specifies the maximum auxiliary clock rate.
    TYPE              uint
    DEFAULT           (INCLUDE_I8253_AUX_CLK)::(5000) \
                      (INCLUDE_HPET_AUX_CLK)::(5001) \
                      (5000)
}

Parameter AUX_CLK_RATE_MIN  {
    NAME              Auxiliary clock configuration parameter
    SYNOPSIS          This parameter specifies the minimum auxiliary clock rate.
    TYPE              uint
    DEFAULT           (INCLUDE_I8253_AUX_CLK)::(19) \
                      (INCLUDE_HPET_AUX_CLK)::(1) \
                      (19)
}

Parameter AUX_CLK_RATE  {
    NAME              Auxiliary clock rate
    SYNOPSIS          This parameter specifies the default auxiliary clock rate.
    TYPE              uint
    DEFAULT           (128)
}

Component INCLUDE_LOAPIC_SYS_CLK  {
    NAME              LOAPIC system clock
    SYNOPSIS          This component includes the LOAPIC system clock.
    REQUIRES          INCLUDE_SYS_CLK \
                      DRV_TIMER_LOAPIC \
                      INCLUDE_VXB_SYS_CLK
}

Component INCLUDE_I8253_SYS_CLK  {
    NAME              I8253 system clock
    SYNOPSIS          This component includes the I8253 system clock.
    REQUIRES          INCLUDE_SYS_CLK \
                      DRV_TIMER_I8253 \
                      INCLUDE_VXB_SYS_CLK
}

Component INCLUDE_I8253_AUX_CLK  {
    NAME              I8253 auxiliary clock
    SYNOPSIS          This component includes the I8253 auxiliary clock.
    REQUIRES          INCLUDE_AUX_CLK \
                      DRV_TIMER_I8253 \
                      INCLUDE_VXB_AUX_CLK
}

Component INCLUDE_TSC_TIMESTAMP {
    NAME              TSC timestamp
    SYNOPSIS          This component includes functions providing TSC timestamp functionality.
    REQUIRES          INCLUDE_TIMESTAMP \
                      DRV_TIMER_IA_TIMESTAMP
}

Component INCLUDE_HPET_TIMESTAMP  {
    NAME              HPET timestamp
    SYNOPSIS          This component includes functions providing HPET timestamp functionality.
    REQUIRES          INCLUDE_TIMESTAMP \
                      DRV_TIMER_IA_HPET \
                      DRV_TIMER_IA_HPET_BUS \
                      DRV_TIMER_IA_TIMESTAMP
}

Component INCLUDE_HPET_SYS_CLK  {
    NAME              HPET system clock
    SYNOPSIS          This component includes the HPET system clock.
    REQUIRES          INCLUDE_SYS_CLK \
                      DRV_TIMER_IA_HPET \
                      DRV_TIMER_IA_HPET_BUS \
                      INCLUDE_VXB_SYS_CLK
}

Component INCLUDE_HPET_AUX_CLK  {
    NAME              HPET auxiliary clock
    SYNOPSIS          This component includes the HPET auxiliary clock.
    REQUIRES          DRV_TIMER_IA_HPET \
                      DRV_TIMER_IA_HPET_BUS \
                      INCLUDE_VXB_AUX_CLK \
                      INCLUDE_AUX_CLK
}

Selection SELECT_SYS_CLK {
    NAME              System clock source selection
    COUNT             1-1
    CHILDREN          INCLUDE_HPET_SYS_CLK \
                      INCLUDE_I8253_SYS_CLK \
                      INCLUDE_LOAPIC_SYS_CLK
    _CHILDREN         FOLDER_CLOCK
    DEFAULTS          INCLUDE_LOAPIC_SYS_CLK
}

Parameter SYSCLK_TIMER_NAME {
    NAME              Force configured system clock use
    SYNOPSIS          This parameter overrides VxBus timer selection code to use a configured clock.
    TYPE              string
    DEFAULT           (INCLUDE_HPET_SYS_CLK)::("iaHpetTimerDev-0") \
                      (INCLUDE_I8253_SYS_CLK)::("Intel 8253 Timer") \
                      (INCLUDE_LOAPIC_SYS_CLK)::("loApicTimer") \
                      (0)
}

Selection SELECT_AUX_CLK {
    NAME              Auxiliary clock source selection
    COUNT             1-1
    CHILDREN          INCLUDE_I8253_AUX_CLK \
                      INCLUDE_HPET_AUX_CLK
    _CHILDREN         FOLDER_CLOCK
    DEFAULTS          INCLUDE_I8253_AUX_CLK
}

Parameter AUXCLK_TIMER_NAME {
    NAME              Force configured auxiliary clock use
    SYNOPSIS          This parameter overrides the VxBus timer selection code to use an auxiliary clock.
    TYPE              string
    DEFAULT           (INCLUDE_HPET_AUX_CLK)::("iaHpetTimerDev-1") \
                      (INCLUDE_I8253_AUX_CLK)::("Intel 8253 Timer") \
                      (0)
}

Parameter AUXCLK_TIMER_NUM {
    NAME              Requested unit number for auxiliary clock
    TYPE              uint
    DEFAULT           (INCLUDE_HPET_AUX_CLK)::(1)  \
                      (0)
}

/*******************************************************************************
*
* Cache Configuration Parameters
*
*/

Parameter USER_D_CACHE_MODE  {
    NAME              Data cache mode
    SYNOPSIS          This parameter selects the write-back data cache mode.
    TYPE              uint
    DEFAULT           (CACHE_COPYBACK | CACHE_SNOOP_ENABLE)
}

/*******************************************************************************
*
* Additional Intel Architecture show routines
*
*/

Component INCLUDE_INTEL_CPU_SHOW {
    NAME              Intel Architecture processor show routines
    SYNOPSIS          This component includes functions to printout information from the IA-32 processor.
    HDR_FILES         vxLib.h
    MODULES           vxShow.o
    INIT_RTN          vxShowInit ();
    _INIT_ORDER       usrShowInit
    _CHILDREN         FOLDER_SHOW_ROUTINES
    _DEFAULTS         += FOLDER_SHOW_ROUTINES
}

/*******************************************************************************
*
* itl_64_vx7 BSP-specific configuration folder
*
*/

Folder FOLDER_BSP_CONFIG  {
    NAME              itl_64_vx7 BSP configuration options
    SYNOPSIS          This folder provides BSP-specific configuration.
    CHILDREN          += INCLUDE_ITL_64_VX7_PARAMS \
                         INCLUDE_BOARD_CONFIG \
                         INCLUDE_LEGACY_USB_INT_MODE \
                         GRUB_MULTIBOOT \
                         INCLUDE_MULTI_STAGE_BOOT \
                         SELECT_MULTI_STAGE_REBOOT_TYPE
    CHILDREN          -= SELECT_AMP_CPU_NUMBER
    DEFAULTS          += INCLUDE_ITL_64_VX7_PARAMS
    _CHILDREN         FOLDER_HARDWARE
    _DEFAULTS         += FOLDER_HARDWARE
}

/*******************************************************************************
*
* BSP parameters Component
*
*/

Component INCLUDE_ITL_64_VX7_PARAMS  {
    NAME              BSP build parameters
    SYNOPSIS          This component exposes BSP configurable parameters.
    LAYER             1
    CFG_PARAMS        INCLUDE_MTRR_GET    \
                      MP_APIC_DATA_LOC \
                      MP_APIC_BOOT_OP \
                      MP_APIC_DATA_FUNCTION \
                      INCLUDE_PMC \
                      PIT_BASE_ADR \
                      PIT0_INT_LVL \
                      PIT_REG_ADDR_INTERVAL \
                      PIT_CLOCK \
                      DIAG_CTRL \
                      BEEP_PITCH_L \
                      BEEP_PITCH_S \
                      BEEP_TIME_L \
                      BEEP_TIME_S \
                      VM_STATE_MASK_FOR_ALL \
                      VM_STATE_FOR_IO \
                      VM_STATE_FOR_MEM_OS \
                      VM_STATE_FOR_MEM_APPLICATION \
                      VM_STATE_FOR_PCI
    HELP              itl_64_vx7
}

#ifdef _WRS_CONFIG_SMP
Component INCLUDE_ITL_64_VX7_PARAMS  {
    CFG_PARAMS        += SYS_AP_LOOP_COUNT \
                         SYS_AP_TIMEOUT
}
#endif /* _WRS_CONFIG_SMP */

Parameter INCLUDE_MTRR_GET  {
    NAME              Show memory type register range
    SYNOPSIS          This parameter allows display of the memory type range register settings from the BIOS.
    TYPE              exists
    DEFAULT           TRUE
}

Parameter INCLUDE_PMC  {
    NAME              Performance monitoring counter support
    SYNOPSIS          This parameter enables support for the performance monitoring counter library.
    TYPE              exists
    DEFAULT           TRUE
}

/*******************************************************************************
*
* Physical Address Space Components
*
*/

Component INCLUDE_MMU_P6_32BIT  {
    NAME              32-bit physical address space
    SYNOPSIS          This component includes support for configuring the 32-bit address space.
    CFG_PARAMS        VM_PAGE_SIZE
    EXCLUDES          INCLUDE_MMU_P6_36BIT
    _CHILDREN         FOLDER_MMU
    _DEFAULTS         += FOLDER_MMU
    HELP              itl_64_vx7
}


Parameter VM_PAGE_SIZE {
    NAME              VM page size
    SYNOPSIS          This parameter specifies the virtual memory page size.
                      (0x1000(4KB), 0x200000(2MB), 0x400000(4MB))
    TYPE              uint
    DEFAULT           0x1000
}

/*******************************************************************************
*
* Legacy SMI USB interrupt mode.  Only used in PC CONSOLE mode.
*
*/

Component INCLUDE_LEGACY_USB_INT_MODE {
    NAME              PC console legacy SMI USB interrupt
    SYNOPSIS          This component turns on legacy SMI USB interrupt mode while in PC console mode.
    REQUIRES          INCLUDE_PC_CONSOLE
    INCLUDE_WHEN      INCLUDE_PC_CONSOLE
}

/*******************************************************************************
*
* ACPI MPAPIC initialization component
*
*/

Component INCLUDE_USR_MPAPIC {
    NAME            MPAPIC boot component
    SYNOPSIS        This component includes support for MPAPIC boot initialization.
    CONFIGLETTES    usrMpapic.c
    _CHILDREN       FOLDER_NOT_VISIBLE
    _REQUIRES       INCLUDE_VXBUS
}

/*******************************************************************************
*
* GEN2 USER CONFIGURABLE INITIALIZATION COMPONENTS
*
*******************************************************************************/

/*******************************************************************************
*
* MPAPIC initialization components
*
*/

Parameter MP_APIC_DATA_LOC {
    NAME            mpApic data location
    SYNOPSIS        This parameter specifies the memory location where MP data is stored.
    TYPE            unsigned long
    DEFAULT         MPAPIC_DATA_START
}

Parameter       MP_APIC_BOOT_OP {
    NAME        mpApic boot option
    SYNOPSIS    This parameter specifies the startup structure to be used for initialization.
    TYPE        uint
    DEFAULT     (INCLUDE_MPTABLE_BOOT_OP)::(MP_MP_STRUCT) \
                (INCLUDE_ACPI_BOOT_OP)::(ACPI_MP_STRUCT) \
                (INCLUDE_USR_BOOT_OP)::(USR_MP_STRUCT) \
                (MP_MP_STRUCT)
}

Parameter     MP_APIC_DATA_FUNCTION {
    NAME      mpApic get user defined data function
    SYNOPSIS  This parameter specifies a user defined function which is used to \
              retrieve the startup data.
    TYPE      unsigned long
    DEFAULT   (INCLUDE_MPTABLE_BOOT_OP)::((unsigned long *) NULL) \
              (INCLUDE_ACPI_BOOT_OP)::((unsigned long *) NULL) \
              (INCLUDE_USR_BOOT_OP)::((unsigned long *)&sysStaticMpApicDataGet) \
              ((unsigned long *) NULL)
}

/*******************************************************************************
*
* 8253/8254 Programmable Interrupt Timers (PIT)
*
*/

Parameter PIT_BASE_ADR {
    NAME            8253/8254 PIT counter 0 port address
    SYNOPSIS        This parameter specifies the 8253/8254 programmable interrupt timer (PIT) counter 0 port address.
    DEFAULT         0x40
}

Parameter PIT0_INT_LVL {
    NAME            8253/8254 PIT interrupt level
    SYNOPSIS        This parameter specifies the 8253/8254 programmable interrupt timer (PIT) interrupt level.
    DEFAULT         0x2
}

Parameter PIT_REG_ADDR_INTERVAL {
    NAME            8253/8254 PIT register interval
    SYNOPSIS        This parameter specifies the 8253/8254 programmable interrupt timer \
                    (PIT) register interval.
    DEFAULT         1
}

Parameter PIT_CLOCK {
    NAME            8253/8254 PIT frequency
    SYNOPSIS        This parameter defines the 8253/8254 programmable interrupt timer (PIT) frequency.
    DEFAULT         1193182
}

/*******************************************************************************
*
* Beep generator (PC speaker)
* Only used when INCLUDE_PC_CONSOLE and DRV_TIMER_I8253 components are
* included.
*
*/

Parameter DIAG_CTRL {
    NAME            PC speaker control port
    SYNOPSIS        This parameter defines the PC speaker control port.
    DEFAULT         0x61
}

Parameter BEEP_PITCH_L {
    NAME            PC speaker long beep pitch
    SYNOPSIS        This parameter specifies the PC speaker long beep pitch. The default value is 1280 (932 Hz).
    DEFAULT         1280
}

Parameter BEEP_PITCH_S {
    NAME            PC speaker short beep pitch
    SYNOPSIS        This parameter specifies the PC speaker short beep pitch. The default value is 1208 (987 Hz).
    DEFAULT         1208
}

Parameter BEEP_TIME_L {
    NAME            PC speaker long beep duration
    SYNOPSIS        This parameter specifies the PC speaker long beep duration.
    DEFAULT         (sysClkRateGet () / 3)
}

Parameter BEEP_TIME_S {
    NAME            PC speaker short beep duration
    SYNOPSIS        This parameter defines the PC speaker short beep duration.
    DEFAULT         (sysClkRateGet () / 8)
}

/*******************************************************************************
*
* The VM_STATE_XXX macros
*
*/

Parameter VM_STATE_MASK_FOR_ALL {
    NAME            VM state mask
    DEFAULT         VM_STATE_MASK_VALID | VM_STATE_MASK_WRITABLE | \
                    VM_STATE_MASK_WBACK | VM_STATE_MASK_GLOBAL
}

Parameter VM_STATE_FOR_IO {
    NAME            IO VM state 
    DEFAULT         VM_STATE_VALID | VM_STATE_WRITABLE | \
                    VM_STATE_CACHEABLE_NOT | VM_STATE_GLOBAL_NOT
}

Parameter VM_STATE_FOR_MEM_OS {
    NAME            OS VM state 
    DEFAULT         VM_STATE_VALID | VM_STATE_WRITABLE | \
                    VM_STATE_WBACK | VM_STATE_GLOBAL_NOT
}

Parameter VM_STATE_FOR_MEM_APPLICATION {
    NAME            Application VM state
    DEFAULT         VM_STATE_VALID | VM_STATE_WRITABLE | \
                    VM_STATE_WBACK | VM_STATE_GLOBAL_NOT
}

Parameter VM_STATE_FOR_PCI {
    NAME            PCI VM state
    DEFAULT         VM_STATE_VALID | VM_STATE_WRITABLE | \
                    VM_STATE_CACHEABLE_NOT | VM_STATE_GLOBAL_NOT
}

/*******************************************************************************
*
*/

Parameter MAX_REBOOT_HOOKS {
        NAME            Maximum number of reboot hooks
        TYPE            int
        DEFAULT         8
}

/*******************************************************************************
*
* GEI MSI component
*
*/

Component INCLUDE_GEI_MSI_DISABLE {
     NAME               GEI MSI disable
     SYNOPSIS           This component disables the Intel GEI MSI support when included.
     _CHILDREN	        FOLDER_BSP_CONFIG
}
