/* 20IaDrvCfg.cdf - BSP driver configuration description file */

/*
 * Copyright (c) 2014, 2015, 2017 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

/*
modification history
--------------------
19jun17,mjn  text updates (F8862)
06jul15,dos  Moved VNIC specific portion into the vnic layer
30apr15,dot  US58966 - Added support for VxBus IA TPM device driver
08apr15,jmz  V7PRO-1994 - Do not share a node name between DRV_CONSOLE_EFI and
             DRV_VGA_M6845
31mar15,rbc  US54437 - Add default trigger and polarity for Walnut Canyon component
03mar15,rbc  US54437 - Add default trigger and polarity for Camelback Mountain component
26feb15,ten  US50798 - Choose SIO interrupt trigger type and level
15dec14,j_l  V7PRO-1491 - HPET driver not working in shared IRQs
17oct14,jmz  V7PRO-1403 - Add support for multiple PCI bus controllers
06oct14,rec  US45792 - Add VNIC
14aug14,j_l  V7PRO-1152 - Disable mouse entry in default configuration
12aug14,scm  US43452 - add back in sysSmi support routines
12may14,jmz  US35979 - Add support for DRV_IA_PCI_BUS
02jun14,jmz  US24640 - Add additional HPET configuration options
15may14,scm  US37146/TA52932 - Fix Aux Clk Test Failures seen...
06may14,jmz  US35979 - Update SIO driver based on IADT changes
01may14,j_l  US37558 - Create IA configuration configlettes for
                       VxBus GEN2 drivers
29apr14,j_l  US37556 - Convert *MPT* names to *IADT*
21apr14,j_l  US35957 - Add device list processing capability
28mar14,j_l  Fix compile warning
12mar14,jmz  Initial version
             Add support for Local APIC timer configuration
*/

/*******************************************************************************
*
* Intel Architecture Device Table (IADT) bus driver names
*
*/

Parameter DRV_BUS_IADT_ROOT_NAME {
    NAME            IADT bus driver name
    SYNOPSIS        This parameter specifies the IADT bus driver (DRV_BUS_IADT_ROOT) name.  Must match \
                    value in pkgs/os/drv/vxbus/drv/src/busCtlr/vxbIadtBus.c.
    TYPE            string
    DEFAULT         "iadtBus"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

Parameter DRV_SIO_IA_NS16550_NAME {
    NAME            ns16550 SIO driver name
    SYNOPSIS        This parameter specifies the ns16550 SIO driver (DRV_SIO_IA_NS16550) name.  Must match \
                    value in pkgs/os/drv/vxbus/drv/src/sio/vxbIaNs16550Sio.c.
    TYPE            string
    DEFAULT         "iaNs16550"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

Parameter DRV_KBD_I8042_NAME {
    NAME            Intel 8042 keyboard driver name
    SYNOPSIS        This parameter specifies the Intel 8042 keyboard driver (DRV_KBD_I8042) name.  Must \
                    match value in pkgs/os/platform/intel/src/vxbI8042Kbd.c.
    TYPE            string
    DEFAULT         "vxbI8042Kbd"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

Parameter DRV_TIMER_I8253_NAME {
    NAME            Intel 8253 timer driver name
    SYNOPSIS        This parameter specifies the i8253 timer driver (DRV_TIMER_I8253) name. Must match \
                    value in pkgs/os/platform/intel/src/vxbI8253Timer.c.
    TYPE            string
    DEFAULT         "Intel 8253 Timer"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

Parameter DRV_TIMER_IA_HPET_BUS_NAME {
    NAME            IA HPET bus driver name
    SYNOPSIS        This parameter specifies the Intel High Precision Event Timer (HPET) bus driver (DRV_TIMER_IA_HPET_BUS) name. Must \
                    match value in pkgs/os/platform/intel/src/vxbIaHpetBus.c.
    TYPE            string
    DEFAULT         "HPET-Bus"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

Parameter DRV_TIMER_IA_HPET_NAME {
    NAME            IA HPET timer driver name
    SYNOPSIS        This parameter specifies the Intel High Precision Event Timer (HPET) timer driver (DRV_TIMER_IA_HPET) name.  Must match \
                    value in pkgs/os/platform/intel/src/vxbIaHpetTimer.c.
    TYPE            string
    DEFAULT         "HPET-Timer"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

Parameter DRV_TIMER_IA_TIMESTAMP_NAME {
    NAME            Intel timestamp driver name
    SYNOPSIS        This parameter specifies the Intel timestamp driver (DRV_TIMER_IA_TIMESTAMP) name. Must \
                    match value in \
                    pkgs/os/platform/intel/src/vxbIntelTimestamp.c.
    TYPE            string
    DEFAULT         "Intel Timestamp Timer"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

Parameter DRV_INTCTLR_IOAPIC_NAME {
    NAME            IO APIC interrupt controller driver name
    SYNOPSIS        This parameter specifies the IO APIC interrupt controller driver (DRV_INTCTLR_IOAPIC) \
                    name.  Must match value in \
                    pkgs/os/platform/intel/src/vxbIoApicIntCtlr.c.
    TYPE            string
    DEFAULT         "ioApicIntrCtlr"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

Parameter DRV_INTCTLR_LOAPIC_NAME {
    NAME            Local APIC interrupt controller driver name
    SYNOPSIS        This parameter specifies the Local APIC interrupt controller driver \
                    (DRV_INTCTLR_LOAPIC) name.  Must match value in \
                    pkgs/os/platform/intel/src/vxbLoApicIntCtlr.c.
    TYPE            string
    DEFAULT         "loApicIntrCtlr"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

Parameter DRV_TIMER_LOAPIC_NAME {
    NAME            IA Local APIC timer driver name
    SYNOPSIS        This parameter specifies the IA Local APIC timer driver (DRV_TIMER_LOAPIC) name.  Must \
                    match value in pkgs/os/platform/intel/src/vxbLoApicTimer.c.
    TYPE            string
    DEFAULT         "LoApic-Timer"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

Parameter DRV_VGA_M6845_NAME {
    NAME            Motorola 6845 VGA driver name
    SYNOPSIS        This parameter specifies the Motorola 6845 VGA driver name.  DRV_VGA_M6845 and \
                    DRV_CONSOLE_EFI shares this name.  Must match values in \
                    pkgs/os/platform/intel/src/vxbEfiConsole.c and \
                    pkgs/os/platform/intel/src/vxbM6845Vga.c.
    TYPE            string
    DEFAULT         "vxbM6845Vga"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

Parameter DRV_CONSOLE_EFI_NAME {
    NAME            DRV_CONSOLE_EFI driver name
    SYNOPSIS        This parameter specifies the DRV_CONSOLE_EFI driver name. Must match values in \
                    pkgs/os/platform/intel/src/vxbEfiConsole.c
    TYPE            string
    DEFAULT         "vxbEfiConsole"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

Parameter DRV_IA_PCI_BUS_NAME {
    NAME            Intel PCI bus driver name
    SYNOPSIS        This parameter specifies the Intel PCI bus driver (DRV_IA_PCI_BUS) name.  Must \
                    match value in pkgs/os/platform/intel/src/vxbIaPciBus.c.
    TYPE            string
    DEFAULT         "iaPciBus"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

Parameter DRV_IA_TPM_NAME {
    NAME            Intel TPM driver name
    SYNOPSIS        This parameter specifies the Intel Architecture TPM device driver (DRV_IA_TPM) name. \
                    Must match value in pkgs/os/platform/intel/src/vxbIaTpm.c.
    TYPE            string
    DEFAULT         "iaTpm"
    _CFG_PARAMS     INCLUDE_ITL_64_VX7_PARAMS
}

/*******************************************************************************
*
* IA device table (static) init group
*
* Static portion is for devices that need to be initialized in a
* deterministic order.  Static portion uses index values 0 or greater
* when using usrIadtInit().  Actual static entries are specified
* further down below.
*
*/

InitGroup usrIaDevTableStaticInit {
    PROTOTYPE       void usrIaDevTableStaticInit (void)
    INIT_RTN        usrIaDevTableStaticInit ();
    SYNOPSIS        IA device table (static) initialization group.
    _INIT_ORDER     usrRoot
    INIT_BEFORE     usrIaDevTableDynamicInit
}

/*******************************************************************************
*
* IA device table (dynamic) init group
*
* Dynamic part is for devices that does not need to be initialized in
* a deterministic order.  This init group follows the static portion
* above.  Dynamic portion uses an index value of -1 when using
* usrIadtInit() and the items are appended to the end of the device
* list.  Actual dynamic entries are specified further down below.
*
*/

InitGroup usrIaDevTableDynamicInit {
    PROTOTYPE       void usrIaDevTableDynamicInit (void)
    INIT_RTN        usrIaDevTableDynamicInit ();
    SYNOPSIS        IA device table (dynamic) initialization group.
    _INIT_ORDER     usrRoot
    INIT_BEFORE     usrIaDevTableInit
}

/*******************************************************************************
*
* IA device table (process entries) init group
*
*/

InitGroup usrIaDevTableInit {
    PROTOTYPE       void usrIaDevTableInit (void)
    INIT_RTN        usrIaDevTableInit ();
    SYNOPSIS        IA device table (process entries) initialization group.
    _INIT_ORDER     usrRoot
    INIT_BEFORE     INCLUDE_VXBUS
}

/*******************************************************************************
*
* IADT bus device table to initialize
*
* The entries here represent the following tree of devices if all
* devices have only a single instance when dynamically adjusted (ones
* without @device_unit in entries.
*
* |-- iadtBus@0
* |   |-- ioApicIntrCtlr@0
*
* ioApicIntrCtlr is in the "static" portion of device table because it
* has to be the first driver to be initialized.
*
* Other drivers may be appended to this list in the "dynamic" portion
* of initialization.
*
* Please note that devices will only be initialized if a driver is
* available for it.
*
*/

Selection SELECT_DEV_TABLE {
    NAME            Device table selection
    SYNOPSIS        Select device table to initialize.
    COUNT           1-1
    CHILDREN        INCLUDE_DEV_TABLE_DEFAULT
    DEFAULTS        INCLUDE_DEV_TABLE_DEFAULT
    _CHILDREN       FOLDER_BSP_CONFIG
}

/***** INCLUDE_DEV_TABLE_DEFAULT *****/

Component INCLUDE_DEV_TABLE_DEFAULT {
    NAME            Device table
    SYNOPSIS        Device table to initialize.
    CONFIGLETTES    usrIadtInit.c
    PROTOTYPE       void usrIadtInit \
                        ( \
                        int    index, \
                        char * pNodeStr \
                        );
    REQUIRES        DEV_TABLE_DEFAULT_NODE_0 \
                    DEV_TABLE_NODE_LAST
    CFG_PARAMS      DEV_TABLE_MAX_COUNT \
                    DEV_TABLE_NODE_MAX_STRING_LENGTH
}

Component DEV_TABLE_DEFAULT_NODE_0 {
    NAME            Default device table node 0
    SYNOPSIS        This component specifies the default node 0 of the device table.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableStaticInit
    INIT_RTN        usrIadtInit \
                        ( \
                        DEV_TABLE_DEFAULT_NODE_0_INDEX, \
                        DEV_TABLE_DEFAULT_NODE_0_ENTRIES \
                        );
    CFG_PARAMS      DEV_TABLE_DEFAULT_NODE_0_INDEX   \
                    DEV_TABLE_DEFAULT_NODE_0_ENTRIES
}

Parameter DEV_TABLE_DEFAULT_NODE_0_INDEX {
    NAME            Default device table node 0 index
    TYPE            int
    DEFAULT         0
}

Parameter DEV_TABLE_DEFAULT_NODE_0_ENTRIES {
    NAME            Device table node 0 entries
    SYNOPSIS        Each entry is a list of comma delimited strings \
                    of which the first entry is the parent node and \
                    the rest are the names of its children. \
                    Individual fields are of the form \
                    device_name@device_unit or just device_name.  If \
                    device_unit is not specified, additional entries for \
                    device_name will be added dynamically up to the \
                    number of devices determined to be present on the \
                    board.
    DEFAULT         DRV_BUS_IADT_ROOT_NAME        "@0" "," \
                    DRV_INTCTLR_IOAPIC_NAME
}

/***** Common items *****/

Component DEV_TABLE_NODE_LAST {
    NAME            Device table last node
    SYNOPSIS        This component defines the last node of the device table.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableInit
    INIT_RTN        usrIadtInit \
                        ( \
                        0, \
                        NULL \
                        );
}

Parameter DEV_TABLE_MAX_COUNT {
    NAME            Device table maximum entries
    SYNOPSIS        This parameter specifies the maximum entries allowed in the device table. \
                    Number of device table entries excluding \
                    DEV_TABLE_NODE_LAST.  Must be 1 or greater.
    TYPE            int
    DEFAULT         20
}

Parameter DEV_TABLE_NODE_MAX_STRING_LENGTH {
    NAME            Device table node entry maximum string length
    SYNOPSIS        This parameter specifies the device table node entry maximum string length.
    TYPE            int
    DEFAULT         80
}

/*******************************************************************************
*
* DRV_INTCTLR_LOAPIC device table entry
*
*/

Component DRV_INTCTLR_LOAPIC {
    REQUIRES     += DRV_INTCTLR_LOAPIC_INSTANCE
}

Component DRV_INTCTLR_LOAPIC_INSTANCE {
    NAME            INTCTLR LOAPIC driver instance
    SYNOPSIS        This component specifies the INTCTLR LOAPIC driver instance.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_RTN        usrLoApicIntCtlrInit ();
}

/*******************************************************************************
*
* DRV_SIO_IA_NS16550
*
* Device table entry is automatically added when usrIaNs16550Init is
* called (usrIadtInit is called within usrIaNs16550Init).
*
* Only two UART Channels are supported out-of-the-box using defacto standard
* PC com port resources because the serial driver does not support shared
* interrupts. Up to 4 serial channels can be supported if the user is
* willing customize this BSP by finding unused IRQs. For example,
* IRQ5 and IRQ2(9) are often unused. IADT_SIO_N_UART_CHANNELS must be changed
* to reflect the actual number of serial ports supported.
*
* Sometimes the BIOS also needs adjustment.
*/

Component DRV_SIO_IA_NS16550 {
    REQUIRES             += DRV_SIO_IA_NS16550_INSTANCE_0
    CFG_PARAMS           += DRV_SIO_IA_NS16550_MAX_COUNT
}

Parameter DRV_SIO_IA_NS16550_MAX_COUNT {
    NAME                 Maximum instances allowed
    SYNOPSIS             This parameter specifies the driver maximum instances allowed. \
                         Each instance is of the form \
                         DRV_SIO_IA_NS16550_INSTANCE_<n> where <n> is the \
                         instance number starting with 0.
    TYPE                 uint
    DEFAULT              2
}

Component DRV_SIO_IA_NS16550_INSTANCE_0 {
    NAME                 SIO Intel NS16550 driver instance 0
    SYNOPSIS             This component specifies instance 0 of the SIO Intel NS16550 driver instance 0.
    _CHILDREN            FOLDER_BSP_CONFIG
    _INIT_ORDER          usrIaDevTableDynamicInit
    INIT_RTN             (void) usrIaNs16550Init ( \
                         0, \
                         (void *) DRV_SIO_IA_NS16550_INSTANCE_0_BASE_ADDR, \
                         DRV_SIO_IA_NS16550_INSTANCE_REG_INTERVAL, \
                         DRV_SIO_IA_NS16550_INSTANCE_0_INT_NUM, \
                         DRV_SIO_IA_NS16550_INSTANCE_0_INT_LEVEL, \
                         DRV_SIO_IA_NS16550_INSTANCE_FIFO_LEN, \
                         DRV_SIO_IA_NS16550_INSTANCE_CLK_FREQ, \
                         DRV_SIO_IA_NS16550_INSTANCE_BAUD_RATE, \
                         DRV_SIO_IA_NS16550_INSTANCE_0_TRIGGER, \
                         DRV_SIO_IA_NS16550_INSTANCE_0_POLARITY \
                         );
    CFG_PARAMS           DRV_SIO_IA_NS16550_INSTANCE_0_BASE_ADDR \
                         DRV_SIO_IA_NS16550_INSTANCE_REG_INTERVAL \
                         DRV_SIO_IA_NS16550_INSTANCE_0_INT_NUM \
                         DRV_SIO_IA_NS16550_INSTANCE_0_INT_LEVEL \
                         DRV_SIO_IA_NS16550_INSTANCE_FIFO_LEN \
                         DRV_SIO_IA_NS16550_INSTANCE_CLK_FREQ \
                         DRV_SIO_IA_NS16550_INSTANCE_BAUD_RATE \
                         DRV_SIO_IA_NS16550_INSTANCE_0_TRIGGER \
                         DRV_SIO_IA_NS16550_INSTANCE_0_POLARITY
}

Component DRV_SIO_IA_NS16550_INSTANCE_1 {
    NAME                 SIO Intel NS16550 driver instance 1
    SYNOPSIS             This component specifies instance 1 of the SIO Intel NS16550 driver.
    _CHILDREN            FOLDER_BSP_CONFIG
    _INIT_ORDER          usrIaDevTableDynamicInit
    INIT_RTN             (void) usrIaNs16550Init ( \
                         1, \
                         (void *) DRV_SIO_IA_NS16550_INSTANCE_1_BASE_ADDR, \
                         DRV_SIO_IA_NS16550_INSTANCE_REG_INTERVAL, \
                         DRV_SIO_IA_NS16550_INSTANCE_1_INT_NUM, \
                         DRV_SIO_IA_NS16550_INSTANCE_1_INT_LEVEL, \
                         DRV_SIO_IA_NS16550_INSTANCE_FIFO_LEN, \
                         DRV_SIO_IA_NS16550_INSTANCE_CLK_FREQ, \
                         DRV_SIO_IA_NS16550_INSTANCE_BAUD_RATE, \
                         DRV_SIO_IA_NS16550_INSTANCE_1_TRIGGER, \
                         DRV_SIO_IA_NS16550_INSTANCE_1_POLARITY \
                         );
    CFG_PARAMS           DRV_SIO_IA_NS16550_INSTANCE_1_BASE_ADDR \
                         DRV_SIO_IA_NS16550_INSTANCE_REG_INTERVAL \
                         DRV_SIO_IA_NS16550_INSTANCE_1_INT_NUM \
                         DRV_SIO_IA_NS16550_INSTANCE_1_INT_LEVEL \
                         DRV_SIO_IA_NS16550_INSTANCE_FIFO_LEN \
                         DRV_SIO_IA_NS16550_INSTANCE_CLK_FREQ \
                         DRV_SIO_IA_NS16550_INSTANCE_BAUD_RATE \
                         DRV_SIO_IA_NS16550_INSTANCE_1_TRIGGER \
                         DRV_SIO_IA_NS16550_INSTANCE_1_POLARITY
}

Component DRV_SIO_IA_NS16550_INSTANCE_2 {
    NAME                 SIO Intel NS16550 driver instance 2
    SYNOPSIS             This component specifies instance 2 of the SIO Intel NS16550 driver.
    _CHILDREN            FOLDER_BSP_CONFIG
    _INIT_ORDER          usrIaDevTableDynamicInit
    INIT_RTN             (void) usrIaNs16550Init ( \
                         2, \
                         (void *) DRV_SIO_IA_NS16550_INSTANCE_2_BASE_ADDR, \
                         DRV_SIO_IA_NS16550_INSTANCE_REG_INTERVAL, \
                         DRV_SIO_IA_NS16550_INSTANCE_2_INT_NUM, \
                         DRV_SIO_IA_NS16550_INSTANCE_2_INT_LEVEL, \
                         DRV_SIO_IA_NS16550_INSTANCE_FIFO_LEN, \
                         DRV_SIO_IA_NS16550_INSTANCE_CLK_FREQ, \
                         DRV_SIO_IA_NS16550_INSTANCE_BAUD_RATE, \
                         DRV_SIO_IA_NS16550_INSTANCE_2_TRIGGER, \
                         DRV_SIO_IA_NS16550_INSTANCE_2_POLARITY \
                         );
    CFG_PARAMS           DRV_SIO_IA_NS16550_INSTANCE_2_BASE_ADDR \
                         DRV_SIO_IA_NS16550_INSTANCE_REG_INTERVAL \
                         DRV_SIO_IA_NS16550_INSTANCE_2_INT_NUM \
                         DRV_SIO_IA_NS16550_INSTANCE_2_INT_LEVEL \
                         DRV_SIO_IA_NS16550_INSTANCE_FIFO_LEN \
                         DRV_SIO_IA_NS16550_INSTANCE_CLK_FREQ \
                         DRV_SIO_IA_NS16550_INSTANCE_BAUD_RATE \
                         DRV_SIO_IA_NS16550_INSTANCE_2_TRIGGER \
                         DRV_SIO_IA_NS16550_INSTANCE_2_POLARITY
}

Component DRV_SIO_IA_NS16550_INSTANCE_3 {
    NAME                 SIO Intel NS16550 driver instance 3
    SYNOPSIS             This component specifies instance 3 of the SIO Intel NS16550 driver.
    _CHILDREN            FOLDER_BSP_CONFIG
    _INIT_ORDER          usrIaDevTableDynamicInit
    INIT_RTN             (void) usrIaNs16550Init ( \
                         3, \
                         (void *) DRV_SIO_IA_NS16550_INSTANCE_3_BASE_ADDR, \
                         DRV_SIO_IA_NS16550_INSTANCE_REG_INTERVAL, \
                         DRV_SIO_IA_NS16550_INSTANCE_3_INT_NUM, \
                         DRV_SIO_IA_NS16550_INSTANCE_3_INT_LEVEL, \
                         DRV_SIO_IA_NS16550_INSTANCE_FIFO_LEN, \
                         DRV_SIO_IA_NS16550_INSTANCE_CLK_FREQ, \
                         DRV_SIO_IA_NS16550_INSTANCE_BAUD_RATE, \
                         DRV_SIO_IA_NS16550_INSTANCE_3_TRIGGER, \
                         DRV_SIO_IA_NS16550_INSTANCE_3_POLARITY \
                         );
    CFG_PARAMS           DRV_SIO_IA_NS16550_INSTANCE_3_BASE_ADDR \
                         DRV_SIO_IA_NS16550_INSTANCE_REG_INTERVAL \
                         DRV_SIO_IA_NS16550_INSTANCE_3_INT_NUM \
                         DRV_SIO_IA_NS16550_INSTANCE_3_INT_LEVEL \
                         DRV_SIO_IA_NS16550_INSTANCE_FIFO_LEN \
                         DRV_SIO_IA_NS16550_INSTANCE_CLK_FREQ \
                         DRV_SIO_IA_NS16550_INSTANCE_BAUD_RATE \
                         DRV_SIO_IA_NS16550_INSTANCE_3_TRIGGER \
                         DRV_SIO_IA_NS16550_INSTANCE_3_POLARITY
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_0_BASE_ADDR {
    NAME                Base address
    SYNOPSIS             This parameter specifies the base address for com 1.
    TYPE                 (void *)
    DEFAULT              (0x3f8)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_1_BASE_ADDR {
    NAME                 Base address
    SYNOPSIS             This parameter specifies the base address for com 2.
    TYPE                 (void *)
    DEFAULT              (0x2f8)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_2_BASE_ADDR {
    NAME                 Base address
    SYNOPSIS             This parameter specifies the base address for com 3.
    TYPE                 (void *)
    DEFAULT              (0x3e8)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_3_BASE_ADDR {
    NAME                 Base address
    SYNOPSIS             This parameter specifies the base address for com 4.
    TYPE                 (void *)
    DEFAULT              (0x2e8)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_REG_INTERVAL {
    NAME                 Register interval
    SYNOPSIS             This parameter specifies the UART register address interval.
    TYPE                 UINT16
    DEFAULT              (0x1)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_0_INT_NUM {
    NAME                 Interrupt number
    SYNOPSIS             This parameter specifies the interrupt number for com 1.
    TYPE                 uint
    DEFAULT              (0x9c)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_1_INT_NUM {
    NAME                 Interrupt number
    SYNOPSIS             This parameter specifies the interrupt number for com 2.
    TYPE                 uint
    DEFAULT              (0xa4)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_2_INT_NUM {
    NAME                 Interrupt number
    SYNOPSIS             This parameter specifies the interrupt number for com 3.
    TYPE                 uint
    DEFAULT              (0x94)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_3_INT_NUM {
    NAME                 Interrupt number
    SYNOPSIS             This parameter specifies the interrupt number for com 4.
    TYPE                 uint
    DEFAULT              (0x7c)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_0_INT_LEVEL {
    NAME                 Interrupt level
    SYNOPSIS             This parameter specifies the interrupt level for com 1.
    TYPE                 uint
    DEFAULT              (0x4)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_1_INT_LEVEL {
    NAME                 Interrupt level
    SYNOPSIS             This parameter specifies the interrupt level for com 2.
    TYPE                 uint
    DEFAULT              (0x3)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_2_INT_LEVEL {
    NAME                 Interrupt level
    SYNOPSIS             This parameter specifies the interrupt level for com 3.
    TYPE                 uint
    DEFAULT              (0x5)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_3_INT_LEVEL {
    NAME                 Interrupt level
    SYNOPSIS             This parameter specifies the interrupt level for com 4.
    TYPE                 uint
    DEFAULT              (0x9)
}


Parameter DRV_SIO_IA_NS16550_INSTANCE_0_TRIGGER {
    NAME                 Trigger
    SYNOPSIS             This parameter specifies the trigger type for the com1 interrupt, 0 is EDGE, 1 is LEVEL.
    TYPE                 uint
    DEFAULT              (INCLUDE_CAMELBACK_MOUNTAIN)::(1) \
                         (INCLUDE_WALNUT_CANYON_2)::(1) \
                         (INCLUDE_WALNUT_CANYON)::(1) \
                         (0)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_1_TRIGGER {
    NAME                 Trigger
    SYNOPSIS             This parameter specifies the trigger type for the com2 interrupt, 0 is EDGE, 1 is LEVEL.
    TYPE                 uint
    DEFAULT              0
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_2_TRIGGER {
    NAME                 Trigger
    SYNOPSIS             This parameter specifies the trigger type for the com3 interrupt, 0 is EDGE, 1 is LEVEL.
    TYPE                 uint
    DEFAULT              0
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_3_TRIGGER {
    NAME                 Trigger
    SYNOPSIS             This parameter specifies the trigger type for the com4 interrupt, 0 is EDGE, 1 is LEVEL.
    TYPE                 uint
    DEFAULT              0
}


Parameter DRV_SIO_IA_NS16550_INSTANCE_0_POLARITY {
    NAME                 Polarity
    SYNOPSIS             This parameter specifies the polarity for the com1 interrupt, 0 is HIGH, 1 is LOW.
    TYPE                 uint
    DEFAULT              (INCLUDE_CAMELBACK_MOUNTAIN)::(1) \
                         (INCLUDE_WALNUT_CANYON_2)::(0) \
                         (INCLUDE_WALNUT_CANYON)::(0) \
                         (0)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_1_POLARITY {
    NAME                 Polarity
    SYNOPSIS             This parameter specifies the polarity for the com2 interrupt, 0 is HIGH, 1 is LOW.
    TYPE                 uint
    DEFAULT              0
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_2_POLARITY {
    NAME                 Polarity
    SYNOPSIS             This parameter specifies the polarity for the com3 interrupt, 0 is HIGH, 1 is LOW.
    TYPE                 uint
    DEFAULT              0
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_3_POLARITY {
    NAME                 Polarity
    SYNOPSIS             This parameter specifies the polarity for the com4 interrupt, 0 is HIGH, 1 is LOW.
    TYPE                 uint
    DEFAULT              0
}



Parameter DRV_SIO_IA_NS16550_INSTANCE_FIFO_LEN {
    NAME                 FIFO length
    SYNOPSIS             This parameter specifies the fifo length.
    TYPE                 UINT16
    DEFAULT              (0x8)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_CLK_FREQ {
    NAME                 Clock frequency
    SYNOPSIS             This parameter specifies the PCI clock frequency.
    TYPE                 unsigned long
    DEFAULT              (1843200)
}

Parameter DRV_SIO_IA_NS16550_INSTANCE_BAUD_RATE {
    NAME                 Baud rate
    SYNOPSIS             This parameter specifies the PCI clock frequency.
    TYPE                 unsigned long
    DEFAULT              (CONSOLE_BAUD_RATE)
}

/*******************************************************************************
*
* DRV_VGA_M6845, DRV_CONSOLE_EFI device table entry
*
*/

Component DRV_VGA_M6845 {
    REQUIRES     += DRV_VGA_M6845_INSTANCE_0
    CFG_PARAMS   += DRV_VGA_M6845_MAX_COUNT
}

Parameter DRV_VGA_M6845_MAX_COUNT {
    NAME            Maximum instances allowed
    SYNOPSIS        This parameter specifies the maximum number of driver instances allowed. \
                    Each instance is of the form \
                    DRV_VGA_M6845_INSTANCE_<n> where <n> is the \
                    instance number starting with 0. \
                    DRV_VGA_M6845_MAX_COUNT can not be greater than \
                    M6845_MAX_DEV define in vxbM6845Vga.h.
    TYPE            int
    DEFAULT         1
}

Component DRV_VGA_M6845_INSTANCE_0 {
    NAME            VGA M6845 driver instance 0
    SYNOPSIS        This component defines instance 0 of the VGA M6845 driver.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_RTN        usrM6845VgaInit                                            \
                        (                                                      \
                        (int)           DRV_VGA_M6845_INSTANCE_0_INDEX,        \
                        (unsigned long) DRV_VGA_M6845_INSTANCE_0_REG_BASE_ADRS,\
                        (unsigned int)  DRV_VGA_M6845_INSTANCE_0_MEM_BASE_ADRS,\
                        (unsigned long) DRV_VGA_M6845_INSTANCE_0_COLOR_MODE,   \
                        (unsigned int)  DRV_VGA_M6845_INSTANCE_0_COLOR_SETTING \
                        );
    CFG_PARAMS      DRV_VGA_M6845_INSTANCE_0_INDEX         \
                    DRV_VGA_M6845_INSTANCE_0_REG_BASE_ADRS \
                    DRV_VGA_M6845_INSTANCE_0_MEM_BASE_ADRS \
                    DRV_VGA_M6845_INSTANCE_0_COLOR_MODE    \
                    DRV_VGA_M6845_INSTANCE_0_COLOR_SETTING
}

Parameter DRV_VGA_M6845_INSTANCE_0_INDEX {
    NAME            Instance index
    SYNOPSIS        This parameter specifies the VGA M6845 driver instance index. Set as 0 or (-1) to deactivate.
    TYPE            int
    DEFAULT         0
}

Parameter DRV_VGA_M6845_INSTANCE_0_REG_BASE_ADRS {
    NAME            Register base address
    SYNOPSIS        This parameter specifies the video controller chip register base address.
    TYPE            unsigned long
    DEFAULT         0x3d4
}

Parameter DRV_VGA_M6845_INSTANCE_0_MEM_BASE_ADRS {
    NAME            Memory base address
    SYNOPSIS        This parameter specifies the memory base address for VGA.
    TYPE            unsigned long
    DEFAULT         0xb8000
}

Parameter DRV_VGA_M6845_INSTANCE_0_COLOR_MODE {
    NAME            Color mode
    SYNOPSIS        This parameter specifies the specifies color mode, COLOR(1) or MONO(0).
    TYPE            uint
    DEFAULT         0x1
}

Parameter DRV_VGA_M6845_INSTANCE_0_COLOR_SETTING {
    NAME            Color setting
    SYNOPSIS        This parameter specifies the foreground and background display colors.
    TYPE            uint
    DEFAULT         0x1f
}

Component DRV_CONSOLE_EFI {
    REQUIRES     += DRV_CONSOLE_EFI_INSTANCE_0
    CFG_PARAMS   += DRV_CONSOLE_EFI_MAX_COUNT
}

Component DRV_CONSOLE_EFI_INSTANCE_0 {
    NAME            EFI Console driver instance 0
    SYNOPSIS        This component defines instance 0 of the EFI console driver instance 0.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_RTN        usrEfiConsoleInit                            \
                        (                                        \
                        (int) DRV_CONSOLE_EFI_INSTANCE_0_INDEX   \
                        );
    CFG_PARAMS      DRV_CONSOLE_EFI_INSTANCE_0_INDEX
}
    
Parameter DRV_CONSOLE_EFI_INSTANCE_0_INDEX {
    NAME            Instance index
    SYNOPSIS        This parameter specifies the EFI console instance index. Set to 0 or (-1) to deactivate.
    TYPE            int
    DEFAULT         0
}

Parameter DRV_CONSOLE_EFI_MAX_COUNT {
    NAME            Maximum instances allowed
    SYNOPSIS        This parameter specifies the maximum number of driver instances allowed. \
                    Each instance is of the form \
                    DRV_CONSOLE_EFI_INSTANCE_<n> where <n> is the \
                    instance number starting with 0.
    TYPE            int
    DEFAULT         1
}

/*******************************************************************************
*
* DRV_KBD_I8042 device table entry
*
*/

Component DRV_KBD_I8042 {
    REQUIRES     += DRV_KBD_I8042_INSTANCE_0 \
                    DRV_KBD_I8042_INSTANCE_1
    CFG_PARAMS   += DRV_KBD_I8042_MAX_COUNT
}

Parameter DRV_KBD_I8042_MAX_COUNT {
    NAME            Maximum instances allowed
    SYNOPSIS        This parameter specifies the maximum number of driver instances allowed. \
                    Each instance is of the form \
                    DRV_KBD_I8042_INSTANCE_<n> where <n> is the \
                    instance number starting with 0. \
                    DRV_KBD_I8042_MAX_COUNT can not be greater than \
                    I8042_MAX_DEV define in vxbI8042Kbd.h. \
                    Default set to 2 (keyboard and mouse)
    TYPE            int
    DEFAULT         2
}

/*** DRV_KBD_I8042 instance 0 ***/

Component DRV_KBD_I8042_INSTANCE_0 {
    NAME            KBD I8042 driver instance 0
    SYNOPSIS        This component defines instance 0 of the KBD I8042 driver for keyboard configuration.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_RTN        usrI8042KbdInit                                            \
                        (                                                      \
                        (int)           DRV_KBD_I8042_INSTANCE_0_INDEX,        \
                        (unsigned long) DRV_KBD_I8042_INSTANCE_0_BASE_ADRS,    \
                        (unsigned int)  DRV_KBD_I8042_INSTANCE_0_ADRS_INTERVAL,\
                        (int)           DRV_KBD_I8042_INSTANCE_0_MODE,         \
                        (unsigned int)  DRV_KBD_I8042_INSTANCE_0_IRQ_NUM,      \
                        (unsigned int)  DRV_KBD_I8042_INSTANCE_0_IRQ_LVL       \
                        );
    CFG_PARAMS      DRV_KBD_I8042_INSTANCE_0_INDEX         \
                    DRV_KBD_I8042_INSTANCE_0_BASE_ADRS     \
                    DRV_KBD_I8042_INSTANCE_0_ADRS_INTERVAL \
                    DRV_KBD_I8042_INSTANCE_0_MODE          \
                    DRV_KBD_I8042_INSTANCE_0_IRQ_NUM       \
                    DRV_KBD_I8042_INSTANCE_0_IRQ_LVL
}

Parameter DRV_KBD_I8042_INSTANCE_0_INDEX {
    NAME            Instance index
    SYNOPSIS        This parameter specifies the DRV_KBD_I8042 instance index. Set to 0 or (-1) to deactivate.
    TYPE            int
    DEFAULT         0
}

Parameter DRV_KBD_I8042_INSTANCE_0_BASE_ADRS {
    NAME            Base address
    SYNOPSIS        This parameter specifies the base address of the Intel 8042 keyboard controller chip.
    TYPE            unsigned long
    DEFAULT         0x60
}

Parameter DRV_KBD_I8042_INSTANCE_0_ADRS_INTERVAL {
    NAME            Address interval
    SYNOPSIS        This parameter specifies the register address interval of the Intel 8042 Keyboard \
                    Controller Chip.
    TYPE            unsigned int
    DEFAULT         0x4
}

Parameter DRV_KBD_I8042_INSTANCE_0_MODE {
    NAME            Instance mode
    SYNOPSIS        This parameter specifies the keyboard mode, Japanese (0) or English (1) key codes. \
                    (-1) when mouse is being configured.
    TYPE            int
    DEFAULT         0x1
}

Parameter DRV_KBD_I8042_INSTANCE_0_IRQ_NUM {
    NAME            Interrupt number
    SYNOPSIS        This parameter specifies the interrupt vector number for the Intel 8042 keyboard \
                    Controller interrupt source.
    TYPE            unsigned int
    DEFAULT         0xb4
}

Parameter DRV_KBD_I8042_INSTANCE_0_IRQ_LVL {
    NAME            Interrupt level
    SYNOPSIS        This parameter specifies the interrupt number for the Intel 8042 keyboard interrupt \
                    source.
    TYPE            unsigned int
    DEFAULT         0x1
}

/*** DRV_KBD_I8042 instance 1 ***/

Component DRV_KBD_I8042_INSTANCE_1 {
    NAME            KBD I8042 driver instance 1
    SYNOPSIS        This component defines instance 1 of the KBD I8042 driver for mouse configuration.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_RTN        usrI8042KbdInit                                            \
                        (                                                      \
                        (int)           DRV_KBD_I8042_INSTANCE_1_INDEX,        \
                        (unsigned long) DRV_KBD_I8042_INSTANCE_1_BASE_ADRS,    \
                        (unsigned int)  DRV_KBD_I8042_INSTANCE_1_ADRS_INTERVAL,\
                        (int)           DRV_KBD_I8042_INSTANCE_1_MODE,         \
                        (unsigned int)  DRV_KBD_I8042_INSTANCE_1_IRQ_NUM,      \
                        (unsigned int)  DRV_KBD_I8042_INSTANCE_1_IRQ_LVL       \
                        );
    CFG_PARAMS      DRV_KBD_I8042_INSTANCE_1_INDEX         \
                    DRV_KBD_I8042_INSTANCE_1_BASE_ADRS     \
                    DRV_KBD_I8042_INSTANCE_1_ADRS_INTERVAL \
                    DRV_KBD_I8042_INSTANCE_1_MODE          \
                    DRV_KBD_I8042_INSTANCE_1_IRQ_NUM       \
                    DRV_KBD_I8042_INSTANCE_1_IRQ_LVL
}

Parameter DRV_KBD_I8042_INSTANCE_1_INDEX {
    NAME            Instance index
    SYNOPSIS        This parameter specifies the DRV_KBD_I8042 instance index. Set to 1 or (-1) to deactivate.
    TYPE            int
    DEFAULT         (-1)
}

Parameter DRV_KBD_I8042_INSTANCE_1_BASE_ADRS {
    NAME            Base address
    SYNOPSIS        This parameter specifies the base address of the Intel 8042 keyboard controller chip.
    TYPE            unsigned long
    DEFAULT         0x60
}

Parameter DRV_KBD_I8042_INSTANCE_1_ADRS_INTERVAL {
    NAME            DRV_KBD_I8042_INSTANCE_1_ADRS_INTERVAL
    SYNOPSIS        This parameter specifies the register address interval of the Intel 8042 Keyboard \
                    controller chip.
    TYPE            unsigned int
    DEFAULT         0x4
}

Parameter DRV_KBD_I8042_INSTANCE_1_MODE {
    NAME            Keyboard mode
    SYNOPSIS        This parameter specifies the keyboard mode, Japanese (0) or English (1) key codes. \
                    (-1) when mouse is being configured.
    TYPE            int
    DEFAULT         (-1)
}

Parameter DRV_KBD_I8042_INSTANCE_1_IRQ_NUM {
    NAME            Interrupt number
    SYNOPSIS        This parameter specifies the interrupt vector number for the Intel 8042 keyboard \
                    controller interrupt source.
    TYPE            unsigned int
    DEFAULT         0x5c
}

Parameter DRV_KBD_I8042_INSTANCE_1_IRQ_LVL {
    NAME            Interrupt level
    SYNOPSIS        This parameter specifies the interrupt number for the Intel 8042 keyboard interrupt \
                    source.
    TYPE            unsigned int
    DEFAULT         0xc
}

/*******************************************************************************
*
* DRV_TIMER_IA_HPET_BUS device table entry
*
*/

Component DRV_TIMER_IA_HPET_BUS {
    REQUIRES     += DRV_TIMER_IA_HPET_BUS_INSTANCE_0
    CFG_PARAMS   += DRV_TIMER_IA_HPET_BUS_MAX_COUNT
}

Parameter DRV_TIMER_IA_HPET_BUS_MAX_COUNT {
    NAME            Maximum instances allowed
    SYNOPSIS        This parameter specifies the maximum number of driver instances allowed. \
                    Each instance is of the form \
                    DRV_TIMER_IA_HPET_BUS_INSTANCE_<n> where <n> is the \
                    instance number starting with 0.
    TYPE            int
    DEFAULT         1
}

Component DRV_TIMER_IA_HPET_BUS_INSTANCE_0 {
    NAME            Intel HPET bus driver instance 0
    SYNOPSIS        This component defines instance 0 of the Intel High Precision Event Timer (HPET) bus driver.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_RTN        usrIaHpetBusInit                                           \
                    (                                                          \
                    (int)           DRV_TIMER_IA_HPET_BUS_INSTANCE_0_INDEX,    \
                    (unsigned long) DRV_TIMER_IA_HPET_BUS_INSTANCE_0_BASE_ADRS,\
                    (unsigned long) DRV_TIMER_IA_HPET_BUS_INSTANCE_0_SIZE      \
                    );
    CFG_PARAMS      DRV_TIMER_IA_HPET_BUS_INSTANCE_0_INDEX \
                    DRV_TIMER_IA_HPET_BUS_INSTANCE_0_BASE_ADRS \
                    DRV_TIMER_IA_HPET_BUS_INSTANCE_0_SIZE
}

Parameter DRV_TIMER_IA_HPET_BUS_INSTANCE_0_INDEX {
    NAME            Instance index
    SYNOPSIS        This parameter specifies the DRV_TIMER_IA_HPET_BUS instance index. Set to 0 or (-1) to deactivate.
    TYPE            int
    DEFAULT         0
}

Parameter DRV_TIMER_IA_HPET_BUS_INSTANCE_0_BASE_ADRS {
    NAME            Base address
    SYNOPSIS        This parameter specifies the High Precision Event Timer (HPET) base address range start.
    TYPE            unsigned long
    DEFAULT         0xfed00000
}

Parameter DRV_TIMER_IA_HPET_BUS_INSTANCE_0_SIZE {
    NAME            Address range size
    SYNOPSIS        This parameter specifies the  High Precision Event Timer (HPET) address range size.
    TYPE            unsigned long
    DEFAULT         0x4000
}

/*******************************************************************************
*
* DRV_TIMER_IA_HPET device table entry
*
* Supports up to 3 comparators by default.
* If additional comparator support is needed, add
* DRV_TIMER_IA_HPET_INSTANCE_<n> entries as needed to this file and
* adjust DRV_TIMER_IA_HPET_BUS_INSTANCE_0_MAX_COUNT value.
*/

Component DRV_TIMER_IA_HPET {
    REQUIRES     += DRV_TIMER_IA_HPET_INSTANCE_0 \
                    DRV_TIMER_IA_HPET_INSTANCE_1 \
                    DRV_TIMER_IA_HPET_INSTANCE_2
    CFG_PARAMS   += DRV_TIMER_IA_HPET_BUS_INSTANCE_0_MAX_COUNT
}

Parameter DRV_TIMER_IA_HPET_BUS_INSTANCE_0_MAX_COUNT {
    NAME            Maximum timer instances allowed on HPET bus instance 0
    SYNOPSIS        This parameter specifies the maximum number of timer driver instances allowed on High Precision Event Timer (HPET) bus \
                    instance 0.  Each instance is of the form \
                    DRV_TIMER_IA_HPET_INSTANCE_<n> where <n> is the \
                    instance number starting with 0.
    TYPE            int
    DEFAULT         3
}

/*** DRV_TIMER_IA_HPET instance 0 - timer 0 ***/

Component DRV_TIMER_IA_HPET_INSTANCE_0 {
    NAME            Intel HPET driver instance 0
    SYNOPSIS        This component defines instance 0 of the Intel High Precision Event Timer (HPET) driver.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_AFTER      DRV_TIMER_IA_HPET_BUS_INSTANCE_0 /* depends on HPET bus 0 */
    INIT_RTN        usrIaHpetTimerInit                                       \
                        (                                                    \
                        (int)    DRV_TIMER_IA_HPET_INSTANCE_0_INDEX,         \
                        (int)    DRV_TIMER_IA_HPET_BUS_INSTANCE_0_INDEX,     \
                        (UINT32) DRV_TIMER_IA_HPET_BUS_INSTANCE_0_MAX_COUNT, \
                        (UINT32) DRV_TIMER_IA_HPET_INSTANCE_0_TIMER_NUM,     \
                        (UINT8)  DRV_TIMER_IA_HPET_INSTANCE_0_MODE,          \
                        (UINT32) DRV_TIMER_IA_HPET_INSTANCE_0_MIN_FREQ,      \
                        (UINT32) DRV_TIMER_IA_HPET_INSTANCE_0_MAX_FREQ,      \
                        (BOOL) DRV_TIMER_IA_HPET_INSTANCE_0_MSI_ENABLED,     \
                        (UINT8) DRV_TIMER_IA_HPET_INSTANCE_0_IOAPIC_PIN,     \
                        (int) DRV_TIMER_IA_HPET_INSTANCE_0_INT_VEC,          \
                        (BOOL) DRV_TIMER_IA_HPET_INSTANCE_0_PERIODIC_SYNC,   \
                        (BOOL) DRV_TIMER_IA_HPET_INSTANCE_0_NDHL_SHARED_INTR \
                        );
    CFG_PARAMS      DRV_TIMER_IA_HPET_INSTANCE_0_INDEX     \
                    DRV_TIMER_IA_HPET_INSTANCE_0_TIMER_NUM \
                    DRV_TIMER_IA_HPET_INSTANCE_0_MODE      \
                    DRV_TIMER_IA_HPET_INSTANCE_0_MIN_FREQ  \
                    DRV_TIMER_IA_HPET_INSTANCE_0_MAX_FREQ  \
                    DRV_TIMER_IA_HPET_INSTANCE_0_MSI_ENABLED \
                    DRV_TIMER_IA_HPET_INSTANCE_0_IOAPIC_PIN \
                    DRV_TIMER_IA_HPET_INSTANCE_0_INT_VEC \
                    DRV_TIMER_IA_HPET_INSTANCE_0_PERIODIC_SYNC \
                    DRV_TIMER_IA_HPET_INSTANCE_0_NDHL_SHARED_INTR
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_0_INDEX {
    NAME            Instance index
    SYNOPSIS        This parameter specifies the DRV_TIMER_IA_HPET instance index. Set to 0 or (-1) to deactivate.
    TYPE            int
    DEFAULT         0
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_0_TIMER_NUM {
    NAME            Timer number
    SYNOPSIS        This parameter specifies the HPET timer/comparator number.
    TYPE            UINT32
    DEFAULT         0
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_0_MODE {
    NAME            Configuration mode
    SYNOPSIS        This parameter specifies the High Precision Event Timer (HPET) configuration mode: \
                    IA_HPET_TIMER_MODE_PERIODIC - periodic timer, \
                    IA_HPET_TIMER_MODE_ONESHOT32 - 32 bit one shot timer, \
                    IA_HPET_TIMER_MODE_ONESHOT64 - 64 bit one shot timer
    TYPE            UINT8
    DEFAULT         IA_HPET_TIMER_MODE_PERIODIC
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_0_MIN_FREQ {
    NAME            Minimum frequency
    SYNOPSIS        This parameter specifies the  High Precision Event Timer (HPET) minimum frequency.
    TYPE            UINT32
    DEFAULT         (SYS_CLK_RATE_MIN)
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_0_MAX_FREQ {
    NAME            Maximum frequency
    SYNOPSIS        This parameter specifies the  High Precision Event Timer (HPET) maximum frequency.
    TYPE            UINT32
    DEFAULT         5001
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_0_MSI_ENABLED {
    NAME            MSI interrupt enable
    SYNOPSIS        When set TRUE MSI interrupts are enabled and values used \
                    in DRV_TIMER_IA_HPET_INSTANCE_n_IOAPIC_PIN and \
                    DRV_TIMER_IA_HPET_INSTANCE_n_INT_VEC are not used.
    TYPE            BOOL
    DEFAULT         TRUE
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_0_IOAPIC_PIN {
    NAME            IOAPIC pin
    SYNOPSIS        This parameter specifies the IOAPIC pin that the High Precision Event Timer (HPET) interrupt will \
                    be delivered to. Must be set to a value that the hardware \
                    supports.
    TYPE            UINT8
    DEFAULT         0x14
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_0_INT_VEC {
    NAME            Interrupt vector
    SYNOPSIS        This parameter specifies the High Precision Event Timer (HPET) interrupt vector. \
                    If set to -1 the default vector set by the BIOS \
                    in the IOAPIC redirection table will be used.
    TYPE            int
    DEFAULT         -1
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_0_PERIODIC_SYNC {
    NAME            Periodic sync
    SYNOPSIS        This parameter specifies the High Precision Event Timer (HPET) Periodic Sync \
                    Setup comparator match value using timer enable timestamp \
                    as base. Next interrupt will occur at timer enable plus \
                    timeoutTicks.
    TYPE            BOOL
    DEFAULT         FALSE
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_0_NDHL_SHARED_INTR {
    NAME            NDHL shared interrupt mode
    SYNOPSIS        This parameter specifies the High Precision Event Timer (HPET) Non-Deterministic High Latency Shared Interrupt Mode. \
                    Selects the use of the I/O Apic to deliver interrupts when \
                    MSI delivery is not supported and the interrupt line and \
                    vector must be shared with another device. This mode is \
                    generally least desirable due to the very real potential \
                    for Clock Interrupt delivery to be delayed due to \
                    interrupt processing by another device driver. \
                    DRV_TIMER_IA_HPET_INSTANCE_0_MSI_ENABLED must be set to \
                    FALSE for this parameter to become active; otherwise, this \
                    parameter is always set to FALSE internally.
    TYPE            BOOL
    DEFAULT         FALSE
}

/*** DRV_TIMER_IA_HPET instance 1 - timer 1 ***/

/*
 * deactivated by default by setting
 * DRV_TIMER_IA_HPET_INSTANCE_1_INDEX to (-1) below
 */

Component DRV_TIMER_IA_HPET_INSTANCE_1 {
    NAME            DRV_TIMER_IA_HPET driver instance 1
    SYNOPSIS        This component defines instance 1 of the Intel High Precision Event Timer (HPET) driver.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_AFTER      DRV_TIMER_IA_HPET_BUS_INSTANCE_0 /* depends on HPET bus 0 */
    INIT_RTN        usrIaHpetTimerInit                                       \
                        (                                                    \
                        (int)    DRV_TIMER_IA_HPET_INSTANCE_1_INDEX,         \
                        (int)    DRV_TIMER_IA_HPET_BUS_INSTANCE_0_INDEX,     \
                        (UINT32) DRV_TIMER_IA_HPET_BUS_INSTANCE_0_MAX_COUNT, \
                        (UINT32) DRV_TIMER_IA_HPET_INSTANCE_1_TIMER_NUM,     \
                        (UINT8)  DRV_TIMER_IA_HPET_INSTANCE_1_MODE,          \
                        (UINT32) DRV_TIMER_IA_HPET_INSTANCE_1_MIN_FREQ,      \
                        (UINT32) DRV_TIMER_IA_HPET_INSTANCE_1_MAX_FREQ,      \
                        (BOOL) DRV_TIMER_IA_HPET_INSTANCE_1_MSI_ENABLED,     \
                        (UINT8) DRV_TIMER_IA_HPET_INSTANCE_1_IOAPIC_PIN,     \
                        (int) DRV_TIMER_IA_HPET_INSTANCE_1_INT_VEC,          \
                        (BOOL) DRV_TIMER_IA_HPET_INSTANCE_1_PERIODIC_SYNC,   \
                        (BOOL) DRV_TIMER_IA_HPET_INSTANCE_1_NDHL_SHARED_INTR \
                        );
    CFG_PARAMS      DRV_TIMER_IA_HPET_INSTANCE_1_INDEX     \
                    DRV_TIMER_IA_HPET_INSTANCE_1_TIMER_NUM \
                    DRV_TIMER_IA_HPET_INSTANCE_1_MODE      \
                    DRV_TIMER_IA_HPET_INSTANCE_1_MIN_FREQ  \
                    DRV_TIMER_IA_HPET_INSTANCE_1_MAX_FREQ  \
                    DRV_TIMER_IA_HPET_INSTANCE_1_MSI_ENABLED \
                    DRV_TIMER_IA_HPET_INSTANCE_1_IOAPIC_PIN \
                    DRV_TIMER_IA_HPET_INSTANCE_1_INT_VEC \
                    DRV_TIMER_IA_HPET_INSTANCE_1_PERIODIC_SYNC \
                    DRV_TIMER_IA_HPET_INSTANCE_1_NDHL_SHARED_INTR
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_1_INDEX {
    NAME            Instance index
    SYNOPSIS        This parameter specifies the DRV_TIMER_IA_HPET instance index. Set to 1 or (-1) to deactivate.
    TYPE            int
    DEFAULT         (1)
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_1_TIMER_NUM {
    NAME            Timer number
    SYNOPSIS        This parameter specifies the HPET timer/comparator number.
    TYPE            UINT32
    DEFAULT         1
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_1_MODE {
    NAME            Instance mode
    SYNOPSIS        This parameter specifies the High Precision Event Timer (HPET) configuration mode: \
                    IA_HPET_TIMER_MODE_PERIODIC - periodic timer, \
                    IA_HPET_TIMER_MODE_ONESHOT32 - 32 bit one shot timer, \
                    IA_HPET_TIMER_MODE_ONESHOT64 - 64 bit one shot timer
    TYPE            UINT8
    DEFAULT         IA_HPET_TIMER_MODE_PERIODIC
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_1_MIN_FREQ {
    NAME            Minimum frequency
    SYNOPSIS        This parameter specifies the High Precision Event Timer (HPET) minimum frequency.
    TYPE            UINT32
    DEFAULT         (SYS_CLK_RATE_MIN)
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_1_MAX_FREQ {
    NAME            Maximum frequency
    SYNOPSIS        This parameter specifies the High Precision Event Timer (HPET) maximum frequency.
    TYPE            UINT32
    DEFAULT         5001
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_1_MSI_ENABLED {
    NAME            MSI interrupt enable
    SYNOPSIS        When set TRUE MSI interrupts are enabled and values used \
                    in DRV_TIMER_IA_HPET_INSTANCE_n_IOAPIC_PIN and \
                    DRV_TIMER_IA_HPET_INSTANCE_n_INT_VEC are not used.
    TYPE            BOOL
    DEFAULT         TRUE
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_1_IOAPIC_PIN {
    NAME            DRV_TIMER_IA_HPET_INSTANCE_1_IOAPIC_PIN
    SYNOPSIS        This parameter specifies the specific IOAPIC pin that the HPET interrupt will \
                    be delivered to. Must be set to a value that the hardware \
                    supports.
    TYPE            UINT8
    DEFAULT         0x15
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_1_INT_VEC {
    NAME            Interrupt vector
    SYNOPSIS        This parameter specifies the High Precision Event Timer (HPET) interrupt vector. \
                    If set to -1 the default vector set by the BIOS \
                    in the IOAPIC redirection table will be used.
    TYPE            int
    DEFAULT         -1
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_1_PERIODIC_SYNC {
    NAME            Periodic sync
    SYNOPSIS        This parameter specifies the High Precision Event Timer (HPET) Periodic Sync \
                    Setup comparator match value using timer enable timestamp \
                    as base. Next interrupt will occur at timer enable plus \
                    timeoutTicks.
    TYPE            BOOL
    DEFAULT         FALSE
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_1_NDHL_SHARED_INTR {
    NAME            NDHL shared interrupt mode
    SYNOPSIS        This parameter specifies the HPET Non-Deterministic High Latency Shared Interrupt Mode. \
                    Selects the use of the I/O Apic to deliver interrupts when \
                    MSI delivery is not supported and the interrupt line and \
                    vector must be shared with another device. This mode is \
                    generally least desirable due to the very real potential \
                    for Clock Interrupt delivery to be delayed due to \
                    interrupt processing by another device driver. \
                    DRV_TIMER_IA_HPET_INSTANCE_1_MSI_ENABLED must be set to \
                    FALSE for this parameter to become active; otherwise, this \
                    parameter is always set to FALSE internally.
    TYPE            BOOL
    DEFAULT         FALSE
}

/*** DRV_TIMER_IA_HPET instance 2 - timer 2 ***/

/*
 * deactivated by default by setting
 * DRV_TIMER_IA_HPET_INSTANCE_2_INDEX to (-1) below
 */

Component DRV_TIMER_IA_HPET_INSTANCE_2 {
    NAME            DRV_TIMER_IA_HPET driver instance 2
    SYNOPSIS        This component defines instance 2 of the Intel High Precision Event Timer (HPET) driver.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_AFTER      DRV_TIMER_IA_HPET_BUS_INSTANCE_0 /* depends on HPET bus 0 */
    INIT_RTN        usrIaHpetTimerInit                                       \
                        (                                                    \
                        (int)    DRV_TIMER_IA_HPET_INSTANCE_2_INDEX,         \
                        (int)    DRV_TIMER_IA_HPET_BUS_INSTANCE_0_INDEX,     \
                        (UINT32) DRV_TIMER_IA_HPET_BUS_INSTANCE_0_MAX_COUNT, \
                        (UINT32) DRV_TIMER_IA_HPET_INSTANCE_2_TIMER_NUM,     \
                        (UINT8)  DRV_TIMER_IA_HPET_INSTANCE_2_MODE,          \
                        (UINT32) DRV_TIMER_IA_HPET_INSTANCE_2_MIN_FREQ,      \
                        (UINT32) DRV_TIMER_IA_HPET_INSTANCE_2_MAX_FREQ,      \
                        (BOOL) DRV_TIMER_IA_HPET_INSTANCE_2_MSI_ENABLED,     \
                        (UINT8) DRV_TIMER_IA_HPET_INSTANCE_2_IOAPIC_PIN,     \
                        (int) DRV_TIMER_IA_HPET_INSTANCE_2_INT_VEC,          \
                        (BOOL) DRV_TIMER_IA_HPET_INSTANCE_2_PERIODIC_SYNC,   \
                        (BOOL) DRV_TIMER_IA_HPET_INSTANCE_2_NDHL_SHARED_INTR \
                        );
    CFG_PARAMS      DRV_TIMER_IA_HPET_INSTANCE_2_INDEX     \
                    DRV_TIMER_IA_HPET_INSTANCE_2_TIMER_NUM \
                    DRV_TIMER_IA_HPET_INSTANCE_2_MODE      \
                    DRV_TIMER_IA_HPET_INSTANCE_2_MIN_FREQ  \
                    DRV_TIMER_IA_HPET_INSTANCE_2_MAX_FREQ  \
                    DRV_TIMER_IA_HPET_INSTANCE_2_MSI_ENABLED \
                    DRV_TIMER_IA_HPET_INSTANCE_2_IOAPIC_PIN \
                    DRV_TIMER_IA_HPET_INSTANCE_2_INT_VEC \
                    DRV_TIMER_IA_HPET_INSTANCE_2_PERIODIC_SYNC \
                    DRV_TIMER_IA_HPET_INSTANCE_2_NDHL_SHARED_INTR
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_2_INDEX {
    NAME            Instance index
    SYNOPSIS        This parameter specifies the Intel High Precision Event Timer (HPET) instance index. Set to 2 or (-1) to deactivate.
    TYPE            int
    DEFAULT         (-1)
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_2_TIMER_NUM {
    NAME            Timer number
    SYNOPSIS        This parameter specifies the HPET timer/comparator number.
    TYPE            UINT32
    DEFAULT         2
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_2_MODE {
    NAME            Instance mode
    SYNOPSIS        This parameter specifies the High Precision Event Timer (HPET) configuration mode: \
                    IA_HPET_TIMER_MODE_PERIODIC - periodic timer, \
                    IA_HPET_TIMER_MODE_ONESHOT32 - 32 bit one shot timer, \
                    IA_HPET_TIMER_MODE_ONESHOT64 - 64 bit one shot timer
    TYPE            UINT8
    DEFAULT         IA_HPET_TIMER_MODE_PERIODIC
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_2_MIN_FREQ {
    NAME            Minimum frequency
    SYNOPSIS        This parameter specifies the High Precision Event Timer (HPET) minimum frequency.
    TYPE            UINT32
    DEFAULT         (SYS_CLK_RATE_MIN)
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_2_MAX_FREQ {
    NAME            Maximum frequency
    SYNOPSIS        This parameter specifies the High Precision Event Timer (HPET) maximum frequency.
    TYPE            UINT32
    DEFAULT         5001
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_2_MSI_ENABLED {
    NAME            MSI interrupt enable
    SYNOPSIS        When set TRUE MSI interrupts are enabled and values used \
                    in DRV_TIMER_IA_HPET_INSTANCE_n_IOAPIC_PIN and \
                    DRV_TIMER_IA_HPET_INSTANCE_n_INT_VEC are not used.
    TYPE            BOOL
    DEFAULT         TRUE
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_2_IOAPIC_PIN {
    NAME            IOAPIC pin
    SYNOPSIS        This parameter specifies the specificI/O Advanced Programmable Interrupt Controller (APIC) pin that the HPET interrupt will \
                    be delivered to. Must be set to a value that the hardware \
                    supports.
    TYPE            UINT8
    DEFAULT         0x16
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_2_INT_VEC {
    NAME            Interrupt vector
    SYNOPSIS        This parameter specifies the HPET interrupt vector. \
                    If set to -1 the default vector set by the BIOS \
                    in the I/O Advanced Programmable Interrupt Controller (APIC) redirection table will be used.
    TYPE            int
    DEFAULT         -1
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_2_PERIODIC_SYNC {
    NAME            Periodic sync
    SYNOPSIS        This parameter specifies the HPET Periodic Sync \
                    Setup comparator match value using timer enable timestamp \
                    as base. Next interrupt will occur at timer enable plus \
                    timeoutTicks.
    TYPE            BOOL
    DEFAULT         FALSE
}

Parameter DRV_TIMER_IA_HPET_INSTANCE_2_NDHL_SHARED_INTR {
    NAME            NDHL shared interrupt mode
    SYNOPSIS        This parameter specifies the HPET Non-Deterministic High Latency Shared Interrupt Mode. \
                    Selects the use of the I/O Apic to deliver interrupts when \
                    MSI delivery is not supported and the interrupt line and \
                    vector must be shared with another device. This mode is \
                    generally least desirable due to the very real potential \
                    for Clock Interrupt delivery to be delayed due to \
                    interrupt processing by another device driver. \
                    DRV_TIMER_IA_HPET_INSTANCE_2_MSI_ENABLED must be set to \
                    FALSE for this parameter to become active; otherwise, this \
                    parameter is always set to FALSE internally.
    TYPE            BOOL
    DEFAULT         FALSE
}

/*******************************************************************************
*
* DRV_TIMER_LOAPIC
*
* Device table entry is automatically added when usrLoApicTimerInit is
* called (usrIadtInit is called within usrLoApicTimerInit).
*
*/

Component DRV_TIMER_LOAPIC {
    REQUIRES     += DRV_TIMER_LOAPIC_INSTANCE_0
    CFG_PARAMS   += DRV_TIMER_LOAPIC_MAX_COUNT
}

Parameter DRV_TIMER_LOAPIC_MAX_COUNT {
    NAME            Maximum instances allowed
    SYNOPSIS        This parameter specifies the maximum number of driver instances allowed. \
                    Each instance is of the form \
                    DRV_TIMER_LOAPIC_INSTANCE_<n> where <n> is the \
                    instance number starting with 0.
    TYPE            int
    DEFAULT         1
}

Component DRV_TIMER_LOAPIC_INSTANCE_0 {
    NAME            LOAPIC timer driver instance 0
    SYNOPSIS        This component defines instance 0 of the LOAPIC timer driver.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_RTN        usrLoApicTimerInit                                  \
                        (                                               \
                        (int)    DRV_TIMER_LOAPIC_INSTANCE_0_INDEX,     \
                        (void *) DRV_TIMER_LOAPIC_INSTANCE_0_BASE_ADDR, \
                        (UINT32) DRV_TIMER_LOAPIC_INSTANCE_0_INT_LEVEL, \
                        (UINT32) DRV_TIMER_LOAPIC_INSTANCE_0_MIN_FREQ,  \
                        (UINT32) DRV_TIMER_LOAPIC_INSTANCE_0_MAX_FREQ,  \
                        (UINT32) DRV_TIMER_LOAPIC_INSTANCE_0_CLK_FREQ   \
                        );
    CFG_PARAMS      DRV_TIMER_LOAPIC_INSTANCE_0_INDEX     \
                    DRV_TIMER_LOAPIC_INSTANCE_0_BASE_ADDR \
                    DRV_TIMER_LOAPIC_INSTANCE_0_INT_LEVEL \
                    DRV_TIMER_LOAPIC_INSTANCE_0_MIN_FREQ  \
                    DRV_TIMER_LOAPIC_INSTANCE_0_MAX_FREQ  \
                    DRV_TIMER_LOAPIC_INSTANCE_0_CLK_FREQ
}

Parameter DRV_TIMER_LOAPIC_INSTANCE_0_INDEX {
    NAME            Instance index
    TYPE            int
    DEFAULT         0
}

Parameter DRV_TIMER_LOAPIC_INSTANCE_0_BASE_ADDR {
    NAME            Base address
    TYPE            (void *)
    DEFAULT         0xfee00000
}

Parameter DRV_TIMER_LOAPIC_INSTANCE_0_INT_LEVEL {
    NAME            Interrupt level
    TYPE            UINT32
    DEFAULT         0xC4
}

Parameter DRV_TIMER_LOAPIC_INSTANCE_0_MIN_FREQ {
    NAME            Minimum clock frequency
    TYPE            UINT32
    DEFAULT         (SYS_CLK_RATE_MIN)
}

Parameter DRV_TIMER_LOAPIC_INSTANCE_0_MAX_FREQ {
    NAME            Maximum clock frequency
    TYPE            UINT32
    DEFAULT         (SYS_CLK_RATE_MAX)
}

Parameter DRV_TIMER_LOAPIC_INSTANCE_0_CLK_FREQ {
    NAME            Clock frequency
    TYPE            UINT32
    DEFAULT         0
}

/*******************************************************************************
*
* DRV_TIMER_IA_TIMESTAMP device table entry
*
* IA Timestamp Timer
*
* For UP, the TSC frequency is auto calibrated now. If users want to use
* static value they can change the define here.
*
* For SMP, system will use HPET as timestamp, the timestamp driver always
* auto detect HPET frequency, so just leave it as 0 will be ok.
*/

Component DRV_TIMER_IA_TIMESTAMP {
    REQUIRES     += DRV_TIMER_IA_TIMESTAMP_INSTANCE_0
    CFG_PARAMS   += DRV_TIMER_IA_TIMESTAMP_MAX_COUNT
}


Parameter DRV_TIMER_IA_TIMESTAMP_MAX_COUNT {
    NAME            Maximum instances allowed
    SYNOPSIS        This parameter specifies the maximum number of driver instances allowed. \
                    Each instance is of the form \
                    DRV_TIMER_IA_TIMESTAMP_INSTANCE_<n> where <n> is the \
                    instance number starting with 0.
    TYPE            int
    DEFAULT         1
}

Component DRV_TIMER_IA_TIMESTAMP_INSTANCE_0 {
    NAME            Intel timestamp timer driver instance 0
    SYNOPSIS        This component defines instance 0 of the Intel timestamp timer driver instance 0.
    PROTOTYPE       IMPORT UINT64 sysGetTSCCountPerSec (void);
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_RTN  usrIntelTimestampInit                                            \
              (                                                                \
              (int)             DRV_TIMER_IA_TIMESTAMP_INSTANCE_0_INDEX,       \
              (unsigned long *) DRV_TIMER_IA_TIMESTAMP_INSTANCE_0_CLK_CAL_FUNC,\
              (unsigned long)   DRV_TIMER_IA_TIMESTAMP_INSTANCE_0_CLK_FREQ     \
              );
    CFG_PARAMS      DRV_TIMER_IA_TIMESTAMP_INSTANCE_0_INDEX        \
                    DRV_TIMER_IA_TIMESTAMP_INSTANCE_0_CLK_CAL_FUNC \
                    DRV_TIMER_IA_TIMESTAMP_INSTANCE_0_CLK_FREQ
}

Parameter DRV_TIMER_IA_TIMESTAMP_INSTANCE_0_INDEX {
    NAME            Instance index
    SYNOPSIS        This parameter specifies the DRV_TIMER_IA_TIMESTAMP instance index. Set to 0 or (-1) to deactivate.
    TYPE            int
    DEFAULT         0
}

Parameter DRV_TIMER_IA_TIMESTAMP_INSTANCE_0_CLK_CAL_FUNC {
    NAME            User defined function to show clock frequency
    SYNOPSIS        This parameter specifies the user defined function used to retrieve the current frequency of the timestamp device.
    TYPE            unsigned long *
    DEFAULT         &sysGetTSCCountPerSec
}

Parameter DRV_TIMER_IA_TIMESTAMP_INSTANCE_0_CLK_FREQ {
    NAME            Clock frequency
    SYNOPSIS        This parameter specifies the Intel timestamp clock frequency.
    TYPE            unsigned long
    DEFAULT         0
}

/*******************************************************************************
*
* DRV_TIMER_I8253 device table entry
*
* Intel 8253 timer
*
* By default, only counter 0 is configured.
*
* It is strongly recommended that users consult the documentation for
* the target hardware platform prior to enabing counters 1 & 2.  In
* particular, users should know how the outputs of the timer channels
* are connected on the target hardware platform (see target.ref).
*/

Component DRV_TIMER_I8253 {
    REQUIRES     += DRV_TIMER_I8253_INSTANCE_0 \
                    DRV_TIMER_I8253_INSTANCE_1 \
                    DRV_TIMER_I8253_INSTANCE_2
    CFG_PARAMS   += DRV_TIMER_I8253_MAX_COUNT
}

Parameter DRV_TIMER_I8253_MAX_COUNT {
    NAME            Maximum instances allowed
    SYNOPSIS        This parameter specifies the maximum number of driver instances allowed. \
                    Each instance is of the form \
                    DRV_TIMER_I8253_INSTANCE_<n> where <n> is the \
                    instance number starting with 0. \
                    DRV_TIMER_I8253_MAX_COUNT can not be greater than \
                    VXB_I8253_MAX_TIMERS define in vxbI8253Timer.h.
    TYPE            int
    DEFAULT         3
}

/*** DRV_TIMER_I8253 instance 0 - timer 0 ***/

Component DRV_TIMER_I8253_INSTANCE_0 {
    NAME            I8253 timer driver instance 0
    SYNOPSIS        This component defines instance 0 of the I8253 timer driver.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_RTN        usrI8253TimerInit                                          \
                      (                                                        \
                      (int)           DRV_TIMER_I8253_INSTANCE_0_INDEX,        \
                      (unsigned long) DRV_TIMER_I8253_INSTANCE_0_BASE_ADRS,    \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_0_ADRS_INTERVAL,\
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_0_TIMER_NUM,    \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_0_TIMER_MODE,   \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_0_IRQ_NUM,      \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_0_IRQ_LVL,      \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_0_MIN_FREQ,     \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_0_MAX_FREQ,     \
                      (unsigned long) DRV_TIMER_I8253_INSTANCE_0_CLK_FREQ      \
                      );
    CFG_PARAMS      DRV_TIMER_I8253_INSTANCE_0_INDEX         \
                    DRV_TIMER_I8253_INSTANCE_0_BASE_ADRS     \
                    DRV_TIMER_I8253_INSTANCE_0_ADRS_INTERVAL \
                    DRV_TIMER_I8253_INSTANCE_0_TIMER_NUM     \
                    DRV_TIMER_I8253_INSTANCE_0_TIMER_MODE    \
                    DRV_TIMER_I8253_INSTANCE_0_IRQ_NUM       \
                    DRV_TIMER_I8253_INSTANCE_0_IRQ_LVL       \
                    DRV_TIMER_I8253_INSTANCE_0_MIN_FREQ      \
                    DRV_TIMER_I8253_INSTANCE_0_MAX_FREQ      \
                    DRV_TIMER_I8253_INSTANCE_0_CLK_FREQ
}

Parameter DRV_TIMER_I8253_INSTANCE_0_INDEX {
    NAME            Instance index
    SYNOPSIS        This parameter specifies the DRV_TIMER_I8253 instance index. Set to 0 or (-1) to deactivate.
    TYPE            int
    DEFAULT         0
}

Parameter DRV_TIMER_I8253_INSTANCE_0_BASE_ADRS {
    NAME            Base address
    SYNOPSIS        This parameter specifies the register base address for Intel 8253.
    TYPE            unsigned long
    DEFAULT         (PIT_BASE_ADR)
}

Parameter DRV_TIMER_I8253_INSTANCE_0_ADRS_INTERVAL {
    NAME            Register address interval
    SYNOPSIS        This parameter specifies the Intel 8253 register address interval.
    TYPE            uint
    DEFAULT         (PIT_REG_ADDR_INTERVAL)
}

Parameter DRV_TIMER_I8253_INSTANCE_0_TIMER_NUM {
    NAME            Timer number
    SYNOPSIS        This parameter specifies the Intel 8253 timer number.
    TYPE            uint
    DEFAULT         0
}

Parameter DRV_TIMER_I8253_INSTANCE_0_TIMER_MODE {
    NAME            Timer mode
    SYNOPSIS        This parameter specifies the Intel 8253 timer mode.
    TYPE            uint
    DEFAULT         0x04
}

Parameter DRV_TIMER_I8253_INSTANCE_0_IRQ_NUM {
    NAME            Interrupt number
    SYNOPSIS        This parameter specifies the interrupt number for Intel 8253 counter 0.
    TYPE            uint
    DEFAULT         0xac
}

Parameter DRV_TIMER_I8253_INSTANCE_0_IRQ_LVL {
    NAME            Interrupt level
    SYNOPSIS        This parameter specifies the interrupt level for Intel 8253 counter 0.
    TYPE            uint
    DEFAULT         (PIT0_INT_LVL)
}


Parameter DRV_TIMER_I8253_INSTANCE_0_MIN_FREQ  {
    NAME            Intel 8253 minimum clock rate
    SYNOPSIS        This parameter specifies the minimum clock rate (in ticks/sec) which can be supported.
    TYPE            uint
    DEFAULT         (SYS_CLK_RATE_MIN)
}

Parameter DRV_TIMER_I8253_INSTANCE_0_MAX_FREQ  {
    NAME            Intel 8253 maximum clock rate
    SYNOPSIS        This parameter specifies the maximum clock rate (in ticks/sec) which can be supported.
    TYPE            uint
    DEFAULT         (SYS_CLK_RATE_MAX)
}

Parameter DRV_TIMER_I8253_INSTANCE_0_CLK_FREQ {
    NAME            Intel 8253 clock frequency
    SYNOPSIS        This parameter specifies the Intel 8253 clock frequency.
    TYPE            unsigned long
    DEFAULT         (PIT_CLOCK)
}

/*** DRV_TIMER_I8253 instance 1 - timer 1 ***/

/*
 * deactivated by default by setting
 * DRV_TIMER_I8253_INSTANCE_1_INDEX to (-1) below
 */

Component DRV_TIMER_I8253_INSTANCE_1 {
    NAME            I8253 timer driver instance 1
    SYNOPSIS        This component defines instance 1 of the I8253 timer driver.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_RTN        usrI8253TimerInit                                          \
                      (                                                        \
                      (int)           DRV_TIMER_I8253_INSTANCE_1_INDEX,        \
                      (unsigned long) DRV_TIMER_I8253_INSTANCE_1_BASE_ADRS,    \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_1_ADRS_INTERVAL,\
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_1_TIMER_NUM,    \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_1_TIMER_MODE,   \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_1_IRQ_NUM,      \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_1_IRQ_LVL,      \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_1_MIN_FREQ,     \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_1_MAX_FREQ,     \
                      (unsigned long) DRV_TIMER_I8253_INSTANCE_1_CLK_FREQ      \
                      );
    CFG_PARAMS      DRV_TIMER_I8253_INSTANCE_1_INDEX         \
                    DRV_TIMER_I8253_INSTANCE_1_BASE_ADRS     \
                    DRV_TIMER_I8253_INSTANCE_1_ADRS_INTERVAL \
                    DRV_TIMER_I8253_INSTANCE_1_TIMER_NUM     \
                    DRV_TIMER_I8253_INSTANCE_1_TIMER_MODE    \
                    DRV_TIMER_I8253_INSTANCE_1_IRQ_NUM       \
                    DRV_TIMER_I8253_INSTANCE_1_IRQ_LVL       \
                    DRV_TIMER_I8253_INSTANCE_1_MIN_FREQ      \
                    DRV_TIMER_I8253_INSTANCE_1_MAX_FREQ      \
                    DRV_TIMER_I8253_INSTANCE_1_CLK_FREQ
}

Parameter DRV_TIMER_I8253_INSTANCE_1_INDEX {
    NAME            Instance index
    SYNOPSIS        This parameter specifies the DRV_TIMER_I8253 instance index. Set to 1 or (-1) to deactivate.
    TYPE            int
    DEFAULT         (-1)
}

Parameter DRV_TIMER_I8253_INSTANCE_1_BASE_ADRS {
    NAME            Base address
    SYNOPSIS        This parameter specifies the register base address for Intel 8253.
    TYPE            unsigned long
    DEFAULT         (PIT_BASE_ADR)
}

Parameter DRV_TIMER_I8253_INSTANCE_1_ADRS_INTERVAL {
    NAME            Register address interval
    SYNOPSIS        This parameter specifies the Intel 8253 register address interval.
    TYPE            uint
    DEFAULT         (PIT_REG_ADDR_INTERVAL)
}

Parameter DRV_TIMER_I8253_INSTANCE_1_TIMER_NUM {
    NAME            Timer number
    SYNOPSIS        This parameter specifies the Intel 8253 timer number.
    TYPE            uint
    DEFAULT         1
}

Parameter DRV_TIMER_I8253_INSTANCE_1_TIMER_MODE {
    NAME            Timer mode
    SYNOPSIS        This parameter specifies the Intel 8253 timer mode.
    TYPE            uint
    DEFAULT         0x04
}

Parameter DRV_TIMER_I8253_INSTANCE_1_IRQ_NUM {
    NAME            Interrupt number
    SYNOPSIS        This parameter specifies the interrupt number for Intel 8253 counter 1.
    TYPE            uint
    DEFAULT         0
}

Parameter DRV_TIMER_I8253_INSTANCE_1_IRQ_LVL {
    NAME            Interrupt level
    SYNOPSIS        This parameter specifies the interrupt level for Intel 8253 counter 1.
    TYPE            uint
    DEFAULT         0
}

Parameter DRV_TIMER_I8253_INSTANCE_1_MIN_FREQ  {
    NAME            Intel 8253 minimum clock rate
    SYNOPSIS        This parameter specifies the minimum clock rate (in ticks/sec) which can be supported.
    TYPE            uint
    DEFAULT         (SYS_CLK_RATE_MIN)
}

Parameter DRV_TIMER_I8253_INSTANCE_1_MAX_FREQ  {
    NAME            Intel 8253 maximum clock rate
    SYNOPSIS        This parameter specifies the maximum clock rate (in ticks/sec) which can be supported.
    TYPE            uint
    DEFAULT         (SYS_CLK_RATE_MAX)
}

Parameter DRV_TIMER_I8253_INSTANCE_1_CLK_FREQ {
    NAME            Intel 8253 clock frequency
    SYNOPSIS        This parameter specifies the Intel 8253 clock frequency.
    TYPE            unsigned long
    DEFAULT         (PIT_CLOCK)
}

/*** DRV_TIMER_I8253 instance 2 - timer 2 ***/

/*
 * deactivated by default by setting
 * DRV_TIMER_I8253_INSTANCE_2_INDEX to (-1) below
 */

Component DRV_TIMER_I8253_INSTANCE_2 {
    NAME            I8253 timer driver instance 2
    SYNOPSIS        This component instance 2 of the I8253 timer driver.
    _CHILDREN       FOLDER_BSP_CONFIG
    _INIT_ORDER     usrIaDevTableDynamicInit
    INIT_RTN        usrI8253TimerInit                                          \
                      (                                                        \
                      (int)           DRV_TIMER_I8253_INSTANCE_2_INDEX,        \
                      (unsigned long) DRV_TIMER_I8253_INSTANCE_2_BASE_ADRS,    \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_2_ADRS_INTERVAL,\
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_2_TIMER_NUM,    \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_2_TIMER_MODE,   \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_2_IRQ_NUM,      \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_2_IRQ_LVL,      \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_2_MIN_FREQ,     \
                      (unsigned int)  DRV_TIMER_I8253_INSTANCE_2_MAX_FREQ,     \
                      (unsigned long) DRV_TIMER_I8253_INSTANCE_2_CLK_FREQ      \
                      );
    CFG_PARAMS      DRV_TIMER_I8253_INSTANCE_2_INDEX         \
                    DRV_TIMER_I8253_INSTANCE_2_BASE_ADRS     \
                    DRV_TIMER_I8253_INSTANCE_2_ADRS_INTERVAL \
                    DRV_TIMER_I8253_INSTANCE_2_TIMER_NUM     \
                    DRV_TIMER_I8253_INSTANCE_2_TIMER_MODE    \
                    DRV_TIMER_I8253_INSTANCE_2_IRQ_NUM       \
                    DRV_TIMER_I8253_INSTANCE_2_IRQ_LVL       \
                    DRV_TIMER_I8253_INSTANCE_2_MIN_FREQ      \
                    DRV_TIMER_I8253_INSTANCE_2_MAX_FREQ      \
                    DRV_TIMER_I8253_INSTANCE_2_CLK_FREQ
}

Parameter DRV_TIMER_I8253_INSTANCE_2_INDEX {
    NAME            Instance index
    SYNOPSIS        This parameter specifies the DRV_TIMER_I8253 instance index. Set to 2 or (-1) to deactivate.
    TYPE            int
    DEFAULT         (-1)
}

Parameter DRV_TIMER_I8253_INSTANCE_2_BASE_ADRS {
    NAME            Base address
    SYNOPSIS        This parameter specifies the register base address for Intel 8253.
    TYPE            unsigned long
    DEFAULT         (PIT_BASE_ADR)
}

Parameter DRV_TIMER_I8253_INSTANCE_2_ADRS_INTERVAL {
    NAME            Register address interval
    SYNOPSIS        This parameter specifies the Intel 8253 register address interval.
    TYPE            uint
    DEFAULT         (PIT_REG_ADDR_INTERVAL)
}

Parameter DRV_TIMER_I8253_INSTANCE_2_TIMER_NUM {
    NAME            Timer number
    SYNOPSIS        This parameter specifies the Intel 8253 timer number.
    TYPE            uint
    DEFAULT         2
}

Parameter DRV_TIMER_I8253_INSTANCE_2_TIMER_MODE {
    NAME            Timer mode
    SYNOPSIS        This parameter specifies the Intel 8253 timer mode.
    TYPE            uint
    DEFAULT         0x04
}

Parameter DRV_TIMER_I8253_INSTANCE_2_IRQ_NUM {
    NAME            Interrupt level
    SYNOPSIS        This parameter specifies the interrupt number for Intel 8253 counter 2.
    TYPE            uint
    DEFAULT         0
}

Parameter DRV_TIMER_I8253_INSTANCE_2_IRQ_LVL {
    NAME            Interrupt level
    SYNOPSIS        This parameter specifies the interrupt level for Intel 8253 counter 2.
    TYPE            uint
    DEFAULT         0
}

Parameter DRV_TIMER_I8253_INSTANCE_2_MIN_FREQ  {
    NAME            Intel 8253 minimum clock rate
    SYNOPSIS        This parameter specifies the minimum clock rate (in ticks/sec) which can be supported.
    TYPE            uint
    DEFAULT         (SYS_CLK_RATE_MIN)
}

Parameter DRV_TIMER_I8253_INSTANCE_2_MAX_FREQ  {
    NAME            Intel 8253 maximum clock rate
    SYNOPSIS        This parameter specifies the maximum clock rate (in ticks/sec) which can be supported.
    TYPE            uint
    DEFAULT         (SYS_CLK_RATE_MAX)
}

Parameter DRV_TIMER_I8253_INSTANCE_2_CLK_FREQ {
    NAME            Intel 8253 clock frequency
    SYNOPSIS        This parameter specifies the Intel 8253 clock frequency.
    TYPE            unsigned long
    DEFAULT         (PIT_CLOCK)
}

/*******************************************************************************
*
* DRV_IA_PCI
*
* Device table entry is automatically added when usrIaPciInit is
* called (usrIadtInit is called within usrIaPciInit).
*
*/

Component DRV_IA_PCI_BUS {
    REQUIRES     += DRV_IA_PCI_BUS_INSTANCE_0 \
                    IA_PCI_UTILS
    CFG_PARAMS   += DRV_IA_PCI_BUS_MAX_COUNT
}

Parameter DRV_IA_PCI_BUS_MAX_COUNT {
    NAME                 Intel PCI bus maximum instances allowed
    SYNOPSIS             This parameter specifies the maximum number of driver instances allowed. \
                         Each instance is of the form \
                         DRV_IA_PCI_BUS_INSTANCE_<n> where <n> is the \
                         instance number starting with 0.
    TYPE                 uint
    DEFAULT              1
}

Component DRV_IA_PCI_BUS_INSTANCE_0 {
    NAME                 Intel PCI bus driver instance 0
    SYNOPSIS             This component defines Intel PCI bus driver instance 0.
    _CHILDREN            FOLDER_BSP_CONFIG
    _INIT_ORDER          usrIaDevTableDynamicInit
    INIT_RTN             (void) usrIaPciBusInit ( \
                         (int) DRV_IA_PCI_BUS_INSTANCE_0_INDEX, \
                         (char *) DRV_IA_PCI_BUS_INSTANCE_0_BUS_LIST \
                         );
    CFG_PARAMS           DRV_IA_PCI_BUS_INSTANCE_0_INDEX \
                         DRV_IA_PCI_BUS_INSTANCE_0_BUS_LIST
}

Parameter DRV_IA_PCI_BUS_INSTANCE_0_INDEX {
    NAME            Intel PCI bus instance index
    SYNOPSIS        This parameter specifies the DRV_IA_PCI_BUS instance index. Set to 0 or (-1) to deactivate.
    TYPE            int
    DEFAULT         (0)
}

Parameter DRV_IA_PCI_BUS_INSTANCE_0_BUS_LIST {
    NAME            Intel PCI bus start
    SYNOPSIS        This parameter specifies the comma separated list of decimal values which represent \
                    the starting value for PCI bus controllers found in the \
                    system. This is used to discover and enumerate PCI devices \
                    that are not bridge connected.
    TYPE            string
    DEFAULT         (INCLUDE_SHUMWAY)::("0,128") \
                    "0"
}

/*******************************************************************************
*
* DRV_IA_TPM
*
* Device table entry is automatically added when usrIaTpmInit() is
* called (Note, the usrIadtInit() routine used to add the TPM device to the 
* VxBus device table entry is called from within the usrIaTpmInit() routine).
*
* Only one TPM device is supported out-of-the-box using the standard TPM 
* hardware resources as defined by the TCG Client Specific TPM Interface
* Specification (TIS version 1.3).
*
* Sometimes the BIOS also needs adjustment to reset, enable and activate 
* the TPM hardware prior to first use.
*/

Component DRV_IA_TPM {
    REQUIRES             += DRV_IA_TPM_DEVICE_INIT \
                            DRV_IA_TPM_INSTANCE_0
    CFG_PARAMS           += DRV_IA_TPM_MAX_COUNT
}


Parameter DRV_IA_TPM_MAX_COUNT {
    NAME                 Maximum number of instances allowed
    SYNOPSIS             This parameter specifies the maximum number of driver instances allowed. \
                         Each instance is of the form \
                         DRV_IA_TPM_INSTANCE_<n> where <n> is the \
                         instance number starting with 0.
    TYPE                 uint
    DEFAULT              1
}


Component DRV_IA_TPM_DEVICE_INIT {
    NAME                 Intel TPM driver initialization
    SYNOPSIS             This component provides functions to initialize the Intel Trusted Platform Module (TPM) driver.
    _CHILDREN            FOLDER_BSP_CONFIG
    _INIT_ORDER          usrRoot 
    INIT_AFTER           usrClkInit 
    INIT_BEFORE          usrTrousersInit
    PROTOTYPE            void usrIaTpmDevInit (void);
    INIT_RTN             usrIaTpmDevInit ();
}


/*** DRV_IA_TPM instance 0 ***/

Component DRV_IA_TPM_INSTANCE_0 {
    NAME                 Intel TPM driver instance 0
    SYNOPSIS             This component defines instance 0 of the Intel Trusted Platform Module (TPM) driver.
    _CHILDREN            FOLDER_BSP_CONFIG
    _INIT_ORDER          usrIaDevTableDynamicInit
    INIT_RTN             (void) usrIaTpmInit                            \
                            (                                           \
                            (int)    DRV_IA_TPM_INSTANCE_0_INDEX,       \
                            (char *) DRV_IA_TPM_INSTANCE_0_DEV_NAME,    \
                            (void *) DRV_IA_TPM_INSTANCE_0_BASE_ADDR,   \
                            (UINT16) DRV_IA_TPM_INSTANCE_0_REG_INTERVAL,\
                            (UINT32) DRV_IA_TPM_INSTANCE_0_INT_NUM,     \
                            (UINT32) DRV_IA_TPM_INSTANCE_0_INT_LEVEL    \
                            );
    CFG_PARAMS           DRV_IA_TPM_INSTANCE_0_INDEX        \
                         DRV_IA_TPM_INSTANCE_0_DEV_NAME     \
                         DRV_IA_TPM_INSTANCE_0_BASE_ADDR    \
                         DRV_IA_TPM_INSTANCE_0_REG_INTERVAL \
                         DRV_IA_TPM_INSTANCE_0_INT_NUM      \
                         DRV_IA_TPM_INSTANCE_0_INT_LEVEL 
}


Parameter DRV_IA_TPM_INSTANCE_0_INDEX {
    NAME                 TPM instance index
    SYNOPSIS             This parameter specifies the Intel Trusted Platform Module (TPM) instance index. Set to 0 or (-1) to deactivate.
    TYPE                 int
    DEFAULT              0
}


Parameter DRV_IA_TPM_INSTANCE_0_DEV_NAME {
    NAME                 Device name
    SYNOPSIS             This parameter specifies the I/O system name for first TPM device.
    TYPE                 string
    DEFAULT              "/tpm0"
}


Parameter DRV_IA_TPM_INSTANCE_0_BASE_ADDR {
    NAME                 Base address
    SYNOPSIS             This parameter specifies the memory mapping register base address for first TPM \
                         device at locality 0.
    TYPE                 (void *)
    DEFAULT              (0xfed40000)
}


Parameter DRV_IA_TPM_INSTANCE_0_REG_INTERVAL {
    NAME                 Register interval
    SYNOPSIS             This parameter specifies the memory mapping register address interval for first \
                         TPM device at locality 0.
    TYPE                 (UINT16)
    DEFAULT              (0x5000)
}

Parameter DRV_IA_TPM_INSTANCE_0_INT_NUM {
    NAME                 Interupt IRQ number
    SYNOPSIS             This parameter specifies the interrupt IRQ number for the first Trusted Platform Module (TPM) device.
    TYPE                 uint
    DEFAULT              (0x9c)
}


Parameter DRV_IA_TPM_INSTANCE_0_INT_LEVEL {
    NAME                 Interrupt level
    SYNOPSIS             This parameter specifies the interrupt level (APIC number) for the first Trusted Platform Module (TPM) device.
    TYPE                 uint
    DEFAULT              (0x4)
}
