/* 20bsp.cdf - BSP component description file */

/*
 * Copyright (c) 2014-2017 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify, or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

/*
modification history
--------------------
19jun17,mjn  text updates (F8862)
22nov16,j_l  Do not use ACPI static memory pool when using overlapped memory
             (F6461)
22oct15,ten  V7PRO-2489 - Using serial over USB on Bay Trail (Bayley Bay CRB).
14sep15,jlv  add fast warm boot support. (F3387)
17aug15,ten  V7PRO-2066 - Added more information to SYS_MODEL, remove "no boot op".
14scm15,scm  V7PRO-2016 - replace INCLUDE_DRV_STORAGE_AHCI with
             PCI_DRV_STORAGE_AHCI
08apr15,jmz  V7PRO-1994 arch.cdf sets N_VIRTUAL_CONSOLES
03apr15,rbc  V7PRO-1943 - Fix build errors adding INCLUDE_CPU_PWR_ARCH
23feb15,rbc  US54437 - Add new IA CPU types
05jan15,ten  US43657 - Unify memory autosize components in Intel BSPs
17dec14,j_l  V7PRO-1506 - Add INCLUDE_SYMMETRIC_IO_MODE for i8259Intr.c
15dec14,j_l  V7PRO-1491 - HPET driver not working in shared IRQs
             V7PRO-1502 - image build failed when add component
                          INCLUDE_AUX_CLK
24nov14,ten  V7PRO-1436 - Add INCLUDE_NET_DRV to default components
13aug14,ten  US44071 - Merge multiboot changes from itl_64 BSP:
                       US38341, V7PRO-891
08aug14,ten  US41103 - Add generic profile to itl_64 gen2
08aug14,jmz  V7PRO-1144 - Add dependency on INCLUDE_EARLY_PRINT to
             INCLUDE_DEBUG_KPRINTF
23Jul14,jmz  V7PRO-1047 - INCLUDE_ACPI requires IA_PCI_UTILS
12may14,jmz  US35979 - Add support for DRV_IA_PCI_BUS
09jun14,jmz  US35979 - Update SIO driver name to DRV_SIO_IA_NS16550
02jun14,jmz  US24640 - Remove unneeded HPET configuration options
20may14,jjk  US35956 - New API to query CPU/SOC versions.
09may14,wap  Add console driver selection (US35788, US35789, US35790, US35791)
08may14,scm  US32988/TA40489 - Test/Debug/Validate IA GEN2 drivers, IPIs...
06may14,jmz  US35979 - Update default SIO parameters
01may14,j_l  US37558 - Create IA configuration configlettes for
                       VxBus GEN2 drivers
29apr14,j_l  US37556 - Convert *MPT* names to *IADT*
10apr14,wap  Fix VXW7-2974, don't require INCLUDE_NVRAM_NULL component
             when NVRAM layer isn't in the VSB
27mar14,j_l  US30676 - remove config.h
21mar14,scm  TA42436 Port PC_CONSOLE to vxbus gen2 for IA
18mar14,j_l  US32795 - restore sysNvRamGet/Set functionality
12mar14,j_l  moved Bsp definition to bsp.cdf
19feb14,jjk  VXW7-1970 - Default size too small can cause boot failure
05feb14,j_l  add SYSTEM_RAM_SIZE, N_UART_CHANNELS, LOCAL_MEM_PHYS_ADRS_MASK,
             INCLUDE_SYS_HW_INIT_0, SYS_HW_INIT_0
04feb14,scm  US20759 - remove unsupported interrupt modes...
03feb14,j_l  US23032 - IA BSP with VxBus GEN2 core library
             US18115 - IA implementation of board system
31jan14,j_l  initial creation based on itl_64 version 17jan14
*/


Profile BSP_DEFAULT {
    NAME        Default BSP profile
    SYNOPSIS    Profile to include default BSP components and parameters.
    COMPONENTS  INCLUDE_MMU_BASIC \
                INCLUDE_CPU_LIGHT_PWR_MGR \
                INCLUDE_NET_DRV
    PROFILES    BSP_PARAMETER_VALUES \
                PROFILE_OS_DEFAULT \
                PROFILE_BSP_NETWORK_DEFAULT
}

Profile BSP_PARAMETER_VALUES {
    COMPONENTS  INCLUDE_EARLY_I_CACHE_ENABLE \
                INCLUDE_EARLY_D_CACHE_ENABLE
}

Profile PROFILE_BSP_NETWORK_DEFAULT {
    COMPONENTS  += INCLUDE_GEI825XX_VXB_END \
                   INCLUDE_GENERICPHY \
                   INCLUDE_MII_SYS
}

Component INCLUDE_BOARD_CONFIG {
    NAME        Board configuration component
    SYNOPSIS    This component provides fundamental board configuration. Included by a board bundle.
    REQUIRES    TARGET_NAME \
                PROCESSOR_CLASS \
                DEFAULT_BOOT_LINE \
                SYS_MODEL
    CFG_PARAMS  TARGET_NAME \
                PROCESSOR_CLASS \
                DEFAULT_BOOT_LINE \
                SYS_MODEL \
                APIC_TIMER_CLOCK_HZ \
                SYS_PCI_BUS_CTRL_NUM \
                SYS_FIRST_PCI_MIN_BUS \
                SYS_FIRST_PCI_MAX_BUS \
                SYSTEM_RAM_SIZE \
                N_UART_CHANNELS \
                LOCAL_MEM_PHYS_ADRS_MASK
}

#if (defined _WRS_CONFIG_SMP)
Component INCLUDE_BOARD_CONFIG {
    REQUIRES    += DRV_TIMER_IA_HPET
    CFG_PARAMS  += SYS_AP_TIMEOUT \
                   SYS_AP_LOOP_COUNT
}
#endif /* _WRS_CONFIG_SMP */

Selection SELECT_BOARD {
    NAME        Target selection
    SYNOPSIS    Selects target board for a build.
    COUNT       1-1
    CHILDREN    INCLUDE_INTEL_GENERIC \
                INCLUDE_WHITETIP_MOUNTAIN \
                INCLUDE_CAMELBACK_MOUNTAIN \
                INCLUDE_WALNUT_CANYON_2 \
                INCLUDE_WALNUT_CANYON \
                INCLUDE_FLATHEAD_CREEK \
                INCLUDE_LAVA_CANYON \
                INCLUDE_OSAGE \
                INCLUDE_GREENCITY \
                INCLUDE_HANLAN_CREEK_WESTMERE \
                INCLUDE_MATXM_CORE_411_WR \
                INCLUDE_EMERALD_LAKE \
                INCLUDE_EMERALD_LAKE_II \
                INCLUDE_SHUMWAY \
                INCLUDE_STARGO \
                INCLUDE_BEARLAKE \
                INCLUDE_MOONCREEK \
                INCLUDE_CEDAR_ROCK \
                INCLUDE_NORCO \
                INCLUDE_BAY_TRAIL
    DEFAULTS    INCLUDE_INTEL_GENERIC
    _CHILDREN   FOLDER_BSP_CONFIG
}

/*
 * This component is included by default to give access to
 * the boardInfoShow routine. Users may remove it by removing
 * this component from their VIP configuration.
 */

Component INCLUDE_BOARD_INFO_SHOW {
    NAME         Board information
    SYNOPSIS     This component provides access to the boardInfoShow routine.
    INCLUDE_WHEN INCLUDE_BOARD_CONFIG
    _CHILDREN    FOLDER_NOT_VISIBLE
}

Component INCLUDE_INTEL_GENERIC {
    NAME        Intel generic
    SYNOPSIS    This component provides generic parameters for Intel boards.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_WHITETIP_MOUNTAIN {
    NAME        Intel WhiteTip Mountain
    SYNOPSIS    This component provides parameters for Intel WhiteTip Mountain board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_CAMELBACK_MOUNTAIN {
    NAME        Intel Camelback mountain
    SYNOPSIS    This component provides parameters for Intel Camelback Mountain board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_WALNUT_CANYON_2 {
    NAME        Intel Walnut Canyon 2
    SYNOPSIS    This component provides parameters for Intel Walnut Canyon 2 board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_WALNUT_CANYON {
    NAME        Intel Walnut Canyon
    SYNOPSIS    This component provides parameters for Intel Walnut canyon board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_FLATHEAD_CREEK {
    NAME        Intel Flathead Creek
    SYNOPSIS    This component provides parameters for Intel Flathead Creek board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_LAVA_CANYON {
    NAME        Intel Lava Canyon
    SYNOPSIS    This component provides parameters for Intel Lava Canyon board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_OSAGE {
    NAME        Osage
    SYNOPSIS    This component provides parameters for Osage board.
    REQUIRES    INCLUDE_BOARD_CONFIG
    EXCLUDES    INCLUDE_CPU_PWR_MGMT \
                INCLUDE_PWR_CPU_MON \
                INCLUDE_PWR_CPU_MON_ARCH \
                INCLUDE_PWR_LIB_FUNCS \
                INCLUDE_PWR_POLICY_MGR \
                INCLUDE_DEV_PWR_MGMT \
                INCLUDE_TICKLESS_IDLE \
                INCLUDE_IDLE_PWR_MGMT_INIT \
                INCLUDE_CPU_SLEEP_MODE_LONG_PARAMS \
                SELECT_POWER_MGMT
}

Component INCLUDE_GREENCITY {
    NAME        Green City
    SYNOPSIS    This component provides parameters for Green City board.
    REQUIRES    INCLUDE_BOARD_CONFIG
    EXCLUDES    INCLUDE_CPU_PWR_MGMT \
                INCLUDE_PWR_CPU_MON \
                INCLUDE_PWR_CPU_MON_ARCH \
                INCLUDE_PWR_LIB_FUNCS \
                INCLUDE_PWR_POLICY_MGR \
                INCLUDE_DEV_PWR_MGMT \
                INCLUDE_TICKLESS_IDLE \
                INCLUDE_IDLE_PWR_MGMT_INIT \
                INCLUDE_CPU_SLEEP_MODE_LONG_PARAMS \
                SELECT_POWER_MGMT
}

Component INCLUDE_MATXM_CORE_411_WR {
    NAME        Emerson MATXM_CORE_411_WR
    SYNOPSIS    This component provides parameters for Emerson MATXM_CORE_411_WR board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_HANLAN_CREEK_WESTMERE {
    NAME        Intel Hanlan Creek Westmere
    SYNOPSIS    This component provides parameters for Intel Hanlan Creek Westmere board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_EMERALD_LAKE {
    NAME        Emerald Lake
    SYNOPSIS    This component provides parameters for Emerald Lake board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_EMERALD_LAKE_II {
    NAME        Emerald Lake II
    SYNOPSIS    This component provides parameters for Emerald Lake II board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_SHUMWAY {
    NAME        Shumway
    SYNOPSIS    This component provides parameters for Shumway board.
    REQUIRES    INCLUDE_BOARD_CONFIG
    CFG_PARAMS  SYS_SECOND_PCI_MIN_BUS \
                SYS_SECOND_PCI_MAX_BUS \
                SYS_THIRD_PCI_MIN_BUS \
                SYS_THIRD_PCI_MAX_BUS
}

Component INCLUDE_STARGO {
    NAME        Stargo
    SYNOPSIS    This component provides parameters for Stargo board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_BEARLAKE {
    NAME        Bearlake
    SYNOPSIS    This component provides parameters for Bearlake board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_MOONCREEK {
    NAME        Moon Creek
    SYNOPSIS   This component provides parameters for Moon Creek board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_CEDAR_ROCK {
    NAME        Cedar Rock
    SYNOPSIS    This component provides parameters for Cedar Rock board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_NORCO {
    NAME        NORCO
    SYNOPSIS   This component provides parameters for Norco board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Component INCLUDE_BAY_TRAIL {
    NAME        Bay Trail
    SYNOPSIS    PThis component provides parameters for Bay Trail board.
    REQUIRES    INCLUDE_BOARD_CONFIG
}

Parameter TARGET_NAME {
    NAME     Target
    SYNOPSIS This parameter specifies the target board name, leading space necessary.
    TYPE     string
    DEFAULT  (INCLUDE_INTEL_GENERIC && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT GENERIC") \
             (INCLUDE_INTEL_GENERIC)::(" GENERIC") \
             (INCLUDE_WHITETIP_MOUNTAIN && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT WhiteTip Mountain") \
             (INCLUDE_WHITETIP_MOUNTAIN)::(" WhiteTip Mountain") \
             (INCLUDE_CAMELBACK_MOUNTAIN && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Camelback Mountain") \
             (INCLUDE_CAMELBACK_MOUNTAIN)::(" Camelback Mountain") \
             (INCLUDE_WALNUT_CANYON_2 && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Walnut Canyon 2") \
             (INCLUDE_WALNUT_CANYON_2)::(" Walnut Canyon 2") \
             (INCLUDE_WALNUT_CANYON && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Walnut Canyon") \
             (INCLUDE_WALNUT_CANYON)::(" Walnut Canyon") \
             (INCLUDE_FLATHEAD_CREEK && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Flathead Creek") \
             (INCLUDE_FLATHEAD_CREEK)::(" Flathead Creek") \
             (INCLUDE_LAVA_CANYON && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Lava Canyon") \
             (INCLUDE_LAVA_CANYON)::(" Lava Canyon") \
             (INCLUDE_GREENCITY && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Green City") \
             (INCLUDE_GREENCITY)::(" Green City") \
             (INCLUDE_HANLAN_CREEK_WESTMERE && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Hanlan Creek Westmere") \
             (INCLUDE_HANLAN_CREEK_WESTMERE)::(" Hanlan Creek Westmere") \
             (INCLUDE_OSAGE && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Osage") \
             (INCLUDE_OSAGE)::(" Osage") \
             (INCLUDE_MATXM_CORE_411_WR && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT MATXM-CORE-411-WR") \
             (INCLUDE_MATXM_CORE_411_WR)::(" MATXM-CORE-411-WR") \
             (INCLUDE_EMERALD_LAKE && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Emerald Lake") \
             (INCLUDE_EMERALD_LAKE)::(" Emerald Lake") \
             (INCLUDE_EMERALD_LAKE_II && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Emerald Lake II") \
             (INCLUDE_EMERALD_LAKE_II)::(" Emerald Lake II") \
             (INCLUDE_SHUMWAY && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Shumway") \
             (INCLUDE_SHUMWAY)::(" Shumway") \
             (INCLUDE_STARGO && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Stargo") \
             (INCLUDE_STARGO)::(" Stargo") \
             (INCLUDE_BEARLAKE && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT BearLake") \
             (INCLUDE_BEARLAKE)::(" BearLake") \
             (INCLUDE_MOONCREEK && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Mooncreek") \
             (INCLUDE_MOONCREEK)::(" Mooncreek") \
             (INCLUDE_CEDAR_ROCK && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT CedarRock") \
             (INCLUDE_CEDAR_ROCK)::(" CedarRock") \
             (INCLUDE_NORCO && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT Norco") \
             (INCLUDE_NORCO)::(" Norco") \
             (INCLUDE_BAY_TRAIL && INCLUDE_SMP_SCHED_SMT_POLICY)::("/SMT BayTrail") \
             (INCLUDE_BAY_TRAIL)::(" BayTrail") \
             "No Bundle Selected"
}

Parameter PROCESSOR_CLASS {
    NAME     Processor
    SYNOPSIS This parameter defines the processor class name.
    TYPE     string
    DEFAULT  (INCLUDE_WHITETIP_MOUNTAIN)::("Intel(R) Nehalem") \
             (INCLUDE_INTEL_GENERIC)::("Intel(R) 64 ISA") \
             (INCLUDE_CAMELBACK_MOUNTAIN)::("Intel(R) Broadwell") \
             (INCLUDE_WALNUT_CANYON_2)::("Intel(R) Broadwell") \
             (INCLUDE_WALNUT_CANYON)::("Intel(R) Haswell") \
             (INCLUDE_FLATHEAD_CREEK)::("Intel(R) Haswell") \
             (INCLUDE_LAVA_CANYON)::("Intel(R) Haswell") \
             (INCLUDE_GREENCITY)::("Intel(R) Nehalem") \
             (INCLUDE_HANLAN_CREEK_WESTMERE)::("Intel(R) Westmere") \
             (INCLUDE_OSAGE)::("Intel(R) Nehalem") \
             (INCLUDE_MATXM_CORE_411_WR)::("Intel(R) Nehalem") \
             (INCLUDE_EMERALD_LAKE)::("Intel(R) Nehalem") \
             (INCLUDE_EMERALD_LAKE_II)::("Intel(R) Nehalem") \
             (INCLUDE_SHUMWAY)::("Intel(R) Nehalem") \
             (INCLUDE_STARGO)::("Intel(R) Nehalem") \
             (INCLUDE_BEARLAKE)::("Intel(R) Core2") \
             (INCLUDE_MOONCREEK)::("Intel(R) Atom") \
             (INCLUDE_CEDAR_ROCK)::("Intel(R) Atom") \
             (INCLUDE_NORCO)::("Intel(R) Atom") \
             (INCLUDE_BAY_TRAIL)::("Intel(R) Atom") \
             "Intel(R)"
}

Parameter SYS_MODEL {
    NAME     System model
    SYNOPSIS This parameter specifies the system model string.
    TYPE     string
#if (!defined _WRS_CONFIG_SMP)
    DEFAULT (INCLUDE_ACPI_BOOT_OP)::(PROCESSOR_CLASS " Processor ACPI" TARGET_NAME) \
             (INCLUDE_MPTABLE_BOOT_OP)::(PROCESSOR_CLASS " Processor BIOS MP Table" TARGET_NAME) \
             (INCLUDE_USR_BOOT_OP)::(PROCESSOR_CLASS " Processor User MP Table" TARGET_NAME)
#else
    DEFAULT (INCLUDE_ACPI_BOOT_OP)::(PROCESSOR_CLASS " Processor ACPI SMP" TARGET_NAME) \
             (INCLUDE_MPTABLE_BOOT_OP)::(PROCESSOR_CLASS " Processor BIOS MP Table SMP" TARGET_NAME) \
             (INCLUDE_USR_BOOT_OP)::(PROCESSOR_CLASS " Processor User MP Table SMP" TARGET_NAME)
#endif
}

/*
 * SYS_AP_LOOP_COUNT is a SMP parameter used in function sysCpuStart.
 * It is used to set the count of times to check if an application
 * processor succeeded in starting up.
 */

#if (defined _WRS_CONFIG_SMP)
Parameter SYS_AP_LOOP_COUNT {
    NAME     System AP startup checks
    SYNOPSIS This parameter defines the maximum times to check if an application processor started.
    TYPE     uint
    DEFAULT  (200000)
}
#endif /* _WRS_CONFIG_SMP */

/*
 * SYS_AP_TIMEOUT is a SMP parameter used in function sysCpuStart.
 * It is used to set the time in microseconds to wait between checking
 * if an application processor succeeded in starting up.
 * The time is specified in microseconds and should be short in duration.
 * SYS_AP_LOOP_COUNT * SYS_AP_TIMEOUT gives the total time to wait for an AP
 * before giving up and moving on to the next application processor.
 */

#if (defined _WRS_CONFIG_SMP)
Parameter SYS_AP_TIMEOUT {
    NAME     System AP startup timeout
    SYNOPSIS This parameter specifies the time between each check to see if application processor started.
    TYPE     uint
    DEFAULT  (10)
}
#endif /* _WRS_CONFIG_SMP */

Component INCLUDE_BOOT_GO_OVERRIDE {
    NAME         User defined Go routine support
    SYNOPSIS     This component includes user defined Go routine support. \
                 The prototype must be in the form of \
                 "void funcName(void * entryPoint,int startType,int elfClass)".
    PROTOTYPE    IMPORT void bootGoOverrideFuncAdd(void *); \
                 IMPORT void BOOT_GO_OVERRIDE_FUNCTION (void * entryPoint,int startType,int elfClass);
    INIT_RTN     bootGoOverrideFuncAdd ((void *)BOOT_GO_OVERRIDE_FUNCTION);
    _INIT_ORDER  usrBootAppInit
    INIT_AFTER   INCLUDE_BOOT_TSFS_LOADER
    INCLUDE_WHEN INCLUDE_BOOT_APP
    REQUIRES     INCLUDE_BOOT_ELF
    CFG_PARAMS   BOOT_GO_OVERRIDE_FUNCTION
    _CHILDREN    INCLUDE_BOOT_APP
}

Parameter       BOOT_GO_OVERRIDE_FUNCTION {
    NAME        User defined Go routine
    SYNOPSIS    This parameter is used to override the original Go routine with the user defined Go routine.
    DEFAULT     sysBootElf32Or64
}

/*
 * Network Boot Devices for a BSP.
 */

Component       INCLUDE_FEI8255X_VXB_END {
        INCLUDE_WHEN \
                += INCLUDE_BOOT_NET_DEVICES
}

Component       INCLUDE_GEI825XX_VXB_END {
        INCLUDE_WHEN \
                += INCLUDE_BOOT_NET_DEVICES
}

/* Console device selection */

Selection SELECT_CONSOLE {
    NAME        Console device selection (serial and/or VGA)
    COUNT       1-
    CHILDREN    DRV_SIO_IA_NS16550 \
                INCLUDE_PC_CONSOLE \
                DRV_SIO_PCI_NS16550
    DEFAULTS    DRV_SIO_IA_NS16550
    _CHILDREN   FOLDER_BSP_CONFIG
}

Component DRV_SIO_PCI_NS16550 {
    REQUIRES += DRV_IA_PCI_BUS
}

/* Multiboot enable/disable selection */

Component GRUB_MULTIBOOT_DISABLED {
    NAME        Disable multiboot
}

Selection SELECT_MULTIBOOT {
    NAME        Select multiboot support (enable/disable)
    COUNT       1-1
    CHILDREN    GRUB_MULTIBOOT \
                GRUB_MULTIBOOT_DISABLED
    DEFAULTS    GRUB_MULTIBOOT
    _CHILDREN   FOLDER_BSP_CONFIG
}

/*
 * Warm boot device selection required for
 * NVRAM support
 */

Selection SELECT_SYS_WARM_TYPE {
    NAME        Warm start device selection
    COUNT       1-1
    CHILDREN    INCLUDE_SYS_WARM_BIOS \
                INCLUDE_SYS_WARM_USB \
                INCLUDE_SYS_WARM_AHCI
    DEFAULTS    INCLUDE_SYS_WARM_BIOS
    _CHILDREN   FOLDER_BSP_CONFIG
}

/*
 * Warm boot device components
 */

Component INCLUDE_SYS_WARM_BIOS {
    NAME        BIOS warm start
    CFG_PARAMS  SYS_WARM_TYPE \
                NV_RAM_SIZE
#ifdef _WRS_CONFIG_BDM_NVRAM
    REQUIRES    INCLUDE_NVRAM_NULL
#endif /* _WRS_CONFIG_BDM_NVRAM */

}

Component INCLUDE_SYS_WARM_USB {
    NAME        USB warm start
    REQUIRES    INCLUDE_USB \
                INCLUDE_USB_INIT \
                INCLUDE_EHCI \
                INCLUDE_EHCI_INIT \
                INCLUDE_DOSFS
    CFG_PARAMS  SYS_WARM_TYPE \
                NV_RAM_SIZE \
                BOOTROM_DIR
}

Component INCLUDE_SYS_WARM_AHCI {
    NAME        AHCI warm start
    REQUIRES    PCI_DRV_STORAGE_AHCI \
                INCLUDE_DOSFS
    CFG_PARAMS  SYS_WARM_TYPE \
                NV_RAM_SIZE \
                BOOTROM_DIR
}

Parameter SYS_WARM_TYPE {
    NAME        Warm start
    DEFAULT     (INCLUDE_SYS_WARM_BIOS)::(SYS_WARM_BIOS) \
                (INCLUDE_SYS_WARM_USB)::(SYS_WARM_USB) \
                (INCLUDE_SYS_WARM_AHCI)::(SYS_WARM_AHCI) \
                (SYS_WARM_BIOS)
}

Parameter NV_RAM_SIZE {
    DEFAULT     (INCLUDE_SYS_WARM_BIOS)::(NONE) \
                (0x1000)
}

Parameter BOOTROM_DIR {
    DEFAULT     (INCLUDE_SYS_WARM_USB)::("/bd0") \
                (INCLUDE_SYS_WARM_AHCI)::("/ata0:1") \
                (NULL)
}

Component INCLUDE_MULTI_STAGE_BOOT {
    NAME        Multi-stage boot support
    SYNOPSIS    This component is used to select either the multi-stage fast or warm reboot mechanism.
    REQUIRES    ROM_BASE_ADRS \
                ROM_TEXT_ADRS \
                ROM_SIZE
}

Selection SELECT_MULTI_STAGE_REBOOT_TYPE {
    NAME        Multi-stage boot type select
    COUNT       1-1
    CHILDREN    INCLUDE_FAST_REBOOT \
                INCLUDE_MULTI_STAGE_WARM_REBOOT
}

Component INCLUDE_FAST_REBOOT {
    NAME        Multi-stage fast reboot type
    SYNOPSIS    This component selectes the multi-stage fast reboot mechanism.
    REQUIRES    INCLUDE_MULTI_STAGE_BOOT
    EXCLUDES    INCLUDE_MULTI_STAGE_WARM_REBOOT
}

Component INCLUDE_MULTI_STAGE_WARM_REBOOT {
    NAME        Multi-stage warm reboot type
    SYNOPSIS    This component selects the multi-stage warm reboot mechanism.
    REQUIRES    INCLUDE_MULTI_STAGE_BOOT
    EXCLUDES    INCLUDE_FAST_REBOOT
}

Component INCLUDE_WARM_BOOT {
    REQUIRES  += ROM_BASE_ADRS \
                 ROM_SIZE
    EXCLUDES  += INCLUDE_FAST_REBOOT \
                 INCLUDE_MULTI_STAGE_WARM_REBOOT
}

Parameter SYSTEM_RAM_SIZE {
    NAME        Minimum system RAM
    DEFAULT     0x80000000
}

Parameter N_UART_CHANNELS {
    NAME            Number of UART channels
    SYNOPSIS        This parameter specifies the number of UART channels supported.
    TYPE            uint
    DEFAULT         (1)
}

Parameter CONSOLE_BAUD_RATE {
    NAME            Console baud rate
    SYNOPSIS        This parameter specifies the console baud rate.
    TYPE            unsigned long
    DEFAULT         (INCLUDE_CAMELBACK_MOUNTAIN)::(115200) \
                    (INCLUDE_BAY_TRAIL)::(115200) \
                    (9600)
}

Parameter NUM_TTY {
    NAME            Number of serial ports
    SYNOPSIS        This parameter specifies the number of serial ports supported.
    TYPE            uint
    DEFAULT         (1)
}

Parameter PC_CONSOLE {
    NAME            PC console number
    SYNOPSIS        This parameter specifies the PC console number.
    TYPE            uint
    DEFAULT         (0x0)
}

Parameter LOCAL_MEM_PHYS_ADRS_MASK {
    NAME        Physical address mask
    DEFAULT     0x000000007fffffff
}

Component INCLUDE_SYS_HW_INIT_0 {
    NAME        BSP specific initialization
    SYNOPSIS    This component provides BSP specific initialization before cacheLibInit() is called and/or the BSS is cleared.
    PROTOTYPE   void sysHwInit0 (void );
    CFG_PARAMS  SYS_HW_INIT_0
}

Parameter SYS_HW_INIT_0 {
    NAME        BSP specific initialization function
    TYPE        void
    DEFAULT     sysHwInit0
}

/*
 * Add loader components for convenience
 */

Component        INCLUDE_BOOT_FS_DEVICES_USB {
    REQUIRES     INCLUDE_BOOT_USB_FS_LOADER
    INCLUDE_WHEN INCLUDE_BOOT_FS_DEVICES INCLUDE_SYS_WARM_USB
    _CHILDREN    FOLDER_NOT_VISIBLE
}

Component        INCLUDE_BOOT_FS_DEVICES_AHCI {
    REQUIRES     INCLUDE_BOOT_FILESYSTEMS
    INCLUDE_WHEN INCLUDE_BOOT_FS_DEVICES INCLUDE_SYS_WARM_AHCI
    _CHILDREN    FOLDER_NOT_VISIBLE
}

/*
 * VX_SMP_NUM_CPUS is a SMP parameter only and only available for SMP
 * builds. Due to a limitation of the project tool at the time this
 * parameter is created where the tool can not recognize the ifdef SMP
 * selection, this parameter is set up such that _CFG_PARAMS is not
 * specified here. In the 00vxWorks.cdf file, where the parameter
 * VX_SMP_NUM_CPUS is defined, the _CFG_PARAMS is specified only for
 * VxWorks SMP. Hence the redefining of VX_SMP_NUM_CPUS here should only
 * override the value and not the rest of the properties. And for UP, the
 * parameter is ignored since the parameter is not tied to any component
 * (_CFG_PARAMS is not specified).
 */

Parameter VX_SMP_NUM_CPUS {
    NAME        Number of CPUs available to be enabled for VxWorks SMP
    TYPE        UINT
    DEFAULT     (INCLUDE_WHITETIP_MOUNTAIN && INCLUDE_SMP_SCHED_SMT_POLICY)::(4) \
                (INCLUDE_WHITETIP_MOUNTAIN)::(2) \
                (INCLUDE_INTEL_GENERIC && INCLUDE_SMP_SCHED_SMT_POLICY)::(8) \
                (INCLUDE_INTEL_GENERIC)::(4) \
                (INCLUDE_CAMELBACK_MOUNTAIN && INCLUDE_SMP_SCHED_SMT_POLICY)::(16) \
                (INCLUDE_CAMELBACK_MOUNTAIN)::(8) \
                (INCLUDE_WALNUT_CANYON_2 && INCLUDE_SMP_SCHED_SMT_POLICY)::(8) \
                (INCLUDE_WALNUT_CANYON_2)::(4) \
                (INCLUDE_WALNUT_CANYON && INCLUDE_SMP_SCHED_SMT_POLICY)::(8) \
                (INCLUDE_WALNUT_CANYON)::(4) \
                (INCLUDE_FLATHEAD_CREEK && INCLUDE_SMP_SCHED_SMT_POLICY)::(8) \
                (INCLUDE_FLATHEAD_CREEK)::(4) \
                (INCLUDE_LAVA_CANYON && INCLUDE_SMP_SCHED_SMT_POLICY)::(8) \
                (INCLUDE_LAVA_CANYON)::(4) \
                (INCLUDE_OSAGE && INCLUDE_SMP_SCHED_SMT_POLICY)::(16) \
                (INCLUDE_OSAGE)::(8) \
                (INCLUDE_GREENCITY && INCLUDE_SMP_SCHED_SMT_POLICY)::(16) \
                (INCLUDE_GREENCITY)::(8) \
                (INCLUDE_HANLAN_CREEK_WESTMERE && INCLUDE_SMP_SCHED_SMT_POLICY)::(24) \
                (INCLUDE_HANLAN_CREEK_WESTMERE)::(12) \
                (INCLUDE_HANLANCREEK && INCLUDE_SMP_SCHED_SMT_POLICY)::(16) \
                (INCLUDE_HANLANCREEK)::(8) \
                (INCLUDE_MATXM_CORE_411_WR && INCLUDE_SMP_SCHED_SMT_POLICY)::(4) \
                (INCLUDE_MATXM_CORE_411_WR)::(2) \
                (INCLUDE_EMERALD_LAKE && INCLUDE_SMP_SCHED_SMT_POLICY)::(8) \
                (INCLUDE_EMERALD_LAKE)::(4) \
                (INCLUDE_EMERALD_LAKE_II && INCLUDE_SMP_SCHED_SMT_POLICY)::(8) \
                (INCLUDE_EMERALD_LAKE_II)::(4) \
                (INCLUDE_SHUMWAY && INCLUDE_SMP_SCHED_SMT_POLICY)::(32) \
                (INCLUDE_SHUMWAY)::(16) \
                (INCLUDE_STARGO && INCLUDE_SMP_SCHED_SMT_POLICY)::(8) \
                (INCLUDE_STARGO)::(4) \
                (INCLUDE_MOONCREEK && INCLUDE_SMP_SCHED_SMT_POLICY)::(4) \
                (INCLUDE_MOONCREEK)::(2) \
                (INCLUDE_CEDAR_ROCK && INCLUDE_SMP_SCHED_SMT_POLICY)::(4) \
                (INCLUDE_CEDAR_ROCK)::(2) \
                (INCLUDE_NORCO && INCLUDE_SMP_SCHED_SMT_POLICY)::(4) \
                (INCLUDE_NORCO)::(2) \
                (INCLUDE_BAY_TRAIL && INCLUDE_SMP_SCHED_SMT_POLICY)::(4) \
                (INCLUDE_BAY_TRAIL)::(2) \
                (8)
}

Parameter DEFAULT_BOOT_LINE {
    NAME        Default boot line
    SYNOPSIS    This parameter specifies the default boot line string.
    TYPE        string
    DEFAULT     "gei(0,0)host:vxWorks h=90.0.0.3 e=90.0.0.50 u=target"
}

Parameter APIC_TIMER_CLOCK_HZ  {
    NAME        APIC timer clock rate configuration parameter
    SYNOPSIS    This parameter specifies the APIC timer clock rate (0 causes auto-calculation).
    TYPE        uint
    DEFAULT     (0)
}

/*******************************************************************************
*
* Interrupt Mode Configuration
*
*/

Component INCLUDE_SYMMETRIC_IO_MODE {
    NAME        Symmetric I/O mode
    SYNOPSIS    This component includes the symmetric I/O interrupt mode.
    _CHILDREN   FOLDER_NOT_VISIBLE
}

/*******************************************************************************
*
* MP Table Configuration Options
*
*/

Component INCLUDE_ACPI_MPAPIC {
    NAME        ACPI MP APIC
    SYNOPSIS    This component includes ACPI MP APIC.
    REQUIRES    INCLUDE_ACPI_BOOT_OP
    _CHILDREN   FOLDER_NOT_VISIBLE
}

Selection SELECT_MPAPIC_MODE {
    NAME        MP APIC boot options
    SYNOPSIS    Selects MP APIC struct creation method.
    COUNT       1-1
    CHILDREN    INCLUDE_ACPI_BOOT_OP \
                INCLUDE_USR_BOOT_OP \
                INCLUDE_MPTABLE_BOOT_OP
    DEFAULTS    INCLUDE_ACPI_BOOT_OP
    _CHILDREN   FOLDER_BSP_CONFIG
}

Component INCLUDE_ACPI_BOOT_OP {
    NAME        ACPI MP APIC boot
    SYNOPSIS    This component provides the ACPI MP APIC boot option.
    REQUIRES    INCLUDE_ACPI_MPAPIC \
                INCLUDE_ACPI_TABLE_MAP \
                INCLUDE_USR_MPAPIC
}

Component INCLUDE_USR_BOOT_OP {
    NAME        User defined MP APIC boot
    SYNOPSIS    This component provides the user defined MP APIC option.
}

Component INCLUDE_MPTABLE_BOOT_OP {
    NAME        BIOS MP APIC boot
    SYNOPSIS    This component provides the BIOS MP APIC boot option.
}

/*******************************************************************************
*
* PCI Bus controller configuration
*
* On shumway board, the first PCI bus controller starts from bus 0 and
* the second PCI bus controller starts from bus 128 but they are not bridge
* connected.  In order to search the bus 128 or above, we create the second
* VxBus PCI bus controller device instance with the start bus number 128.
*
* The sub-class of the PCI bus controller, that bridges to bus 130, on
* the bus 128 is not configured as a PCI-to-PCI bridge on some shumway boards.
* To search the bus 130 or above, we create another VxBus PCI bus controller
* device instance with the start bus number 130.
*
* Note:
* The PCI_BUS_CTRL_NUM configuration currently supported is 1, 2 or 3 only.
*/

Parameter SYS_PCI_BUS_CTRL_NUM {
    NAME     PCI bus controllers
    SYNOPSIS This parameter specifies the number of PCI bus controllers.
    TYPE     uint
    DEFAULT  (INCLUDE_SHUMWAY)::(3) \
             (1)
}

Parameter SYS_FIRST_PCI_MIN_BUS {
    NAME     First controller bus start
    SYNOPSIS This parameter specifies the start bus for first PCI bus controller.
    TYPE     UINT
    DEFAULT  (0)
}

Parameter SYS_FIRST_PCI_MAX_BUS {
    NAME     First controller Bus end
    SYNOPSIS This parameter specifies the end bus for first PCI bus controller.
    TYPE     UINT
    DEFAULT  (INCLUDE_SHUMWAY)::(127) \
             (PCI_MAX_BUS)
}

Parameter SYS_SECOND_PCI_MIN_BUS {
    NAME     Second controller bus start
    SYNOPSIS This parameter specifies the start bus for second PCI bus controller.
    TYPE     UINT
    DEFAULT  (INCLUDE_SHUMWAY)::(128) \
             (0)
}

Parameter SYS_SECOND_PCI_MAX_BUS {
    NAME     Second controller bus end
    SYNOPSIS This parameter specifies the end bus for second PCI bus controller.
    TYPE     UINT
    DEFAULT  (INCLUDE_SHUMWAY)::(129) \
             (0)
}

Parameter SYS_THIRD_PCI_MIN_BUS {
    NAME     Third controller bus start
    SYNOPSIS This parameter specifies the start bus for third PCI bus controller.
    TYPE     UINT
    DEFAULT  (INCLUDE_SHUMWAY)::(130) \
             (0)
}

Parameter SYS_THIRD_PCI_MAX_BUS {
    NAME     Third controller bus end
    SYNOPSIS This parameter specifies the end bus for third PCI bus controller.
    TYPE     UINT
    DEFAULT  (INCLUDE_SHUMWAY)::(PCI_MAX_BUS) \
             (0)
}

/*******************************************************************************
*
* ACPI
*
*/

Component INCLUDE_ACPI {
    REQUIRES +=     IA_PCI_UTILS
}

Component INCLUDE_ACPI_DEBUG_PRINT {
    NAME            ACPI debug boot messages
    SYNOPSIS        This component provides functions allowing the user to view ACPI log and error messages during the boot sequence. \
                    NOTE: This is a debug tool and is not formally supported.
    REQUIRES        INCLUDE_ACPI \
                    INCLUDE_EARLY_PRINT
    _CHILDREN       FOLDER_BSP_CONFIG
}

/*******************************************************************************
*
* Early Print
*
*/

Component INCLUDE_EARLY_PRINT_VGA {
    NAME            Early print
    SYNOPSIS        This component provides functions to print messages to VGA during boot.
    REQUIRES        INCLUDE_PMAP_LIB
    PROTOTYPE       void earlyPrintVga(char *msg);
}

Component INCLUDE_EARLY_PRINT {
    NAME            Early print
    SYNOPSIS        This component provides functions to print messages during boot.
    REQUIRES        INCLUDE_EARLY_PRINT_VGA
    PROTOTYPE       void earlyPrint(char *msg);
    _CHILDREN       FOLDER_BSP_CONFIG
}

Component INCLUDE_DEBUG_KPRINTF {
    REQUIRES +=     INCLUDE_EARLY_PRINT
}

/*******************************************************************************
*
* Cache
*
*/

Component INCLUDE_EARLY_I_CACHE_ENABLE {
    NAME            Enable instruction cache
    SYNOPSIS        This component enables instruction cache early in sysHwInit.
    _CHILDREN       FOLDER_BSP_CONFIG
}

Component INCLUDE_EARLY_D_CACHE_ENABLE {
    NAME            Enable data cache
    SYNOPSIS        This component enables data cache early in sysHwInit.
    _CHILDREN       FOLDER_BSP_CONFIG
}

/*******************************************************************************
*
* GRUB Support
*
*/

Component INCLUDE_MULTIBOOT_MEM_AUTOSIZE {
    NAME          Multiboot memory autosize
    SYNOPSIS      This component enables memory autosize using Multiboot header.
    REQUIRES      GRUB_MULTIBOOT
    INCLUDE_WHEN  GRUB_MULTIBOOT
}

Component GRUB_MULTIBOOT {
    NAME        GRUB multiboot support
    SYNOPSIS    This component provides GRUB multiboot support.
    REQUIRES    SELECT_MPAPIC_MODE
}

/*
 * Force unsupported components to be unavailable.
 *
 * The following component definition(s) forces the named component(s)
 * to be "unavailable" as far as the project build facility (vxprj) is
 * concerned. The required component (COMPONENT_NOT_SUPPORTED) should
 * never be defined, and hence the named component(s) will never be
 * available. This little trick is used by the BSPVTS build scripts
 * (suiteBuild, bspBuildTest.tcl) to automatically exclude test modules
 * that are not applicable to a BSP because the BSP does not support a
 * given component and/or test module. If and when support is added,
 * the following definition(s) should be removed.
 */

Component INCLUDE_ATA {
    REQUIRES    COMPONENT_NOT_SUPPORTED
}

/* Power Management not yet supported */

Component INCLUDE_CPU_PWR_ARCH {
    REQUIRES    COMPONENT_NOT_SUPPORTED
}
