/* boardSys.c - itl_64_vx7 Board system library support */

/*
 * Copyright (c) 2014-2016 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify, or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

/*
modification history
--------------------
23Sep16,y_h  fix warning caused by double in boardInfoShow (V7PRO-3104)
01jul16,jsp  HYP-11845 - removed hvVmReset()
08jan16,jlv  remove BOARD_LIB_NO_PROBE for i86 (V7PRO-2766)
20oct15,j_l  V7PRO-2511 - fix warm reboot causing a cold reboot when ctrl-x
                          pressed just after "Loading..." message is printed
09jul15,l_z  add fast warm boot support. (F3387)
22apr15,j_l  V7PRO-1636 - sysUsDelay() busy waits with preemption locked in SMP
17feb15,j_l  V7PRO-1764 - Additional check in sysUsDelay to see whether
                          it is safe to do a taskCpuLock
09jan15,rec  HYP-10119 - use hvStatus_t for hypervisr return status
09sep14,rec  Merge HV sysToMonitor support from itl_64 BSP
04nov14,rbc  US44720 - Fix static analysis errors
13aug14,ten  US44071 - Merge multiboot changes from itl_64 BSP:
                       US38341, V7PRO-891
20may14,jjk  US35956 - New API to query CPU/SOC versions.
08may14,scm  US32988/TA40489 - Test/Debug/Validate IA GEN2 drivers, IPIs...
18mar14,j_l  US32795 - changes for new gnu compiler
06mar14,j_l  remove ramVirtTop (no longer exists) field from
             BOARD_FUNC_TBL structure
23apr14,to   use inline assembly for LOAPIC memory mapped register access
             (US20864)
04feb14,scm  US22784 - 6.9.3.3 merge - Intel Arch phase #2...
04feb14,jjk  DE5023 - Do not duplicate source in VSB.
04feb14,j_l  US24659 - board system SMP support
04feb14,scm  US20759 - remove unsupported interrupt modes...
03feb14,j_l  US18115 - IA implementation of board system
                       created
*/

/*
DESCRIPTION
This file provides board system library support.
*/

/* includes */

#include <boardLib.h>
#ifdef INCLUDE_BOARD_INFO_SHOW
#include <ctype.h>
#endif /* INCLUDE_BOARD_INFO_SHOW */

#ifdef _WRS_CONFIG_SMP
#include <private/cpcLibP.h>
#endif /* _WRS_CONFIG_SMP */

void           sys1msDelay ();
IMPORT void    sysUsDelay (int);
IMPORT void    sysMsDelay (int);
IMPORT void    sysDelay (void);
LOCAL  void    itl64SysUsDelay (int);
LOCAL  void    itl64SysToMonitor (int);
LOCAL  char   *itl64SysModel (void);
LOCAL  BOOL    itlProbe (char *);
#ifdef INCLUDE_BOARD_INFO_SHOW
LOCAL  void    sysBoardInfoStrInit (void);
LOCAL  void    itl64SysInfoShow (void);
#endif /* INCLUDE_BOARD_INFO_SHOW */

IMPORT STATUS loApicIntrEnable (BOOL * enable);

#ifdef _WRS_CONFIG_SMP
LOCAL  STATUS  itl64SysCpuEnable(unsigned int cpuNum, WIND_CPU_STATE *cpuState);
LOCAL  UINT32  itl64SysCpuAvailableGet(void);
#endif  /* _WRS_CONFIG_SMP */

LOCAL BOARD_FUNC_TBL bspBoardFuncTbl =
    {
    .earlyInit  = sysHwInit,
    .init       = sysHwInit2,
    .reset      = itl64SysToMonitor,
    .model      = itl64SysModel,
    .usDelay    = itl64SysUsDelay,
#ifdef _WRS_CONFIG_SMP
    .cpuEn      = itl64SysCpuEnable,
    .cpuAvail   = itl64SysCpuAvailableGet,
#endif  /* _WRS_CONFIG_SMP */
#ifdef INCLUDE_BOARD_INFO_SHOW
    .infoShow   = itl64SysInfoShow,
#endif /* INCLUDE_BOARD_INFO_SHOW */
    };

LOCAL BOARD_DESC itl64Vx7BoardDesc = {
    .uVer     = BOARD_DESC_VER_2_0,
    .pCompat  = "itl_64_vx7",
    .uFlag    = 0,
    .probe    = itlProbe,
    .pFuncTbl = &bspBoardFuncTbl
    };

BOARD_DEF(itl64Vx7BoardDesc)

#ifdef INCLUDE_BOARD_INFO_SHOW
LOCAL char   sysBoardInfoStr[80];
LOCAL char   sysBoardImageStr[80];
LOCAL STATUS sysBoardInfoStrInitialized = FALSE;
#endif /* INCLUDE_BOARD_INFO_SHOW */

/******************************************************************************
*
* itl64SysModel - return the model name of the CPU board
*
* This routine returns the model name of the CPU board.
*
* RETURNS: A pointer to the SYS_MODEL string.
*/

LOCAL char *itl64SysModel (void)
    {
    return (SYS_MODEL);
    }

#ifdef INCLUDE_BOARD_INFO_SHOW

/******************************************************************************
*
* sysBoardInfoStrInit - initialize boad library infoShow strings
*
* This routine creates strings containing the target cpu and core info.
*
* RETURNS: N/A
*/

LOCAL void sysBoardInfoStrInit (void)
    {
    char* pStr = (char*)(&(sysCpuId.brandString[0]));
    char* pTblStr;
    int tBlStrSize = 0;
    char buf[30];           /* temporary storage area */
    UINT64 freq;            /* temporary for TSC frequency*/
    UINT64 freqMhz;         /* temporary for TSC mega frequency*/
    UINT64 freqFraction;    /* temporary for TSC fraction frequency*/
    UINT64 freqRound;       /* temporary for TSC fraction frequency round*/


    bzero(sysBoardInfoStr, sizeof(sysBoardInfoStr));
    bzero(sysBoardImageStr, sizeof(sysBoardImageStr));

    /* Set MPTABLE type */

#if defined (INCLUDE_ACPI_BOOT_OP)
    pTblStr = " (ACPI)";
    tBlStrSize = 7;
#elif defined (INCLUDE_MPTABLE_BOOT_OP)
    pTblStr = " (MPTABLE)";
    tBlStrSize = 10;
#elif defined (INCLUDE_USR_BOOT_OP)
    pTblStr = " (USRTABLE)";
    tBlStrSize = 11;
#else
    pTblStr = "";
    tBlStrSize = 0;
#endif

    /* use brandstring as CPU info string */

    if (*pStr != 0)
        {
        char *dest = sysBoardInfoStr;
        char *src = sysBoardInfoStr;

        /* skip space and then copy */

        while (*pStr != 0 && isspace ((int) (*pStr))) pStr++;
        (void) snprintf (sysBoardInfoStr, sizeof (sysBoardInfoStr), pStr);

        /* replace multiple spaces in CPUID brandString with 1 space */

        while (*src != 0)
            {
            while (*src == ' ' && *(src + 1) == ' ')
                src++;
            *dest++ = *src++;
            }
        *dest = '\0';

        }
    else /* in case the sysCpuId contains no brandstring (before Pentium4) */
        {
        (void) snprintf (sysBoardInfoStr,
                         sizeof (sysBoardInfoStr),
                         "(CPUID brandString not available)");
        }

    /* add calibrated TSC to sysBoardInfoStr */

    /*
     * The default flag of compiler for kernel source code is set to not using 
     * any hardware float instruction for kernel source, but when the compiler 
     * encounters the float section in C code, it still creates the hardware float 
     * instruction for it and pop a warning message to user. To avoid this warning 
     * we use the integer arithmetic to simulate the float here.
     */
    freq = sysGetTSCCountPerSec();
    freqMhz = freq / 1000000;
    freqFraction = (freq % 1000000) / 10000;
    freqRound = ((freq % 10000) / 1000 >= 5) ? 1ull : 0ull;
    (void) snprintf(buf, sizeof (buf), " (TSC:%llu.%02llu MHz)", freqMhz,(freqFraction+freqRound));

    strncat(sysBoardInfoStr, buf, strlen(buf));

    /* add pTblStr to configured sysModel string */

    (void) snprintf (sysBoardImageStr, sizeof (sysBoardImageStr), SYS_MODEL);
    strncat(sysBoardImageStr, pTblStr, tBlStrSize);

    /* add hardware available CPUs */

    (void) snprintf(buf, sizeof (buf), ", CPUs available %d", vxCpuCount);
    strncat(sysBoardInfoStr, buf, strlen(buf));

    /* add software configured CPUs */

    (void) snprintf(buf, sizeof (buf), ", CPUs configured %d",
                    vxCpuConfiguredGet());
    strncat(sysBoardImageStr, buf, strlen(buf));

    /* indicate strings are up to date. Will save time if called again */

    sysBoardInfoStrInitialized = TRUE;
    }

/******************************************************************************
*
* itl64SysInfoShow - display the target cpu and core info
*
* This routine displays the target cpu and core info.
* It is called via a registered pointer to boardInfoShow.
*
* RETURNS: N/A
*/

LOCAL void itl64SysInfoShow (void)
    {
    /*
     * Initialize string to display, if not done yet.
     * This ensures we fill in the strings only once to speed
     * up multiple calls to boardInfoShow.
     */

    if (sysBoardInfoStrInitialized == FALSE)
        sysBoardInfoStrInit();

    /* Display boardInfoShow string */

    (void) printf("Board: %s\n", sysBoardInfoStr);
    (void) printf("Image: %s\n", sysBoardImageStr);
    }

#endif /* INCLUDE_BOARD_INFO_SHOW */

/*******************************************************************************
*
* itl64SysToMonitor - transfer control to the ROM monitor
*
* This routine transfers control to the ROM monitor.  It is usually called
* only by reboot() -- which services ^X -- and by bus errors at interrupt
* level.  However, in some circumstances, the user may wish to introduce a
* new <startType> to enable special boot ROM facilities.
*
* RETURNS: Does not return.
*/

LOCAL  void  itl64SysToMonitor
    (
    int startType   /* passed to ROM to tell it how to boot */
    )
    {
    FUNCPTR pEntry;
#if defined(INCLUDE_MULTI_STAGE_BOOT)
#if defined(INCLUDE_BOOT_APP)
    UINT    pageState;
    BOOL    reloadSecondStage = FALSE;
#endif /* INCLUDE_BOOT_APP */
#if defined(INCLUDE_FAST_REBOOT) && (!defined(INCLUDE_BOOT_APP))
    BOOL    reloadSecondStage = TRUE;
#endif /* INCLUDE_FAST_REBOOT && !INCLUDE_BOOT_APP */
#endif /* INCLUDE_MULTI_STAGE_BOOT */

#if (defined _WRS_CONFIG_PWR_MGMT) && (defined INCLUDE_DEV_PWR_MGMT)
    {
    PWR_DEVICE_ID pwrDeviceId;
    UINT32 state;

    /*
     * In order to reboot via WARM or FAST reboots, we need all devices
     * in a D0 state (from hardware only point of view) before reboot.
     * This implies that device registers are visible and ready for
     * programming. This does not mean the device have been setup by
     * software to a useable or functionally active state yet. Registers
     * need to be accessible for read/write operations so that the
     * device can be configured when we jump to the bootrom.
     *
     * D0 un-initialized state conforms to above mentioned requirements.
     * This state is used instead of the regular D0 to keep WARM/FAST
     * reboot times short as possible.
     */

    state = WRS_D0_UNINITIALIZED;

    pwrDeviceId = pwrDeviceRootGet();

    /* Transition domain and children to requested state */

    pwrDeviceStateSet (pwrDeviceId->pName, pwrDeviceId->unitNumber, state);

    /* Transition Orphan Devices back to D0 state */

    if (pwrOrphanDeviceSetD0 != NULL)
        pwrOrphanDeviceSetD0();

    }
#endif /* (_WRS_CONFIG_PWR_MGMT) && (INCLUDE_DEV_PWR_MGMT) */

    BOOL enable = FALSE;

#ifdef _WRS_CONFIG_SMP
    ULONG sysShutdownLoopCounter;
#endif /* _WRS_CONFIG_SMP */

    (void) intCpuLock();        /* LOCK INTERRUPTS */

#if (((!defined(INCLUDE_FAST_REBOOT)) &&                                \
     (!defined(INCLUDE_MULTI_STAGE_WARM_REBOOT)) &&                     \
     (!defined(INCLUDE_WARM_BOOT)) &&                                   \
     defined(INCLUDE_MULTI_STAGE_BOOT)) ||                              \
    defined(INCLUDE_HYPERVISOR_HVIF))

    startType = BOOT_COLD;
#endif

#if defined(GRUB_MULTIBOOT) && \
    (!(defined(INCLUDE_WARM_BOOT)))

    /*
     * The GRUB vxWorks image does not support "warm" reboot.
     * Do cold reboots if booted via multiboot.
     */

    if ((*(UINT32 *)(MULTIBOOT_SCRATCH|KERNEL_DIRECT_MAP_RGN_BASE) ==
         MULTIBOOT_BOOTLOADER_MAGIC))
         startType = BOOT_COLD;
#endif

#ifdef _WRS_CONFIG_SMP

    /*
     * At this point we are either the bootstrap processor,
     * or an application processor...
     *
     * ...need to reset APs and go out on the bootstrap processor...
     *
     * generate IPI to have core run sysToMonitor...
     */

    if (vxCpuIndexGet() != 0)
        {
        cpuset_t cpuset;
        CPUSET_ZERO (cpuset);
        CPUSET_SET  (cpuset, 0);  /* CPU 0 */

        /*
         * If code reached here, it means that a reboot is requested
         * and code is executing on an AP core.  Notify core0 (BP) to
         * run sysToMonitor and wait until core0 sends an IPI to shut
         * down the APs and execute the reboot.
         */

        (void) taskCpuLock();   /* Prevent task migration */

        cpcInvoke (cpuset, (CPC_FUNC)sysToMonitor, (void *)((ULONG)startType),
                   sizeof(startType), VX_CPC_ASYNC);

        for ( sysShutdownLoopCounter = 0; ; sysShutdownLoopCounter++)
            sysDelay();
        }

    /* Place APs in reset */

    (void) sysCpuReset_APs ();
    sysDelay();

    /* now running on BSP, APs in reset... */

    /* Map scratch memory area */

    if (vmStateSet((VM_CONTEXT_ID)NULL,
                   (VIRT_ADDR)0UL,
                   ROUND_UP((CPU_SCRATCH_POINT+CPU_AP_AREA), VM_PAGE_SIZE),
                   MMU_ATTR_VALID_MSK,
                   MMU_ATTR_VALID) == ERROR)
        goto sysToMonitorCold;

#endif /* _WRS_CONFIG_SMP */

#if defined(INCLUDE_MULTI_STAGE_BOOT) && defined(INCLUDE_BOOT_APP)

    /*
     * For bootApp, check whether vxWorks image was loaded to bootApp
     * 2nd stage area. ROM_TEXT_ADRS memory area becomes
     * MMU_ATTR_VALID in such a case.  Need to reload bootApp 2nd
     * stage if vxWorks image was loaded.
     */

    if (vmStateGet((VM_CONTEXT_ID)NULL,
                   (VIRT_ADDR)ROUND_DOWN(ROM_TEXT_ADRS, VM_PAGE_SIZE),
                   &pageState) != ERROR)
        {
        if ((pageState & MMU_ATTR_VALID_MSK) == MMU_ATTR_VALID)
            {
            reloadSecondStage = TRUE;
            }
        }
    else
        {
        goto sysToMonitorCold;
        }

#endif /* INCLUDE_MULTI_STAGE_BOOT && INCLUDE_BOOT_APP */

    /* Map bootrom area */

    if (vmStateSet((VM_CONTEXT_ID)NULL,
                   (VIRT_ADDR)ROUND_DOWN(ROM_TEXT_ADRS, VM_PAGE_SIZE),
                   ROUND_UP(ROM_SIZE, VM_PAGE_SIZE),
                   MMU_ATTR_VALID_MSK,
                   MMU_ATTR_VALID) == ERROR)
        goto sysToMonitorCold;

#ifdef _WRS_CONFIG_SMP

    /* restore trampoline code space for warm reboot */

    bcopy ((char *)(&tramp_code[0]),
           (char *)(CPU_ENTRY_POINT),
           (int)CPU_AP_AREA);

#endif /* _WRS_CONFIG_SMP */

    /* If this is a cold reboot */

    if (startType == BOOT_COLD)
        goto sysToMonitorCold;

#if defined(INCLUDE_MULTI_STAGE_BOOT) && \
    (!(defined(INCLUDE_BOOT_APP))) && \
    (!(defined(INCLUDE_BOOT_RAM_IMAGE))) && \
    (!(defined(INCLUDE_WARM_BOOT)))
    pEntry = (FUNCPTR)(FIRST_TEXT_ADRS + ROM_WARM_HIGH);
#elif defined(INCLUDE_WARM_BOOT) 
    IMPORT void        warmBootEntry();

    /*
     * If using warm boot set entry to jump over the cold 
     * boot normal entry point
     */

    pEntry = (FUNCPTR)(warmBootEntry);

    if (sysWarmBootFunc == NULL)
        goto sysToMonitorCold;
 
#else /* INCLUDE_MULTI_STAGE_BOOT */
#if defined(INCLUDE_MULTI_STAGE_BOOT) && defined(INCLUDE_BOOT_APP)
    if (reloadSecondStage == TRUE)
        pEntry = (FUNCPTR)(FIRST_TEXT_ADRS + ROM_WARM_HIGH);
    else
#endif /* INCLUDE_MULTI_STAGE_BOOT && INCLUDE_BOOT_APP */
        pEntry = (FUNCPTR)(ROM_TEXT_ADRS + ROM_WARM_HIGH);
#endif /* INCLUDE_MULTI_STAGE_BOOT */

#if defined(INCLUDE_MULTI_STAGE_BOOT) &&                                \
    (defined(INCLUDE_BOOT_APP) ||                                       \
     (defined(INCLUDE_FAST_REBOOT) && (!defined(INCLUDE_BOOT_APP))))

    /* restore the saved brand new bootrom image, then jump */

    if (reloadSecondStage == TRUE)
        {
        if ((memRom > 0) && (memRomSize > 0))
            {

            /* Map saved bootrom area */

            if (vmStateSet((VM_CONTEXT_ID)NULL,
                           (VIRT_ADDR)ROUND_DOWN(BOOT_MULTI_STAGE_DATA,
                                                 VM_PAGE_SIZE),
                           ROUND_UP(BOOT_MULTI_STAGE_DATA_SIZE, VM_PAGE_SIZE),
                           MMU_ATTR_VALID_MSK,
                           MMU_ATTR_VALID) == ERROR)
                goto sysToMonitorCold;

            if (vmStateSet((VM_CONTEXT_ID)NULL,
                           (VIRT_ADDR)memRom, memRomSize,
                           MMU_ATTR_VALID_MSK,
                           MMU_ATTR_VALID) == ERROR)
                goto sysToMonitorCold;

            if (*(UINT16 *)((ULONG)memRom + (*(UINT32 *)BOOT_IMAGE_SIZE) - 2) !=
                checksum ((UINT16 *)((ULONG)memRom),
                          (*(UINT32 *)BOOT_IMAGE_SIZE) - 2))
                {

                /* Don't do fast reboot if checksum is not right */

                *(UINT32 *)BOOT_IMAGE_ADRS = 0;
                *(UINT32 *)BOOT_IMAGE_SIZE = 0;
                goto sysToMonitorCold;

                }
            }
        else
            {
            goto sysToMonitorCold;
            }
        }

#endif  /* INCLUDE_MULTI_STAGE_BOOT && ... */

    /* warm start entry point */

    sysClkDisable ();   /* disable the system clock interrupt */
    sysIntLock ();      /* lock the used/owned interrupts */

    /* Shutdown Local APIC */

    (void) loApicIntrEnable (&enable);

    /* Restore Local APIC initial settings prior to enable */

#ifdef  _LOAPIC_RESTRICTED_ACCESS_INSNS
#define _LOAPIC_BASE_ADRS glbLoApicBaseAdrs

    _LOAPIC_WRITE (0x0f0, glbLoApicOldSvr);     /* LOAPIC_SVR */
    _LOAPIC_WRITE (0x350, glbLoApicOldLint0);   /* LOAPIC_LINT0 */
    _LOAPIC_WRITE (0x360, glbLoApicOldLint1);   /* LOAPIC_LINT1 */

#else   /* _LOAPIC_RESTRICTED_ACCESS_INSNS */

    *(volatile int *)(glbLoApicBaseAdrs + 0x0f0) = /* LOAPIC_SVR */
        (int)glbLoApicOldSvr;
    *(volatile int *)(glbLoApicBaseAdrs + 0x350) = /* LOAPIC_LINT0 */
        (int)glbLoApicOldLint0;
    *(volatile int *)(glbLoApicBaseAdrs + 0x360) = /* LOAPIC_LINT1 */
        (int)glbLoApicOldLint1;

#endif  /* _LOAPIC_RESTRICTED_ACCESS_INSNS */

#if defined(INCLUDE_PC_CONSOLE) && defined(INCLUDE_LEGACY_USB_INT_MODE)

    /* Turn off legacy SMI USB interrupt mode for warm reboot */

    sysSmiUsbLegacyOff();
#endif /* INCLUDE_PC_CONSOLE */

#if defined(INCLUDE_MULTI_STAGE_BOOT) && \
    (!(defined(INCLUDE_WARM_BOOT))) && \
    (!(defined(INCLUDE_BOOT_APP))) && \
    (!(defined(INCLUDE_BOOT_RAM_IMAGE)))
    sysBootElf32((UINT32)((ULONG)pEntry),startType);
#elif defined (INCLUDE_WARM_BOOT)
    sysWarmBootFunc(pEntry);
#else /* INCLUDE_MULTI_STAGE_BOOT */
#if defined(INCLUDE_MULTI_STAGE_BOOT) && defined(INCLUDE_BOOT_APP)
    if (reloadSecondStage == TRUE)
        sysBootElf32((UINT32)((ULONG)pEntry),startType);
    else
#endif /* INCLUDE_MULTI_STAGE_BOOT && INCLUDE_BOOT_APP */
        (*pEntry) (startType);
#endif /* INCLUDE_MULTI_STAGE_BOOT */

sysToMonitorCold:

    /* perform the cold boot since the warm boot is not possible */

    sysClkDisable ();

    /* ICH9,ICH10 Reset Control Register, Reset CPU and System Reset Register */

    sysOutByte(RST_CNT_REG, RST_CNT_FULL_RST | RST_CNT_CPU_RST |
               RST_CNT_SYS_RST);
    }


/*******************************************************************************
*
* itl64SysUsDelay - delay specified number of microseconds
*
* This routine provides a delay, based on TSC.
*
* RETURNS: N/A
*/

LOCAL void itl64SysUsDelay
    (
    int uSec
    )
    {
    pentiumTSCDelay(uSec);
    }


/*******************************************************************************
* sys1msDelay - Delay 1 ms
*
* _vxb_delayRtn needs to point to a routine that takes exactly 1ms
* NOTE: _vxb_delayRtn must be set, else the vxbTimerLib.c will overwrite
* vxbUsDelay and vxbMsDelay function pointers and replace them with the
* software default pointers.
*
* RETURNS: N/A
*/

void sys1msDelay (void)
    {
    sysUsDelay(1000);
    }

#ifdef _WRS_CONFIG_SMP

/******************************************************************************
*
* sysCpuAvailableGet - return the number of CPUs that are to
*                      be used by vxWorks for the SMP kernel.
*
* RETURNS: The number for the CPUs available
*
*/

LOCAL UINT32 itl64SysCpuAvailableGet
    (
    void
    )
    {
    return vxCpuCount;
    }

#endif /* _WRS_CONFIG_SMP */


#ifdef _WRS_CONFIG_SMP

/******************************************************************************
*
* sysCpuEnable - enable a multi core CPU
*
* This routine brings a CPU out of reset
*
* RETURNS:  OK - Success
*           ERROR - Fail
*
*/

LOCAL STATUS itl64SysCpuEnable
    (
    unsigned int cpuNum,
    WIND_CPU_STATE *cpuState
    )
    {
    if ((cpuNum < 1) || (cpuNum > (vxCpuCount - 1)))
        {
        return (ERROR);
        }

    sysCpuInitTable[cpuNum] = (FUNCPTR) cpuState->regs.pc;
    return (sysCpuStart(cpuNum, cpuState));
    }

#endif /* _WRS_CONFIG_SMP */

/******************************************************************************
*
* itlProbe - probe routine for Board Lib arch board probe
*
* RETURNS:  TRUE
*
*/

LOCAL BOOL itlProbe
    (
    char * compat
    )
    {
    return TRUE;
    }

