/* pc.h - PC common header */

/*
 * Copyright (c) 2014-2015 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

/*
modification history
--------------------
30jan15,scm  V7PRO-1656 multibootShow initialization issue,
             V7PRO-1542 page fault on reboot command
17dec14,j_l  V7PRO-1506 - i8259Init() in symmetric I/O mode
09sep14,rec  Merge from itl_64 - Update MMU macros so MMU table can be moved.
13aug14,ten  US44071 - Merge multiboot changes from itl_64 BSP:
                       US38341, V7PRO-891
12aug14,scm  US43452 - add PCI related sysSmi support definitions
29jul14,jmz  V7PRO-1083 - Move PCI defines to vxbIaPciBus.h
27mar14,j_l  US30676 - remove config.h
06mar14,j_l  add defines from old configAll.h.
04mar14,scm  TA42438 - add 16550 support for IA
03feb14,j_l  US23032 - IA BSP with VxBus GEN2 core library
31jan14,j_l  initial creation based on itl_64 version 04oct13
*/

/*
This file contains IO address, memory address, and related constants
for Intel based BSPs.
*/

#ifndef INCpch
#define INCpch

#include <vsbConfig.h>

#include <drv/parallel/lptDrv.h>

#ifdef INCLUDE_MMU_BASIC
#include <arch/i86/x86_64/mmuX86_64Lib.h>
#endif /* INCLUDE_MMU_BASIC */

#include <arch/i86/ipiI86Lib.h>

#ifdef __cplusplus
extern "C" {
#endif

/* The following section is from old configAll.h */

#if     (_STACK_DIR == _STACK_GROWS_DOWN)
#ifdef  ROM_RESIDENT
#define STACK_ADRS      STACK_RESIDENT
#else
#define STACK_ADRS      _romInit
#endif  /* ROM_RESIDENT */
#else   /* _STACK_DIR == _STACK_GROWS_UP */
#ifdef  ROM_RESIDENT
#define STACK_ADRS      (STACK_RESIDENT-STACK_SAVE)
#else
#define STACK_ADRS      (_romInit-STACK_SAVE)
#endif  /*  ROM_RESIDENT */
#endif  /* (_STACK_DIR == _STACK_GROWS_DOWN) */

#define BUS             BUS_TYPE_PCI    /* PCI Bus + ISA/VME */

#define BOOTCODE_IN_RAM                 /* for ../all/bootInit.c */

#ifdef GRUB_MULTIBOOT
#define MULTIBOOT_HEADER_MAGIC         0x1BADB002
#define MULTIBOOT_BOOTLOADER_MAGIC     0x2BADB002
#define MULTIBOOT_HEADER_FLAGS_ELF     0x00000003
#define MULTIBOOT_HEADER_FLAGS         0x00010003
#define MULTIBOOT_SCRATCH              0x00105a10
#endif /* GRUB_MULTIBOOT */

#ifdef  BOOTCODE_IN_RAM
#undef  ROMSTART_BOOT_CLEAR
#endif

#define ROM_ADRS(x)     (ROM_TEXT_ADRS + FUNC(x) - FUNC(romInit))

/* PCI related legacy USB SMI definitions */

#define LPC_PMBASE_MASK     0x0000ff80 /* PMBASE address mask */
#define LPC_PMBASE_OFFSET   0x40       /* PMBASE offset address */

#define LPC_SMI_EN_OFFSET   0x30       /* SMI_EN i/o address offset */
#define LPC_GBL_SMI         0x0001     /* SMI_EN: SMI interrupt enable */
#define LPC_LEGACY_USB      0x0008     /* SMI_EN: legacy USB SMI int enable */

#define LPC_BUSNO           0x00
#define LPC_DEVNO           0x1f
#define LPC_FUNCNO          0x00

#define IA32_EFER       0xc0000080

#define IA32_EFER_SCE   0x00000001    /* SysCall enable (bit 0) */
#define IA32_EFER_LME   0x00000100    /* IA-32e mode enable (bit 8) */
#define IA32_EFER_LMA   0x00000400    /* IA-32e mode active (bit 10) */
#define IA32_EFER_NX    0x00000800    /* Execute disable bit enable (bit 11) */

#define CR0_ET          0x00000010

#define CR0_INIT_VALUE  (CR0_PE | CR0_MP | CR0_ET | CR0_NE | CR0_WP | \
                         CR0_AM | CR0_PG)

#define CODE_SEGMENT    0x08
#define DATA_SEGMENT    0x10

#define STATIC_MMU_TBL_PAGE_ATTR_OFF         (0x83 | 0x18)
#define STATIC_MMU_TBL_PAGE_ATTR_WT          (0x83 | 0x08)
#define STATIC_MMU_TBL_PAGE_ATTR_CB          (0x83 | 0x00)
#define STATIC_MMU_TBL_PAGE_ATTR_READONLY    (0x81 | 0x18)
#define STATIC_MMU_TBL_PAGE_ATTR_INVALID     (0x80 | 0x18)
#define STATIC_MMU_TBL_PAGE_ATTR_CB_4K       (0x03 | 0x00)
#define STATIC_MMU_TBL_PAGE_ATTR_READONLY_4K (0x01 | 0x18)
#define STATIC_MMU_TBL_PAGE_ATTR_INVALID_4K  (0x00 | 0x18)
#define STATIC_MMU_TBL_PAGE_ATTR_RAM         STATIC_MMU_TBL_PAGE_ATTR_CB
#define STATIC_MMU_TBL_PAGE_ATTR_IO          STATIC_MMU_TBL_PAGE_ATTR_OFF
#define STATIC_MMU_TBL_PAGE_ATTR_PROTECTED   STATIC_MMU_TBL_PAGE_ATTR_READONLY
#define STATIC_MMU_TBL_PAGE_ATTR_RAM_4K      STATIC_MMU_TBL_PAGE_ATTR_CB_4K
#define STATIC_MMU_TBL_PAGE_ATTR_PROTECTED_4K \
                                             STATIC_MMU_TBL_PAGE_ATTR_INVALID_4K

/* Static MMU defines */

#define PAGE_ENTRIES_MAX 512
#define PAGE_OFFSET_PD   21                 /* PD offset bits 21-29 */
#define PAGE_OFFSET      PAGE_SIZE_4KB

#define PAGE_P           0x0000000000000001 /* Present bit */
#define PAGE_RW          0x0000000000000002 /* Read/Write bit */
#define PAGE_US          0x0000000000000004 /* User/Supervisor bit */

#define PAGE_PML4_NAME(label) page_table_ ## label

#define PAGE_BEGIN_MMU_VARS \
        $page = 0;

#define PAGE_BEGIN_PT_VARS \
        index = 0;
#define PAGE_BEGIN_PT(label,base) \
        .balign PAGE_OFFSET ; \
        pd_ ## label = .;
#define PAGE_CREATE_PT(label,properties) \
        .quad   pt_ ## label | properties; \
        index = index + 1;
#define PAGE_CREATE_PTS(count,properties) \
        .rept count; \
        .quad   ((index << PAGE_OFFSET_PD) | properties); \
        index = index + 1; \
        .endr;

#define PAGE_BEGIN_PD_VARS \
        index = 0;
#define PAGE_BEGIN_PD(label,base) \
        .balign PAGE_OFFSET; \
        pdp_ ## label = .;

#define PAGE_CREATE_PD(label,properties) \
        .quad   pd_ ## label | properties; \
        index = index + 1;
#define PAGE_CREATE_PDS(count,properties) \
        .rept count; \
        .quad   ((index << PAGE_OFFSET_PD) | properties); \
        index = index + 1; \
        .endr;

#define PAGE_BEGIN_PDP(label,base) \
        index = 0; \
	.balign PAGE_OFFSET; \
        pml4_ ## label = .;
#define PAGE_CREATE_PDP(label,properties) \
        .quad   pdp_ ## label + properties; \
        index = index + 1;

#define PAGE_PML4_BEGIN(label) \
        index = 0; \
        .balign PAGE_OFFSET; \
PAGE_PML4_NAME(label):;
#define PAGE_CREATE_PML4(label,properties) \
        .quad   pml4_ ## label + properties; \
        index = index + 1;

#define PAGE_CREATE_BLANK(count) \
        .fill   count,8,0; \
        index = index + count;
#define PAGE_END \
        .fill   PAGE_ENTRIES_MAX-index,8,0;
#define PAGE_END_NO_FILL

/* programmable interrupt controller (PIC) */

#define PIC1_BASE_ADR           0x20
#define PIC2_BASE_ADR           0xa0
#define PIC_REG_ADDR_INTERVAL   1       /* address diff of adjacent regs. */

/*
 * This maps a system vector to a PCI irq number, in the range
 * 0 to (PCI_INT_LINES - 1). The output is used as an index
 * into the array of linked lists used for sharing.  Used by
 * vxbPci.c, which is compiled from source.
 */

#undef PCI_INT_VECTOR_TO_IRQ
#define PCI_INT_VECTOR_TO_IRQ(vector)   (sysPciIvecToIrq((_Vx_usr_arg_t)vector))

#ifndef _ASMLANGUAGE
IMPORT int sysPciIvecToIrq (_Vx_usr_arg_t vector);
#endif  /* _ASMLANGUAGE */

#ifndef _WRS_CONFIG_SMP

/* Multi processor (AMP) definitions */

#define MP_ANCHOR_ADRS          0x1400      /* base addr for the counters */
#define MP_N_CPU        (MP_ANCHOR_ADRS)    /* MP CPU counter */
#define MP_LOCK_SEM     (MP_ANCHOR_ADRS+4)  /* MP lock semaphore */
#define MP_N_IPI        (MP_ANCHOR_ADRS+8)  /* MP IPI counter */
#define MP_BOOT_TIMEOUT         4           /* timeout (sec) for AP booting */
#define MP_VACANT       0x00000000          /* MP lock sem - vacant */
#define MP_VACANT_NOT   0xffffffff          /* MP lock sem - vacant not */
#define MP_BP                   1           /* build macro for BP */
#define MP_AP1                  2           /* build macro for AP1 */
#define MP_AP2                  3           /* build macro for AP2 */
#define MP_AP3                  4           /* build macro for AP3 */
#define WARM_RESET_VECTOR       0x0467      /* BIOS: addr to hold the vector */
#define BIOS_SHUTDOWN_STATUS    0x0f        /* BIOS: shutdown status */

#else

/* Multi processor (SMP) definitions */

#define MP_ANCHOR_ADRS          0x1400      /* base addr for the counters */
#define WARM_RESET_VECTOR       0x0467      /* BIOS: addr to hold the vector */
#define BIOS_SHUTDOWN_STATUS    0x0f        /* BIOS: shutdown status */
#endif /* !_WRS_CONFIG_SMP */

/* Reset control register definitions */

#define RST_CNT_REG             0x0cf9      /* Reset Control Register */
#define RST_CNT_SYS_RST         0x02        /* System Reset bit */
#define RST_CNT_CPU_RST         0x04        /* Reset CPU bit */
#define RST_CNT_FULL_RST        0x08        /* Full Reset bit */

#ifdef _WRS_CONFIG_SMP

/* SMP AP specific defines... */

/*
 * NOTE: if sysInitCpuStartup changes the value of CPU_STARTUP_SIZE
 *       will need to be recalculated...
 */

/* size of sysInitCpuStartup */

#ifdef _ASMLANGUAGE
#define CPU_STARTUP_SIZE    (sysInitCpuStartupEnd - sysInitCpuStartup)
#else
#define CPU_STARTUP_SIZE    ((char *)sysInitCpuStartupEnd -     \
                             (char *)sysInitCpuStartup)
#endif /* _ASMLANGUAGE */

#define CPU_ENTRY_POINT     0xC000  /* cold start code location for APs */

/* start of scratch memory */

#define CPU_SCRATCH_START   (((CPU_STARTUP_SIZE) + (0x100 - 1)) & ~(0x100 - 1))

#define CPU_AP_AREA         0x200   /* work area for APs */

/* offset for scratch memory */

#define CPU_SCRATCH_POINT   (CPU_ENTRY_POINT + CPU_SCRATCH_START)

#define NUM_GDT_ENTRIES     0x0A

#define CPU_SCRATCH_GDT     0x2E
#define CPU_SCRATCH_IDT     0x3A

#define LIM_GDT_OFFSET      0x2C
#define BASE_GDT_OFFSET     0x30
#define CR3_OFFSET          0x34
#define LIM_IDT_OFFSET      0x38
#define BASE_IDT_OFFSET     0x3C
#define AP_STK_OFFSET       0x44

#define CPU_INIT_START_ADR  (MP_ANCHOR_ADRS + 0x10)

#endif /* _WRS_CONFIG_SMP */

/* Warm boot (reboot) devices and parameters */

#define SYS_WARM_BIOS       0   /* warm start from BIOS */
#define SYS_WARM_USB        1   /* warm start from USB */
#define SYS_WARM_ICH        2   /* warm start from ICH(IDE) */
#define SYS_WARM_AHCI       3   /* warm start from AHCI */

/* IDT entry type options */

#define SYS_INT_TRAPGATE    0x0000ef00  /* trap gate */
#define SYS_INT_INTGATE     0x0000ee00  /* int gate */

#define LOCAL_MEM_VIRT_TO_PHYS(addr)    ((addr) & (LOCAL_MEM_PHYS_ADRS_MASK))

#if defined(INCLUDE_BOOT_APP)
/* This section for project built bootApp */

/* VxWorks image entry point */

#define SYS_RAM_LOW_ADRS        LOCAL_MEM_VIRT_TO_PHYS(RAM_HIGH_ADRS)

/* Boot image entry point */

#define SYS_RAM_HIGH_ADRS       LOCAL_MEM_VIRT_TO_PHYS(RAM_LOW_ADRS)
#else
/* VxWorks image entry point */

#define SYS_RAM_LOW_ADRS        LOCAL_MEM_VIRT_TO_PHYS(RAM_LOW_ADRS)

/* Boot image entry point */

#define SYS_RAM_HIGH_ADRS       LOCAL_MEM_VIRT_TO_PHYS(RAM_HIGH_ADRS)
#endif /* INCLUDE_BOOT_APP */

#define BOOT_IMAGE_ADRS_OFFSET       (BOOT_MULTI_STAGE_DATA_OFFSET)
#define BOOT_IMAGE_ADRS_OFFSET_SIZE  4
#define BOOT_IMAGE_SIZE_OFFSET       (BOOT_IMAGE_ADRS_OFFSET +          \
                                      BOOT_IMAGE_ADRS_OFFSET_SIZE)
#define BOOT_IMAGE_SIZE_OFFSET_SIZE  4
#define BOOT_MULTI_STAGE_DATA_SIZE   (BOOT_IMAGE_SIZE_OFFSET +          \
                                      BOOT_IMAGE_SIZE_OFFSET_SIZE       \
                                      - BOOT_MULTI_STAGE_DATA_OFFSET)

#define BOOT_MULTI_STAGE_DATA        LOCAL_MEM_VIRT_TO_PHYS(    \
    LOCAL_MEM_LOCAL_ADRS + BOOT_MULTI_STAGE_DATA_OFFSET)
#define BOOT_IMAGE_ADRS              LOCAL_MEM_VIRT_TO_PHYS(    \
    LOCAL_MEM_LOCAL_ADRS + BOOT_IMAGE_ADRS_OFFSET)
#define BOOT_IMAGE_SIZE              LOCAL_MEM_VIRT_TO_PHYS(    \
    LOCAL_MEM_LOCAL_ADRS + BOOT_IMAGE_SIZE_OFFSET)

#ifdef __cplusplus
}
#endif

#include "configInum.h"

#include "prjParams.h"

#endif  /* INCpch */
