/* sysDevPwr.c - Device power management support */

/*
 * Copyright (c) 2014 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

/*
modification history
--------------------
31jan14,j_l  initial creation based on itl_64 version 05sep13
*/

/*
DESCRIPTION
This file provides device definition for power managed devices.
*/

#include <vxWorks.h>
#include <vsbConfig.h>
#include <vxBusLib.h>
#include <hwif/pwr/pwrDeviceLib.h>
#include <hwif/pwr/pwrOrphanPciDevices.h>

/* BSP configuration entries */

PWR_DEVICE_CONFIG defaultConfigEntry = {
              "DEFAULT",          /* device name */
              WILDCARD,           /* unit number */
              DEVICE,             /* DEVICE or DOMAIN */
              0,                  /* not used */
              0,                  /* power down when children reach D3 */
              WRS_D0_STATES,      /* flags indicate states supported */
              WRS_D0_STATES,      /* flags indicate states enabled */
              1,                  /* number of parent domains */
              0,                  /* number of callout functions */
              NULL,               /* no callout functions */
              WAIT_FOREVER,       /* utilization timeout (ticks) */
              };

PWR_DEVICE_CONFIG *pDefaultConfigEntry = &defaultConfigEntry;

PWR_DEVICE_HIERARCHY pwrDomain0 [] = {
  {"ROOT", 0}, HIERARCY_END};

PWR_DEVICE_HIERARCHY *ppSysPwrDeviceHierarchy[] = {
          pwrDomain0, NULL};

/*
 * This is an array of power device configuration records.  Each provide
 * initialization parameters forstatic devices (known at initialization).
 */


PWR_DEVICE_CONFIG pwrRootCfg = {
              "ROOT",             /* device name */
              0,                  /* unit number */
              DOMAIN,             /* DEVICE or DOMAIN */
              INDEPENDENT,        /* children can transition independently */
              3,                  /* power down when children reach D3 */
              WRS_D0D3_STATES,    /* flags indicate states supported */
              WRS_D0D3_STATES,    /* flags indicate states enabled */
              0,                  /* number of parent domains */
              0,                  /* number of callout functions */
              NULL,               /* no callout functions */
              WAIT_FOREVER        /* utilization timeout (ticks) */
      };

PWR_DEVICE_CONFIG *pSysStaticConfigTable [] = {
      &pwrRootCfg,
      NULL
      };

/* dynamic devices */

#include <hwif/pwrDynamicDevices.c>

/*
 * This is a list of orphan PCI devices in the system
 * that will be turned off by setting to D3 state
 */

#define INTEL_VENDORID   0x8086
#define ATI_VENDORID     0x1002
#define MATROX_VENDORID  0x102B

PWR_ORPHAN_PCI_DEVICE sysPwrOrphanPciDeviceList[] = {

    /* {pDev, vendorId, deviceId, pciConfig[]}, */

#ifdef INCLUDE_MATXM_CORE_411_WR
    {0, INTEL_VENDORID, 0x3B56, {0}},   /* High Definition Audio */
    {0, INTEL_VENDORID, 0x3B64, {0}},   /* HECI Controller */
    {0, INTEL_VENDORID, 0x3B67, {0}},   /* KT Controller */
#if (!defined INCLUDE_PC_CONSOLE)
    {0, INTEL_VENDORID, 0x0046, {0}},   /* Integrated Graphics Controller */
    {0, ATI_VENDORID, 0x7152, {0}},     /* Primary Graphics Controller */
    {0, ATI_VENDORID, 0x7172, {0}},     /* Secondary Graphics Controller */
    {0, MATROX_VENDORID, 0x2527, {0}},  /* MGA G550 AGP Graphics Controller */
#endif /* (!defined INCLUDE_PC_CONSOLE) */
#if ((!defined INCLUDE_DRV_STORAGE_PIIX) || (!defined INCLUDE_DRV_STORAGE_AHCI))
    {0, INTEL_VENDORID, 0x3B66, {0}},   /* PT IDER Controller */
    {0, INTEL_VENDORID, 0x3B2D, {0}},   /* 2 port SATA IDE Controller */
    {0, INTEL_VENDORID, 0x3B2E, {0}},   /* 4 port SATA IDE Controller */
#endif /* ((!defined INCLUDE_DRV_STORAGE_PIIX) || (!defined INCLUDE_DRV_STORAGE_AHCI)) */
#if (!defined INCLUDE_GEI825XX_VXB_END)
    {0, INTEL_VENDORID, 0x10EA, {0}},   /* On Board 82577LM Gigabit */
    {0, INTEL_VENDORID, 0x105E, {0}},   /* PCIe 2 Port GEI Card */
    {0, INTEL_VENDORID, 0x105E, {0}},
#endif /* (!defined INCLUDE_GEI825XX_VXB_END) */
#if (!defined INCLUDE_USB)
    {0, INTEL_VENDORID, 0x3B3C, {0}},   /* USB2 Controller */
    {0, INTEL_VENDORID, 0x3B34, {0}},   /* USB2 Controller */
#endif /* (!defined INCLUDE_USB) */
#endif /* INCLUDE_MATXM_CORE_411_WR */

#ifdef INCLUDE_GREENCITY
    {0, INTEL_VENDORID, 0x3A3E, {0}},   /* High Definition Audio */
#if (!defined INCLUDE_PC_CONSOLE)
    {0, ATI_VENDORID, 0x7152, {0}},     /* Primary Graphics Controller */
    {0, ATI_VENDORID, 0x7172, {0}},     /* Secondary Graphics Controller */
    {0, MATROX_VENDORID, 0x2527, {0}},  /* MGA G550 AGP Graphics Controller */
#endif /* (!defined INCLUDE_PC_CONSOLE) */
#if ((!defined INCLUDE_DRV_STORAGE_PIIX) || (!defined INCLUDE_DRV_STORAGE_AHCI))
    {0, INTEL_VENDORID, 0x3A22, {0}},   /* SATA AHCI Controller */
#endif /* ((!defined INCLUDE_DRV_STORAGE_PIIX) || (!defined INCLUDE_DRV_STORAGE_AHCI)) */
#if (!defined INCLUDE_GEI825XX_VXB_END)
    {0, INTEL_VENDORID, 0x10A7, {0}},   /* 82575EB Gigabit */
#endif /* (!defined INCLUDE_GEI825XX_VXB_END) */
#if (!defined INCLUDE_USB)
    {0, INTEL_VENDORID, 0x3A34, {0}},   /* USB UHCI Controller #1 */
    {0, INTEL_VENDORID, 0x3A35, {0}},   /* USB UHCI Controller #2 */
    {0, INTEL_VENDORID, 0x3A36, {0}},   /* USB UHCI Controller #3 */
    {0, INTEL_VENDORID, 0x3A37, {0}},   /* USB UHCI Controller #4 */
    {0, INTEL_VENDORID, 0x3A38, {0}},   /* USB UHCI Controller #5 */
    {0, INTEL_VENDORID, 0x3A39, {0}},   /* USB UHCI Controller #6 */
    {0, INTEL_VENDORID, 0x3A3A, {0}},   /* USB2 EHCI Controller #1 */
    {0, INTEL_VENDORID, 0x3A3C, {0}},   /* USB2 EHCI Controller #2 */
#endif /* (!defined INCLUDE_USB) */
#endif /* INCLUDE_GREENCITY */

#if ((defined INCLUDE_WHITETIP_MOUNTAIN) || \
     (defined INCLUDE_WALNUT_CANYON)     || \
     (defined INCLUDE_FLATHEAD_CREEK)    || \
     (defined INCLUDE_LAVA_CANYON))
    {0, INTEL_VENDORID, 0x0c0c, {0}},   /* MULTI_MEDIA */
    {0, INTEL_VENDORID, 0x8c3a, {0}},   /* COMMUNICATION */
    {0, INTEL_VENDORID, 0x8c20, {0}},   /* MULTI_MEDIA */
    {0, INTEL_VENDORID, 0x8c26, {0}},   /* SERIAL BUS */
    {0, INTEL_VENDORID, 0x8c2d, {0}},   /* SERIAL BUS */
#if (!defined INCLUDE_PC_CONSOLE)
    {0, INTEL_VENDORID, 0x0416, {0}},   /* DISP_CNTLR */
#endif /* !INCLUDE_PC_CONSOLE */
#endif /* INCLUDE_WHITETIP_MOUNTAIN || INCLUDE_WALNUT_CANYON || 
	* INCLUDE_FLATHEAD_CREEK || INCLUDE_LAVA_CANYON
	*/

    /* Add additional Orphan PCI Devices to turn off here */

};

UINT32 numPwrOrphanPciDevices = NELEMENTS(sysPwrOrphanPciDeviceList);
