/* sysLib.c - System-dependent library */

/*
 * Copyright (c) 2014-2017 Wind River Systems, Inc.
 *
 * The right to copy, distribute, modify, or otherwise make use
 * of this software may be licensed only pursuant to the terms
 * of an applicable Wind River license agreement.
 */

/*
modification history
--------------------
16dec16,cww  conditionalize fppArchCpuInit
13jan17,cfm  add X attributes for text regions (F7142)
11jul16,md   removed vmShutdown()
09sep15,jlv  add fast warm boot support. (F3387)
04sep15,j_l  V7PRO-2091 - add spurious interrupt handling for GRUB boots
19aug15,dot  V7PRO-2102 - fixed apigen errors and warnings.
29jun15,jjk  US54437 - Add support for Camelback Mountain 400, fix earlyPrint
19jun15,ebr  HYP-10581 - fixed compilation warning
14may15,scm  V7PRO-2061 - Fix typo in sysCpuStart
11may15,scm  V7PRO-2024 MMU ebdaSart is being unmapped
22apr15,j_l  V7PRO-1636 - sysUsDelay() busy waits with preemption locked in SMP
11mar15,scm  V7PRO-1804 - prevent rtp/sim rtp from passing back bad gs reg value
06mar15,dot  V7PRO-1648 - Remove dependency on INCLUDE_ACPI component when
             using INCLUDE_EARLY_PRINT. V7PRO-1830 - vxMpTablePrint routine
             contains text not enclosed in comments.
04feb15,to   V7PRO-1690 - fix SMP boot issue
06jan15,rec  HYP-10119 - use hvStatus_t for hypervisr return status
05jan15,ten  US43657 - Unify memory autosize components in Intel BSPs
17dec14,j_l  V7PRO-1506 - i8259Init() in symmetric I/O mode
04nov14,rbc  US44720 - Fix static analysis errors
09sep14,rec  Merge HV support from itl_64 BSP
14aug14,j_l  V7PRO-1152 - add sysSmiUsbLegacyOn() to sysHwInit2()
13aug14,ten  US44071 - Merge multiboot changes from itl_64 BSP:
                       US38341, V7PRO-891
12aug14,scm  US43452 - add back in sysSmi support routines
08aug14,ten  US41103 - Add generic profile to itl_64 gen2
08aug14,jmz  V7PRO-1144 - Register kwriteSerial with kprintfLib
21jul14,jmz  V7PRO-1047 - Move PCI stub functions to usrIaPciUtils.c
24jun14,jmz  US35979 - Remove unnecessary pentiumMcfgBaseAddress definition
17jun14,jmz  US35979 - Restore PCI stub functions
16jun14,jmz  US35979 - Removed unnecessary PCI stub functions
20may14,j_l  US22498 - Code clean up
06may14,jmz  US35979 - Move serial IO parameters to CDF
08may14,scm  US32988/TA40489 - Test/Debug/Validate IA GEN2 drivers, IPIs...
01may14,j_l  US37558 - Create IA configuration configlettes for
                       VxBus GEN2 drivers
29apr14,j_l  US37556 - Convert *MPT* names to *IADT*
23apr14,to   use inline assembly for LOAPIC memory mapped register access
             (US20864)
22apr24,jmz  US35979 Remove PCI BUSTYPE_DEF declaration
14apr14,scm  US24642/TA47728 Port I8253 Timer Driver to Gen2...
04apr14,scm  TA46620 Port IA Timestamp Timer Driver to Gen2...
27mar14,j_l  US30676 - remove config.h
21mar14,scm  TA42436 Port PC_CONSOLE to vxbus gen2 for IA
18mar14,j_l  US32795 - changes for new gnu compiler
06mar14,j_l  add iv.h include
04mar14,scm  TA42438 - add 16550 support for IA
10feb14,scm  TA31952 - Add GEN2 IPI Support...
05feb14,jmz  VXW7-1606 - Fix static analysis warnings
04feb14,scm  US22784 - 6.9.3.3 merge - Intel Arch phase #2...
04feb14,jjk  DE5023 - Do not duplicate source in VSB.
04feb14,j_l  US24659 - board library SMP support
04feb14,scm  US20759 - remove unsupported interrupt modes...
03feb14,j_l  US23032 - IA BSP with VxBus GEN2 core library
             US18115 - IA implementation of board system
31jan14,j_l  initial creation based on itl_64 version 22jan14
*/

/*
DESCRIPTION
This library provides board-specific routines.  The device configuration
modules and device drivers included are:

    i8253Timer.c - Intel 8253 timer driver
    ioApicIntr.c - Intel IO APIC/xAPIC driver
    ioApicIntrShow.c - Intel IO APIC/xAPIC driver show routines
    loApicIntr.c - Intel Pentium[234] Local APIC/xAPIC driver
    loApicIntrShow.c - Intel Local APIC/xAPIC driver show routines
    loApicTimer.c - Intel Pentium2/3/4 Local APIC timer library

INCLUDE FILES: sysLib.h

SEE ALSO:
.pG "Configuration"
*/

/* includes (header file) */

#include <vxWorks.h>
#include <vsbConfig.h>
#include <memLib.h>
#include <sysLib.h>
#include <string.h>
#include <intLib.h>
#include <pc.h>
#include <logLib.h>
#include <taskLib.h>
#include <vxLib.h>
#include <errnoLib.h>
#include <stdio.h>
#include <cacheLib.h>
#include <vmLib.h>
#include <arch/i86/ipiI86Lib.h>
#include <arch/i86/pentiumLib.h>
#include <iv.h>
#ifdef INCLUDE_EXC_TASK
#include <private/excLibP.h>            /* _func_excJobAdd */
#endif /* INCLUDE_EXC_TASK */

#if defined(INCLUDE_BOOT_GO_OVERRIDE) && \
    defined(BOOT_GO_OVERRIDE_FUNCTION)
#include <bootApp.h>
#include <private/loadElfLibP.h>
#endif /* INCLUDE_BOOT_GO_OVERRIDE */

#include <hwif/fw/mpapic/mpApic.h>

#if defined (INCLUDE_ACPI)
#include <hwif/fw/acpiLib.h>
#endif /* INCLUDE_ACPI */

#ifdef _WRS_CONFIG_SMP
#include <private/kernelLibP.h>
#include <private/windLibP.h>
#include <private/vxSmpP.h>

#include <regs.h>

#include <fppLib.h>
#include <cpuset.h>
#include <vxCpuLib.h>
#include <subsys/int/vxIpiLib.h>
#endif /* _WRS_CONFIG_SMP */

#include <private/vxCpuLibP.h>
#include <arch/i86/vxCpuArchLib.h>
#include <hwif/vxbus/vxbLib.h>
#include <private/vmLibP.h>

#ifdef  INCLUDE_PC_CONSOLE
#include <pcConsole.h>
#endif /* INCLUDE_PC_CONSOLE */

/* defines */

/* TSC calibration duration, can be 1 to 20 ms */

#define TSC_CALIBRATE_DURATION    (2)

/* imports */

#ifdef DRV_IA_PCI_BUS
IMPORT STATUS pciConfigInLong (int busNo,
                               int deviceNo,
                               int funcNo,
                               int offset,
                               UINT32 * pData);
#endif /* DRV_IA_PCI_BUS */

IMPORT MP_APIC_DATA  *pMpApicData;

#ifdef INCLUDE_USR_MPAPIC
IMPORT STATUS usrMpapicInit (BOOL earlyInit, char * pBuf);
#endif /* INCLUDE_USR_MPAPIC */

#ifdef _WRS_CONFIG_SMP
IMPORT cpuset_t vxCpuEnabled;

IMPORT TSS      sysTss;
#endif /* _WRS_CONFIG_SMP */

#ifndef _WRS_CONFIG_SMP
IMPORT int         intCnt;                    /* interrupt mode counter */
#endif /* !_WRS_CONFIG_SMP */

IMPORT char        end;                       /* linker defined end-of-image */
IMPORT GDT         sysGdt[];                  /* the global descriptor table */
IMPORT void        elcdetach (int unit);
IMPORT VOIDFUNCPTR intEoiGet;                 /* BOI/EOI function pointer */
IMPORT void        intEnt (void);

IMPORT void        sysInitGDT (void);
IMPORT UINT32      sysCalTSC (int duration);

#if defined (INCLUDE_ACPI)
IMPORT void        sysConfInit0 (void);
IMPORT void        sysConfInit (void);
IMPORT void        sysConfStop (void);
#endif /* INCLUDE_ACPI */

#ifdef _WRS_CONFIG_SMP
IMPORT void sysInitCpuStartup(void);
IMPORT void sysInitCpuStartupEnd(void);
IMPORT void cpuInit1(void);
IMPORT char sysTableEnd;
IMPORT GDT         sysGdt32[];

#if defined (INCLUDE_RTP)
IMPORT STATUS syscallArchInit (void);
#endif /* INCLUDE_RTP */

#if defined INCLUDE_PROTECT_TASK_STACK ||       \
    defined INCLUDE_PROTECT_INTERRUPT_STACK
#if defined(_WRS_OSM_INIT)
IMPORT STATUS excOsmInit (UINT32, UINT32);
#endif /* defined(_WRS_OSM_INIT) */
#endif  /* INCLUDE_PROTECT_TASK_STACK || INCLUDE_PROTECT_INTERRUPT_STACK */

#endif /* _WRS_CONFIG_SMP */

IMPORT void (*_vxb_usDelayRtn)(int);
IMPORT void (*_vxb_msDelayRtn)(int);

#ifdef INCLUDE_BCM43XX_END
IMPORT void sysBroadComWLBusInit(void);
#endif /* INCLUDE_BCM43XX_END */

#ifdef INCLUDE_EARLY_PRINT
UINT   kwriteSerialInitialized = 0;
STATUS kwriteSerial(char * pBuffer, size_t len);

/* aiop, 06jun18 start: add featuers to kwriteVga() */
/*
 * Variables and functions related to the "page" facility added to kwriteVga().
 * See kwriteVgaPageSizeSet() for details.
 */
UINT   kwriteVgaPageSizeLines = 0;
UINT   kwriteVgaLineNum = 0; 
void   kwriteVgaPageSizeSet (UINT lines);
void   kwriteVgaClearLine (int line);
void   kwriteVgaClearScreen (void);
void   kwriteVgaScroll (void);
/* aiop, 06jun18 end: add featuers to kwriteVga() */

#include <hwif/drv/sio/vxbNs16550Sio.h>
#endif /* INCLUDE_EARLY_PRINT */

/* globals */

#ifdef DRV_IA_PCI_BUS
UINT32 smi_en_addr = 0;
#endif /* DRV_IA_PCI_BUS */

static UINT64 calibratedTSC = 2000000000ULL;    /* default as 2 GHz */

#ifdef INCLUDE_EARLY_PRINT
UINT    vgaScreenOffset = 0;
IMPORT  void earlyPrint(char *outString);
IMPORT  void earlyPrintVga(char *outString);
IMPORT  void setVgaBufferVirtAddress (void);
STATUS kwriteSerial(char * pBuffer, size_t len);
#if defined (INCLUDE_ACPI)
IMPORT  void errorPrint(char *outString);
#endif /* INCLUDE_ACPI */
#endif /* INCLUDE_EARLY_PRINT */

void sysLedOut(short val);

#if defined (INCLUDE_ACPI)

/*
 * ACPI configuration tables.  Used as static tables when ACPI isn't included.
 *
 * Information contained in this table would be considered proprietary by
 * Intel.  ACPI can be used to construct the table by executing
 * sysBoardInfoPrint in a target shell using a vxWorks image created
 * with the INCLUDE_ACPI_CPU_CONFIG component.
 */

ACPI_BOARD_INFO  sysBspBoardInfo =
    {
    0,              /* acpiNumCpus */
    NULL,           /* pAcpiCpu */
    0,              /* acpiNumThermZones */
    NULL,           /* pThermalZone */
    };

#ifdef INCLUDE_CPU_PWR_ARCH
#ifndef INCLUDE_ACPI_CPU_CONFIG
ACPI_BOARD_INFO  *pSysBoardInfo = &sysBspBoardInfo;  /* static table */
#else
ACPI_BOARD_INFO  *pSysBoardInfo = &sysAcpiBoardInfo; /* constructed by ACPI */
#endif /* INCLUDE_ACPI_CPU_CONFIG */
#else /* INCLUDE_CPU_PERFORMANCE_MGMT */
ACPI_BOARD_INFO  *pSysBoardInfo = &sysAcpiBoardInfo; /* constructed by ACPI */
#endif /* INCLUDE_CPU_PWR_ARCH */

#endif /* INCLUDE_ACPI */

#ifdef  INCLUDE_LPT

LPT_RESOURCE lptResources [N_LPT_CHANNELS] =
    {
    {LPT0_BASE_ADRS, INT_NUM_LPT0, LPT0_INT_LVL,
    TRUE, 10000, 10000, 1, 1, 0
    },

    {LPT1_BASE_ADRS, INT_NUM_LPT1, LPT1_INT_LVL,
    TRUE, 10000, 10000, 1, 1, 0
    },

    {LPT2_BASE_ADRS, INT_NUM_LPT2, LPT2_INT_LVL,
    TRUE, 10000, 10000, 1, 1, 0
    }
    };

#endif  /* INCLUDE_LPT */

UINT    sysIntIdtType   = SYS_INT_INTGATE;  /* IDT entry type */
UINT    sysProcessor    = (UINT) NONE;      /* 2=P5, 4=P6, 5=P7 */
UINT    sysCoprocessor  = 0;        /* 0=none, 1=387, 2=487 */

int     sysWarmType = SYS_WARM_TYPE;      /* system warm boot type */

UINT    sysStrayIntCount = 0;       /* Stray interrupt count */

int     sysSmiUsbLegacyInitState = -1;

UINT32  memRom     = 0; /* saved bootrom image */
UINT32  memRomSize = 0; /* saved bootrom image size */

/* Initial GDT to be initialized */

GDT *pSysGdt    = (GDT *)(LOCAL_MEM_LOCAL_ADRS + GDT_BASE_OFFSET);

/*
 * mpCpuIndexTable is used in vxCpuArchLib.c, this global avoids
 * calling through vxbus and reduces overhead, table translations
 * are done everywhere, but once up this table should never change...
 */

UINT8 mpCpuIndexTable[256];

#ifdef _WRS_CONFIG_SMP
/* TSS initialized */

TSS *sysTssCurrent = &sysTss;                   /* current sysTss */

FUNCPTR sysCpuInitTable[VX_MAX_SMP_CPUS] = {NULL};
unsigned int sysCpuLoopCount[VX_MAX_SMP_CPUS] = {0};
#else
TSS      sysTss;
#endif /* _WRS_CONFIG_SMP */

/* forward declarations for user defined parameters */

#ifdef INCLUDE_USR_BOOT_OP
void sysStaticMpApicDataGet(MP_APIC_DATA *pMpApicData);
#endif /* INCLUDE_USR_BOOT_OP */

UINT64 sysGetTSCCountPerSec (void);

/* user defined parameters established in 00bsp.cdf */

/* MpApic config params... */

const unsigned int    mp_apic_boot_op = MP_APIC_BOOT_OP;
const unsigned long   mp_apic_data_loc = MP_APIC_DATA_LOC;
const unsigned long * mp_apic_data_function = MP_APIC_DATA_FUNCTION;


/*
 * These are used in sysLib.c to avoid calling out
 * through vxbus to reduces overhead, and potential
 * spinLock nesting...
 */

#ifdef _WRS_CONFIG_SMP
LOCAL UINT8 tramp_code[CPU_AP_AREA];
#endif /* _WRS_CONFIG_SMP */

IMPORT STATUS loApicIntrIntLock (void);
IMPORT STATUS loApicIntrIntUnlock (void);

IMPORT STATUS loApicIntrInitAP (int cpuNum);

#include <hwif/cpu/arch/i86/vxCpuIdLib.h>

IMPORT int vxCpuIdProbe(VX_CPUID *);

UINT vxProcessor = (UINT) NONE; /* 6=core, 7=atom, 8=nehalem, 9=sandybridge, */
                                /* 10=haswell                                */

VX_CPUID vxCpuId = {{0,{0},0,0,0,0,0,0,0,0,{0,1},{0}}}; /* VX_CPUID struct. */

/* for backward compatibility, maintain old CPUID struct */

CPUID sysCpuId = {0,{0},0,0,0,0,0,0,0,0,{0,1},{0}}; /* CPUID struct. */

#ifndef _WRS_CONFIG_SMP
BOOL    sysBp       = TRUE;     /* TRUE for BP, FALSE for AP */
#endif /* !_WRS_CONFIG_SMP */

/*
 * sysInumTbl is created dynamically under the IO Apic driver initialization.
 *
 * Multiple IO Apics are used in high-end severs to distribute irq load.
 * At this time there may be as many as 8 IO Apics in a system. The most common
 * number of inputs on a single IO Apic is 24, but they also designed for
 * 16/32/or 64 inputs.
 *
 * The common case here is a 1:1 irq<=>pin mapping.
 *
 * The mapping is critical to system operation. We need to support the first
 * 16 interrupts as legacy ISA/EISA.
 *
 * irq 0,1,2,3,4,6,8,12,13,14, and 15 are typically consumed by legacy devices.
 *
 * irq 5,7,9,10, and 11 are typically available for general use (though 5 is
 * sometimes used for audio devices and 7 is used for LPT).
 *
 * 16,17,18, and 19 are typically used for standard PCI.
 *
 * Each of the interrupts can be used programmed with any of the 224 vectors the
 * CPU is capable of. The first IO Apic will handle the ISA/EISA interrupts.
 *
 * The way this will work: if (irq <= 16) just map the irq to whatever pin is
 * requested. If (irq > 16) we enforce the mapping irq =
 * (apic*num_ioapic_entriess)+pin.
 *
 * This means that given any irq we can always derive the apic and pin.
 *
 *          apic from irq => irq / num_ioapic_entriess
 *
 *          pin from irq  => irq % num_ioapic_entriess
 */

UINT8   dfltInumTbl[] = /* IRQ vs IntNum table for default interrupt handling */
    {
    INT_NUM_IRQ0,               /* IRQ  0 Vector No */
    INT_NUM_IRQ0 + 1,           /* IRQ  1 Vector No */
    INT_NUM_IRQ0 + 2,           /* IRQ  2 Vector No */
    INT_NUM_IRQ0 + 3,           /* IRQ  3 Vector No */
    INT_NUM_IRQ0 + 4,           /* IRQ  4 Vector No */
    INT_NUM_IRQ0 + 5,           /* IRQ  5 Vector No */
    INT_NUM_IRQ0 + 6,           /* IRQ  6 Vector No */
    INT_NUM_IRQ0 + 7,           /* IRQ  7 Vector No */
    INT_NUM_IRQ0 + 8,           /* IRQ  8 Vector No */
    INT_NUM_IRQ0 + 9,           /* IRQ  9 Vector No */
    INT_NUM_IRQ0 + 10,          /* IRQ 10 Vector No */
    INT_NUM_IRQ0 + 11,          /* IRQ 11 Vector No */
    INT_NUM_IRQ0 + 12,          /* IRQ 12 Vector No */
    INT_NUM_IRQ0 + 13,          /* IRQ 13 Vector No */
    INT_NUM_IRQ0 + 14,          /* IRQ 14 Vector No */
    INT_NUM_IRQ0 + 15           /* IRQ 15 Vector No */
    };

UINT8 *sysInumTbl;
UINT32 sysInumTblNumEnt;

/* locals */

    /*
     * The cache control flags and MTRRs operate hierarchically for restricting
     * caching.  That is, if the CD flag is set, caching is prevented globally.
     * If the CD flag is clear, either the PCD flags and/or the MTRRs can be
     * used to restrict caching.  If there is an overlap of page-level caching
     * control and MTRR caching control, the mechanism that prevents caching
     * has precedence.  For example, if an MTRR makes a region of system memory
     * uncachable, a PCD flag cannot be used to enable caching for a page in
     * that region.  The converse is also true; that is, if the PCD flag is
     * set, an MTRR cannot be used to make a region of system memory cacheable.
     * If there is an overlap in the assignment of the write-back and write-
     * through caching policies to a page and a region of memory, the write-
     * through policy takes precedence.  The write-combining policy takes
     * precedence over either write-through or write-back.
     */

LOCAL MTRR sysMtrr =
    {                   /* MTRR table */
    {0,0},              /* MTRR_CAP register */
    {0,0},              /* MTRR_DEFTYPE register */

                        /* Fixed Range MTRRs */

    {{{MTRR_WB, MTRR_WB, MTRR_WB, MTRR_WB, MTRR_WB, MTRR_WB, MTRR_WB, MTRR_WB}},
     {{MTRR_WB, MTRR_WB, MTRR_WB, MTRR_WB, MTRR_WB, MTRR_WB, MTRR_WB, MTRR_WB}},
     {{MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_WC, MTRR_WC, MTRR_WC, MTRR_WC}},
     {{MTRR_WP, MTRR_WP, MTRR_WP, MTRR_WP, MTRR_WP, MTRR_WP, MTRR_WP, MTRR_WP}},
     {{MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC}},
     {{MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC}},
     {{MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC}},
     {{MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC}},
     {{MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC}},
     {{MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC}},
     {{MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC, MTRR_UC}}
    },
    {{0LL, 0LL},            /* Variable Range MTRRs */
     {0LL, 0LL},
     {0LL, 0LL},
     {0LL, 0LL},
     {0LL, 0LL},
     {0LL, 0LL},
     {0LL, 0LL},
     {0LL, 0LL}
    }
    };

/* forward declarations */

int sysSmiUsbLegacyState(void);
void sysSmiUsbLegacyOff(void);
void sysSmiUsbLegacyOn(void);

UINT64 sysCalibrateTSC (void);

void sysStrayInt   (void);

void sysIntInitPIC (void);
VOID sysIntLock (void);
void sysIntEoiGet (VOIDFUNCPTR * vector,
                   VOIDFUNCPTR * routineBoi, _Vx_usr_arg_t * parameterBoi,
                   VOIDFUNCPTR * routineEoi, _Vx_usr_arg_t * parameterEoi);

STATUS sysHpetOpen(void);

#if defined(INCLUDE_BOOT_GO_OVERRIDE) && \
    defined(BOOT_GO_OVERRIDE_FUNCTION)
void sysBootElf32Or64 (void * entryPoint,int startType,int elfClass);
void sysBootElf32 (UINT32 entryPoint, int startType);
#elif defined(INCLUDE_MULTI_STAGE_BOOT)
void sysBootElf32 (UINT32 entryPoint, int startType);
#endif /* INCLUDE_BOOT_GO_OVERRIDE */

#ifdef _WRS_CONFIG_SMP
void sysCpuInit (void);

STATUS sysCpuStart(int cpuNum, WIND_CPU_STATE *cpuState);
STATUS vxCpuStateInit(unsigned int cpu, WIND_CPU_STATE *cpuState,
                      char *stackBase, FUNCPTR entry);
STATUS sysCpuDisable(int cpuNum);

STATUS sysCpuStop(int cpu);
STATUS sysCpuStop_APs(void);
STATUS sysCpuStop_ABM(void);

STATUS sysCpuReset(int cpu);
STATUS sysCpuReset_APs(void);
STATUS sysCpuReset_ABM(void);
#endif /* _WRS_CONFIG_SMP */

unsigned int dummyGetCpuIndex (void);

/* includes (source file) */

#ifdef INCLUDE_DEV_PWR_MGMT
#include "sysDevPwr.c"
#endif /* INCLUDE_DEV_PWR_MGMT */

#if (NV_RAM_SIZE != NONE)
#include <sysNvRam.c>
#endif  /* (NV_RAM_SIZE != NONE) */

#include <i8259Intr.c>

#include <hwif/vxBus.h>

#if defined(INCLUDE_PC_CONSOLE) && defined (DRV_TIMER_I8253)
void sysConBeep (BOOL);
IMPORT char * pcConBeep_desc;
#else
/* dummy, when PC Console/Timers not present. */

void sysConBeep (BOOL);
#endif /* INCLUDE_PC_CONSOLE && DRV_TIMER_I8253 */

#if defined INCLUDE_BIOS_E820_MEM_AUTOSIZE
#   include <hwif/fw/bios/vxBiosE820Lib.h>
#endif /* INCLUDE_BIOS_E820_MEM_AUTOSIZE */

#include "boardSys.c"

#if defined(INCLUDE_EARLY_PRINT)
#include <pmapLib.h>
static void *vgaBufferVirtAddress;


/*******************************************************************************
*
* setVgaBufferVirtAddress - Assign a virtual address to the VGA display buffer
*
* RETURNS: N/A
*
*/

void setVgaBufferVirtAddress (void)
    {
    vgaBufferVirtAddress =
        pmapGlobalMap ((PHYS_ADDR)0x0b8000, 0x8000,
                       (MMU_ATTR_VALID | MMU_ATTR_SUP_RW |
                        MMU_ATTR_CACHE_OFF | MMU_ATTR_VALID_MSK));
    if (vgaBufferVirtAddress == (void *)(-1))
        earlyPrint ("\npmapGlobalMap of VGA buffer failed.\n");
    else
#ifdef INCLUDE_ACPI_BOOT_OP
        earlyPrintVga ("Booting with ACPI. ");
#elif defined INCLUDE_MPTABLE_BOOT_OP
        earlyPrintVga ("Booting with BIOS MP Table. ");
#elif defined INCLUDE_USR_BOOT_OP
        earlyPrintVga ("Booting with User MP Table. ");
#else
        earlyPrintVga ("Booting with No MP Table. ");
#endif
    }

#endif /* INCLUDE_EARLY_PRINT */


#if defined(INCLUDE_BOOT_GO_OVERRIDE) && \
    defined(BOOT_GO_OVERRIDE_FUNCTION)
/*******************************************************************************
*
* sysBootElf32Or64 - Execute the Image after configuring for ELF32 or ELF64
*
* Starts execution of image at the specified address after configuring for
* ELF32 or ELF64.
*
* RETURNS: N/A
*
* ERRNO: N/A
*
* \NOMANUAL
*
*/

void sysBootElf32Or64
    (
    void *  entryPoint, /* entry point address */
    int     startType,  /* system start type */
    int     elfClass    /* ELF file class: ELF32 or ELF64 */
    )
    {
    FUNCPTR pEntry = (FUNCPTR) entryPoint;

#if defined(INCLUDE_PC_CONSOLE) && defined(INCLUDE_LEGACY_USB_INT_MODE)
    /* Turn off legacy SMI USB interrupt mode */

    sysSmiUsbLegacyOff();
#endif /* INCLUDE_PC_CONSOLE */

    /* go to image entry point - never to return */

    if (elfClass == ELFCLASS64)
        {

        /* ELF 64 image */

        (*pEntry) (startType);
        }
    else
        {

        /* ELF 32 image */

        sysBootElf32((UINT32)((ULONG)pEntry),startType);
        }
    }
#endif /* INCLUDE_BOOT_GO_OVERRIDE */


/*******************************************************************************
*
* sysSmiMapInit - SMI I/O memory area map initialization
*
* RETURNS: N/A
*
*/

LOCAL void sysSmiMapInit (void)
    {
#ifdef DRV_IA_PCI_BUS
    UINT32 pmbase;

    if (pciConfigInLong (LPC_BUSNO,
                         LPC_DEVNO,
                         LPC_FUNCNO,
                         LPC_PMBASE_OFFSET,
                         &pmbase) == ERROR)
        {
        return;
        }
    pmbase &= LPC_PMBASE_MASK;
    smi_en_addr  = pmbase + LPC_SMI_EN_OFFSET;
    if (vmMap ((VM_CONTEXT_ID)NULL,
               (VIRT_ADDR)ROUND_DOWN(smi_en_addr, VM_PAGE_SIZE),
               (PHYS_ADDR)ROUND_DOWN(smi_en_addr, VM_PAGE_SIZE),
               ROUND_UP(sizeof(UINT32), VM_PAGE_SIZE)) == ERROR)
        {
        smi_en_addr = 0;
        return;
        }

    (void) vmStateSet((VM_CONTEXT_ID)NULL,
               (VIRT_ADDR)ROUND_DOWN(smi_en_addr, VM_PAGE_SIZE),
               ROUND_UP(sizeof(UINT32), VM_PAGE_SIZE),
               MMU_ATTR_SPL_MSK | MMU_ATTR_VALID_MSK,
               MMU_ATTR_NO_BLOCK | MMU_ATTR_VALID_NOT);

    return;
#else
    return;
#endif /* DRV_IA_PCI_BUS */
    }


#ifdef DRV_IA_PCI_BUS
/*******************************************************************************
*
* sysSmiMap - Set SMI I/O memory area valid
*
* RETURNS: OK, or ERROR if failed.
*
*/

LOCAL STATUS sysSmiMap (void)
    {
    if (smi_en_addr == 0)
        {
        return ERROR;
        }

    return (vmStateSet((VM_CONTEXT_ID)NULL,
               (VIRT_ADDR)ROUND_DOWN(smi_en_addr, VM_PAGE_SIZE),
               ROUND_UP(sizeof(UINT32), VM_PAGE_SIZE),
               MMU_ATTR_VALID_MSK,
               MMU_ATTR_VALID));
    }


/*******************************************************************************
*
* sysSmiUnmap - Set SMI I/O memory area invalid
*
* RETURNS: N/A
*
*/

LOCAL void sysSmiUnmap (void)
    {
    (void) vmStateSet((VM_CONTEXT_ID)NULL,
               (VIRT_ADDR)ROUND_DOWN(smi_en_addr, VM_PAGE_SIZE),
               ROUND_UP(sizeof(UINT32), VM_PAGE_SIZE),
               MMU_ATTR_SPL_MSK | MMU_ATTR_VALID_MSK,
               MMU_ATTR_NO_BLOCK | MMU_ATTR_VALID_NOT);
    }
#endif /* DRV_IA_PCI_BUS */


/*******************************************************************************
*
* sysSmiUsbLegacyState - Get legacy USB SMI interrupt mode
*
* RETURNS: 0  if legacy USB SMI interrupt mode is off or
*          1  if legacy USB SMI interrupt mode is on or
*          -1 if ERROR
*
*/

int sysSmiUsbLegacyState (void)
    {
#ifdef DRV_IA_PCI_BUS
    int state;

    if (sysSmiMap() == ERROR)
        {
        return (-1);
        }
    if ((sysInLong(smi_en_addr) & LPC_LEGACY_USB) == 0)
        {
        state = 0;
        }
    else
        {
        state = 1;
        }
    sysSmiUnmap();

    return (state);
#else
    return (-1);
#endif /* DRV_IA_PCI_BUS */
    }


/*******************************************************************************
*
* sysSmiUsbLegacyOff - Turn legacy USB SMI interrupts off
*
* RETURNS: N/A
*
*/

void sysSmiUsbLegacyOff (void)
    {
#ifdef DRV_IA_PCI_BUS
    if (sysSmiUsbLegacyInitState == -1)
        {
        sysSmiUsbLegacyInitState = sysSmiUsbLegacyState();
        }

    if (sysSmiUsbLegacyInitState == 0)
        {
        return;
        }

    if (sysSmiMap() == ERROR)
        {
        return;
        }
    sysOutLong(smi_en_addr, (sysInLong(smi_en_addr) & (~LPC_LEGACY_USB)));
    sysSmiUnmap();
    sysSmiUsbLegacyInitState = 0;
#endif /* DRV_IA_PCI_BUS */
    }


/*******************************************************************************
*
* sysSmiUsbLegacyOn - Turn legacy USB SMI interrupts on
*
* RETURNS: N/A
*
*/

void sysSmiUsbLegacyOn (void)
    {
#ifdef DRV_IA_PCI_BUS
    if (sysSmiUsbLegacyInitState == -1)
        {
        sysSmiUsbLegacyInitState = sysSmiUsbLegacyState();
        }

    if (sysSmiUsbLegacyInitState == 1)
        {
        return;
        }

    if (sysSmiMap() == ERROR)
        {
        return;
        }
    sysOutLong(smi_en_addr, (sysInLong(smi_en_addr) | LPC_LEGACY_USB));
    sysSmiUnmap();
    sysSmiUsbLegacyInitState = 1;
#endif /* DRV_IA_PCI_BUS */
    }


/*******************************************************************************
*
* sysSmiOff - Turn global SMI interrupts off
*
* NOTE: Please use this function with caution.  SMI (System Management
*       Interrupts) interrupts on some hardware are involved in thermal
*       throttling and other functions.  If this is the case, turning
*       global SMI interrupts off may lead to hardware damage or cause
*       other things to not operate properly.
*
* RETURNS: N/A
*
*/

void sysSmiOff (void)
    {
#ifdef DRV_IA_PCI_BUS
    if (sysSmiMap() == ERROR)
        {
        return;
        }
    sysOutLong(smi_en_addr, (sysInLong(smi_en_addr) & (~LPC_GBL_SMI)));
    sysSmiUnmap();
#endif /* DRV_IA_PCI_BUS */
    }


/*******************************************************************************
*
* sysSmiOn - Turn global SMI interrupts on
*
* RETURNS: N/A
*
*/

void sysSmiOn (void)
    {
#ifdef DRV_IA_PCI_BUS
    if (sysSmiMap() == ERROR)
        {
        return;
        }
    sysOutLong(smi_en_addr, (sysInLong(smi_en_addr) | LPC_GBL_SMI));
    sysSmiUnmap();
#endif /* DRV_IA_PCI_BUS */
    }


/*******************************************************************************
*
* sysSmiShow - Show contents of SMI_EN register
*
* RETURNS: N/A
*
*/

void sysSmiShow (void)
    {
#ifdef DRV_IA_PCI_BUS
    if (sysSmiMap() == ERROR)
        {
        return;
        }
    (void) printf("SMI_EN = 0x%08x\n", sysInLong(smi_en_addr));
    sysSmiUnmap();
#endif /* DRV_IA_PCI_BUS */
    }


#ifdef INCLUDE_SYS_HW_INIT_0

#ifdef GRUB_MULTIBOOT
#include "multiboot.c"
#endif /* GRUB_MULTIBOOT */

/*******************************************************************************
*
* sysHwInit0 - BSP-specific hardware initialization
*
* This routine is called from usrInit() to perform BSP-specific initialization
* that must be done before cacheLibInit() is called and/or the BSS is cleared.
*
* The vxCpuIdProbe() routine is called for the purpose of
* identifying IA target CPU variants, and the features or functions
* supported by the target CPU.  This information must be obtained relatively
* early during system hardware initialization, as some support libraries
* (mmuLib, cacheLib, &c.) will use the processor feature information to
* enable or disable architecture-specific and/or BSP-specific functionality.
*
* RETURNS: N/A
*
* \NOMANUAL
*
*/

void sysHwInit0 (void)
    {

    _func_cpuIndexGet = NULL;

#ifdef _WRS_CONFIG_SMP
    unsigned int cpuIndex;
#endif /* _WRS_CONFIG_SMP */

#ifdef INCLUDE_EARLY_PRINT
    _func_kwrite = kwriteSerial; /* Register routine with kprintfLib */
#endif /* INCLUDE_EARLY_PRINT */

#if defined (INCLUDE_ACPI)
#ifdef INCLUDE_ACPI_DEBUG_PRINT
    func_acpiEarlyPrint = earlyPrint;   /* print ACPI debug info */
#endif /* INCLUDE_ACPI_DEBUG_PRINT */
#ifdef INCLUDE_EARLY_PRINT
    func_acpiErrorPrint = errorPrint;   /* print info and pause */
#endif /* INCLUDE_EARLY_PRINT */
#endif /* defined (INCLUDE_ACPI) */

#ifdef _WRS_CONFIG_SMP

    cpuIndex = vxCpuIndexGet();

    /* temporarily use structure to load the brand new GDT into RAM */

    _WRS_WIND_VARS_ARCH_SET(cpuIndex, pSysGdt, pSysGdt);

#endif /* _WRS_CONFIG_SMP */

    sysInitGDT ();

#ifdef GRUB_MULTIBOOT
    multibootInit ();
#endif /* GRUB_MULTIBOOT */

    /* initialize CPUID library */

    vxCpuIdLibInit();

    /* perform CPUID probe */

    vxProcessor = sysProcessor = vxCpuIdProbe( &vxCpuId );

    /*
     * for backward compatibility, populate sysCpuId with
     * vxCpuId standard feature structure contents
     */

    bcopy ((char *)&vxCpuId.std, (char *)&sysCpuId, (int)sizeof(VX_CPUID_STD));

    /* Calibrate TSC before VxBus subsystem init */

    (void) vxCpuIdSetFreq(sysCalibrateTSC());

    }

#endif  /* INCLUDE_SYS_HW_INIT_0 */


/*******************************************************************************
*
* sysHwInit - initialize the system hardware
*
* This routine initializes various features of the i386/i486 board.
* It is called from usrInit() in usrConfig.c.
*
* NOTE: This routine should not be called directly by the user application.
*
* RETURNS: N/A
*
*/

void sysHwInit (void)
    {
#ifdef _WRS_CONFIG_SMP
    INT64  cpuKernelVars;
    unsigned int cpuIndex;
#endif /* _WRS_CONFIG_SMP */

#ifdef  INCLUDE_EARLY_I_CACHE_ENABLE
    (void) cacheEnable (INSTRUCTION_CACHE);    /* enable instruction cache */
#endif  /* INCLUDE_EARLY_I_CACHE_ENABLE */
#ifdef  INCLUDE_EARLY_D_CACHE_ENABLE
    (void) cacheEnable (DATA_CACHE);           /* enable data cache */
#endif  /* INCLUDE_EARLY_D_CACHE_ENABLE */

#ifdef _WRS_CONFIG_SMP

    /* store trampoline code space for restoration on warm reboot */

    bcopy ((char *)(CPU_ENTRY_POINT),
                    (char *)(&tramp_code[0]),
                    (int)CPU_AP_AREA);

    /*
     * establish pointers to global descriptor table & task state segment
     * TSS is assigned a cpu unique structure...
     */

    cpuIndex = vxCpuIndexGet();

    _WRS_WIND_VARS_ARCH_SET(cpuIndex, pSysGdt, pSysGdt);

    _WRS_WIND_VARS_ARCH_SET(cpuIndex, sysTssCurrent, sysTssCurrent);

    _WRS_WIND_VARS_ARCH_SET(cpuIndex, vxIntStackEnabled, 1);

    /* establish dedicated segment register for vxKernelVars */

    vxKernelVars[cpuIndex].vars.cpu_archVars.cpu_cpuIndex = cpuIndex;
    cpuKernelVars = (INT64) &vxKernelVars[cpuIndex];
    vxKernelVars[cpuIndex].vars.cpu_archVars.cpu_cpuGsBaseCurrent =
                    (ULONG) &vxKernelVars[cpuIndex];

    pentiumMsrSet (IA32_GS_BASE, (long long *) &cpuKernelVars);
    pentiumMsrSet (MSR_IA32_KERNEL_GS_BASE, (long long *) &cpuKernelVars);

#endif /* _WRS_CONFIG_SMP */

    /* enable the MTRR (Memory Type Range Registers) */

    if ((vxCpuId.std.featuresEdx & CPUID_MTRR) == CPUID_MTRR)
        {
        pentiumMtrrDisable ();      /* disable MTRR */
#   ifdef INCLUDE_MTRR_GET
        (void) pentiumMtrrGet (&sysMtrr); /* get MTRR initialized by BIOS */
#   else
        (void) pentiumMtrrSet (&sysMtrr); /* set your own MTRR */
#   endif /* INCLUDE_MTRR_GET */
        pentiumMtrrEnable ();       /* enable MTRR */
        }

#   ifdef INCLUDE_PMC

    /* enable PMC (Performance Monitoring Counters) */

    pentiumPmcStop ();          /* stop PMC0 and PMC1 */
    pentiumPmcReset ();         /* reset PMC0 and PMC1 */

#   endif /* INCLUDE_PMC */

    /* enable the MCA (Machine Check Architecture) */

    pentiumMcaEnable (TRUE);

#ifdef INCLUDE_SHOW_ROUTINES

    /*
     * if excMcaInfoShow is not NULL, it is called in the default
     * exception handler when Machine Check Exception happened
     */

    {
    IMPORT FUNCPTR excMcaInfoShow;

    /*
     * LP64 support: excMcaInfoShow ptr set to NULL while pentiumShow is not
     * 64-bit ready.
     */

    excMcaInfoShow = (FUNCPTR) NULL;
    }

    vxShowInit ();
#endif /* INCLUDE_SHOW_ROUTINES */

    }


/*******************************************************************************
*
* sysHwInit1 - additional system configuration and initialization
*
* This routine connects system interrupts and does any additional
* configuration necessary.  It is called from usrRoot() prior to
* the enabling of interrupts.
*
* RETURNS: N/A
*
*/

void sysHwInit1 (void)
    {

#if defined (INCLUDE_EARLY_PRINT)

    setVgaBufferVirtAddress();

#endif

#if defined (INCLUDE_ACPI)
    /* load ACPI tables */

#ifdef INCLUDE_ACPI_TABLE_MAP
    (void) acpiTableInit ();        /* map in ACPI tables (if found) */
#endif  /* INCLUDE_ACPI_TABLES */
#endif /* INCLUDE_ACPI */

#if defined (INCLUDE_USR_MPAPIC)

    /* ACPI initialzation writes to static _MP_ struct location */

    (void) usrMpapicInit (TRUE, (char *) MPAPIC_DATA_START);
#endif /* INCLUDE_USR_MPAPIC */

    /*
     * Map CPU Scratch area for SMP if only one core.
     * This will be setup automatically if there are AP cores.
     * It's mapped here for the case of one core or 
     * ENABLE_ALL_CPUS is defined to FALSE.
     */

#if (defined(_WRS_CONFIG_SMP) && \
     ((ENABLE_ALL_CPUS == FALSE) || (VX_SMP_NUM_CPUS == 1)))
    STATUS status = OK;

    status = vmPageMap ((VM_CONTEXT_ID) NULL,
                        (VIRT_ADDR)0UL,
                        (PHYS_ADDR)0UL,
                        ROUND_UP((CPU_SCRATCH_POINT+CPU_AP_AREA), VM_PAGE_SIZE),
                        DEFAULT_VM_MAP_ATTR_MASK | MMU_ATTR_VALID_MSK,
                        MMU_ATTR_SUP_RWX | MMU_ATTR_CACHE_DEFAULT |
                        MMU_ATTR_NO_BLOCK | MMU_ATTR_VALID_NOT);
#endif 

    /*
     * Map inactive areas needed for warm reboots
     * Map bootrom area
     */

    /* the return value of vmPageMap () is not used */

    (void) vmPageMap ((VM_CONTEXT_ID) NULL,
                      (VIRT_ADDR)ROUND_DOWN(ROM_TEXT_ADRS, VM_PAGE_SIZE),
                      (PHYS_ADDR)ROUND_DOWN(ROM_TEXT_ADRS, VM_PAGE_SIZE),
                      ROUND_UP(ROM_SIZE, VM_PAGE_SIZE),
                      DEFAULT_VM_MAP_ATTR_MASK,
                      MMU_ATTR_SUP_RWX | MMU_ATTR_CACHE_DEFAULT |
                      MMU_ATTR_NO_BLOCK);

#if defined(INCLUDE_FAST_REBOOT) && defined(INCLUDE_MULTI_STAGE_BOOT)

    if ((memRom > 0) && (memRomSize > 0))
        {

        /* the return value of vmMap() is not used */

        (void) vmMap ((VM_CONTEXT_ID)NULL,
               (VIRT_ADDR)ROUND_DOWN(BOOT_MULTI_STAGE_DATA, VM_PAGE_SIZE),
               (PHYS_ADDR)ROUND_DOWN(BOOT_MULTI_STAGE_DATA, VM_PAGE_SIZE),
               ROUND_UP(BOOT_MULTI_STAGE_DATA_SIZE, VM_PAGE_SIZE));

        /* the return value of vmMap() is not used */

        (void) vmMap((VM_CONTEXT_ID)NULL,(VIRT_ADDR)memRom,(PHYS_ADDR)memRom,
              memRomSize);

#if defined(INCLUDE_BOOT_APP)

        /*
         * save the brand new bootrom image that will be protected by MMU.
         * The last 2 bytes are for the checksum.
         * - compression would minimize the DRAM usage.
         * - when restore, jumping to the saved image would be faster.
         */

        bcopy ((char *)ROM_TEXT_ADRS,
               (char *)((ULONG)memRom),
               (*(UINT32 *)BOOT_IMAGE_SIZE));
        *(UINT16 *)((ULONG)memRom + (*(UINT32 *)BOOT_IMAGE_SIZE) - 2) =
            checksum ((UINT16 *)((ULONG)memRom),
                      (*(UINT32 *)BOOT_IMAGE_SIZE) - 2);

#endif  /* defined(INCLUDE_BOOT_APP) */

        (void) vmStateSet((VM_CONTEXT_ID)NULL,
                   (VIRT_ADDR)memRom, memRomSize,
                   MMU_ATTR_SPL_MSK | MMU_ATTR_VALID_MSK,
                   MMU_ATTR_NO_BLOCK | MMU_ATTR_VALID_NOT);

        (void) vmStateSet((VM_CONTEXT_ID)NULL,
                   (VIRT_ADDR)ROUND_DOWN(BOOT_MULTI_STAGE_DATA, VM_PAGE_SIZE),
                   ROUND_UP(BOOT_MULTI_STAGE_DATA_SIZE, VM_PAGE_SIZE),
                   MMU_ATTR_SPL_MSK | MMU_ATTR_VALID_MSK,
                   MMU_ATTR_NO_BLOCK | MMU_ATTR_VALID_NOT);
        }

#endif  /* defined(INCLUDE_FAST_REBOOT) */

    (void) vmStateSet((VM_CONTEXT_ID)NULL,
               (VIRT_ADDR)ROUND_DOWN(ROM_TEXT_ADRS, VM_PAGE_SIZE),
               ROUND_UP(ROM_SIZE, VM_PAGE_SIZE),
               MMU_ATTR_SPL_MSK | MMU_ATTR_VALID_MSK,
               MMU_ATTR_NO_BLOCK | MMU_ATTR_VALID_NOT);

    (void) mpApicInstAttach ();

    _vxb_usDelayRtn = sysUsDelay;
    _vxb_msDelayRtn = sysMsDelay;

    /* establish link to vxb APIC methods */

    /* initialize the PIC (Programmable Interrupt Controller) */

    sysIntInitPIC ();       /* should be after the PCI init for IOAPIC */
    intEoiGet = sysIntEoiGet;   /* function pointer used in intConnect () */

#ifdef _WRS_CONFIG_SMP
    /* init IPI vector support setup */

    ipiVecInit ();
#endif /* _WRS_CONFIG_SMP */

    sysSmiMapInit();

    /* Turn off legacy SMI USB interrupt mode */

    sysSmiUsbLegacyOff();

    }

#ifdef INCLUDE_IO_BASIC
IMPORT BOOL iosLibInitialized;
#endif /* INCLUDE_IO_BASIC */


/*******************************************************************************
*
* sysHwInit2 - additional system configuration and initialization
*
* This routine connects system interrupts and does any additional
* configuration necessary.
*
* RETURNS: N/A
*
*/

void sysHwInit2 (void)
    {

    /* Turn on legacy SMI USB interrupt mode */

    sysSmiUsbLegacyOn();

    /* connect stray(spurious/phantom) interrupt */

    (void)intConnect (INUM_TO_IVEC ((ULONG)INT_NUM_LOAPIC_SPURIOUS),
                      sysStrayInt, 0);
#ifdef GRUB_MULTIBOOT
    (void)intConnect (INUM_TO_IVEC ((ULONG)INT_NUM_LOAPIC_SPURIOUS2),
                      sysStrayInt, 0);
#endif /* GRUB_MULTIBOOT */

#ifdef INCLUDE_DEV_PWR_MGMT
    pwrDeviceLibInit ();                 /* device power management */
#endif /* INCLUDE_DEV_PWR_MGMT */

    }


/*******************************************************************************
*
* sysIntInitPIC - initialize the interrupt controller
*
* This routine initializes the interrupt controller.
*
* RETURNS: N/A
*
*/

void sysIntInitPIC (void)
    {
    i8259Init ();
    }


/*******************************************************************************
*
* sysIntLock - lock out all interrupts
*
* This routine saves the mask and locks out all interrupts.
*
* SEE ALSO: sysIntUnlock()
*
*/

VOID sysIntLock (void)
    {
    INT32 oldLevel = intCpuLock();  /* LOCK INTERRUPTS */

    (void) loApicIntrIntLock();

    intCpuUnlock(oldLevel);         /* UNLOCK INTERRUPTS */
    }


/*******************************************************************************
*
* sysIntUnlock - unlock the PIC interrupts
*
* This routine restores the mask and unlocks the PIC interrupts
*
* SEE ALSO: sysIntLock()
*
*/

VOID sysIntUnlock (void)
    {
    INT32 oldLevel = intCpuLock();  /* LOCK INTERRUPTS */

    (void) loApicIntrIntUnlock();

    intCpuUnlock(oldLevel);         /* UNLOCK INTERRUPTS */
    }

#ifdef _WRS_VX_IA_IPI_INSTRUMENTATION
extern struct ipiCounters *ipiCounters[VX_MAX_SMP_CPUS];
#endif


/*******************************************************************************
* sysLoApicIntEoi -  send EOI (End Of Interrupt) signal to Local APIC
*
* This routine sends an EOI signal to the Local APIC's interrupting source.
*
* RETURNS: N/A
*
*/

void sysLoApicIntEoi
    (
    INT32 irqNo         /* INIIN number to send EOI */
    )
    {
    *(int *)(glbLoApicBaseAdrs + LOAPIC_EOI) = 0;
    }


#ifdef _WRS_VX_IA_IPI_INSTRUMENTATION

/*******************************************************************************
* sysLoApicIpiBoi -  Called before the interrupt service routine to increment
*                    IPI instrumentation interrupt count
*
* This routine is called before the interrupt service routine to increment
* IPI instrumentation interrupt count.
*
* RETURNS: N/A
*
*/

void sysLoApicIpiBoi
    (
    INT32 irqNo
    )
    {
    int ipiNum =  END_IPI_INUM - irqNo;

    struct ipiCounters *ipiCnt = ipiCounters[vxCpuPhysIndexGet()];

    if (ipiNum >= 0 && ipiNum < IPI_MAX_HANDLERS)
        vxAtomicInc((atomic_t *)(&(ipiCnt->ipiRecv[ipiNum])));
    }

#endif /* _WRS_VX_IA_IPI_INSTRUMENTATION */


/*******************************************************************************
*
* sysIntEoiGet - get EOI/BOI function and its parameter
*
* This routine gets EOI function and its parameter for the interrupt controller.
* If returned EOI/BOI function is NULL, intHandlerCreateX86_64() replaces
* "call _routineBoi/Eoi" in intConnectCode[] with NOP instruction.
*
* RETURNS: N/A
*
*/

void sysIntEoiGet
    (
    VOIDFUNCPTR * vector,   /* interrupt vector to attach to */
    VOIDFUNCPTR * routineBoi,   /* BOI function */
    _Vx_usr_arg_t * parameterBoi,     /* a parameter of the BOI function */
    VOIDFUNCPTR * routineEoi,   /* EOI function */
    _Vx_usr_arg_t * parameterEoi      /* a parameter of the EOI function */
    )
    {
    int intNum = IVEC_TO_INUM ((ULONG)vector);
    int irqNo;

    /* set default BOI routine & parameter */

    *routineBoi   = NULL;
    *parameterBoi = 0;

    /* find a match in sysInumTbl[] */

    for (irqNo = 0; irqNo < sysInumTblNumEnt; irqNo++)
        {
        if (sysInumTbl[irqNo] == intNum)
            break;
        }

    *parameterEoi = irqNo;  /* irq is sysInumTblNumEnt, if no match */

    if (*parameterEoi == sysInumTblNumEnt)
        *parameterEoi = intNum;  /* irq is requested intNum, if no match */

    if (intNum == INT_NUM_LOAPIC_SPURIOUS)
        *routineEoi = NULL;        /* no EOI is necessary */
    else
        *routineEoi = sysLoApicIntEoi; /* set Local APIC's EOI routine */

#ifdef _WRS_VX_IA_IPI_INSTRUMENTATION
      /*
       * Add Ipi Counter BOI
       */

    if (intNum >= INT_NUM_LOAPIC_IPI &&
        intNum < (INT_NUM_LOAPIC_IPI + IPI_MAX_HANDLERS))
        {
        *routineBoi = sysLoApicIpiBoi;
        *parameterBoi = intNum;
        }
#endif /* _WRS_VX_IA_IPI_INSTRUMENTATION */
    }


/*******************************************************************************
*
* sysCalibrateTSC - Calibrate the TSC use PIT and CPUID
*
* This routine will calibrate TSC and save the value to calibratedTSC
*
* RETURNS: N/A
*
*/

UINT64 sysCalibrateTSC (void)
    {
    UINT64 pitCalTsc;

    UINT32 duration = TSC_CALIBRATE_DURATION;   /* in milliseconds */

    /*
     * use PIT to calibrate the TSC frequency.
     * In initial stage it should be safe to not call
     * intLock/intUnlock to protect this code.
     */

    pitCalTsc = sysCalTSC (duration);
    pitCalTsc = (pitCalTsc * 1000) / duration;  /* extend to Hz */

    calibratedTSC = pitCalTsc;
    return pitCalTsc;
    }


/*******************************************************************************
*
* sysGetTSCCountPerSec - get the calibrated TSC frequency (Hz)
*
* This routine will return the calibrated TSC frequency in Hz.
*
* RETURNS: N/A
*
* SEE ALSO: sysCalibrateTSC()
*
*/

UINT64 sysGetTSCCountPerSec (void)
    {
    return (calibratedTSC);
    }


/*******************************************************************************
*
* sysStrayInt - Do nothing for stray interrupts
*
* Do nothing for stray interrupts.
*
*/

void sysStrayInt (void)
    {
    sysStrayIntCount++;
    }


/*******************************************************************************
*
* sysLedOut - Output to I/O port 0x80,0x81
*
* byte or word value output to I/O port 0x80,0x81
*
* RETURNS: N/A
*
* \NOMANUAL
*
*/

void sysLedOut
    (
    short val           /* value to output */
    )
    {
    sysOutWord(0x80, val);
    }


#if defined(INCLUDE_PC_CONSOLE) && defined (DRV_TIMER_I8253)
/*******************************************************************************
*
* sysConBeep - sound beep function (using timer 2 for tone)
*
* This function is responsible for producing the beep
*
* RETURNS: N/A
*
* \NOMANUAL
*
*/

void sysConBeep
    (
    BOOL        mode    /* TRUE:long beep  FALSE:short beep */
    )
    {
    int         beepTime;
    int         beepPitch;
    FAST int    oldlevel;

    if (mode)
        {
        beepPitch = BEEP_PITCH_L;
        beepTime  = BEEP_TIME_L;        /* long beep */
        }
    else
        {
        beepPitch = BEEP_PITCH_S;
        beepTime  = BEEP_TIME_S;        /* short beep */
        }

    oldlevel = intCpuLock ();

    /* set command for counter 2, 2 byte write */

    sysOutByte(PIT_BASE_ADR + 3, (char)0xb6);
    sysOutByte(PIT_BASE_ADR + 2, (beepPitch & 0xff));
    sysOutByte(PIT_BASE_ADR + 2, (beepPitch >> 8));

    /* enable counter 2 */

    sysOutByte(DIAG_CTRL, sysInByte(DIAG_CTRL) | 0x03);

    taskDelay (beepTime);

    /* disable counter 2 */

    sysOutByte(DIAG_CTRL, sysInByte(DIAG_CTRL) & ~0x03);

    intCpuUnlock (oldlevel);
    return;
    }
#else
/* dummy, when PC Console/Timers not present. */

void sysConBeep
    (
    BOOL mode
    )
    {
    }
#endif /* INCLUDE_PC_CONSOLE && DRV_TIMER_I8253 */


/******************************************************************************
*
* sysHpetOpen - map and initialize the HPET hardware interface
*
* This function supplied to vxbIntelTimestamp driver that make the driver being
* able to use the HPET timer hardware in its first level initialization.
*
* RETURNS: OK or ERROR depending on supported of HPET.
*
*/

STATUS sysHpetOpen (void)
    {
#if defined (INCLUDE_HPET_TIMESTAMP)
    return OK;
#else  /* defined (INCLUDE_HPET_TIMESTAMP) */
    return ERROR;
#endif /* defined (INCLUDE_HPET_TIMESTAMP) */
    }


#ifdef _WRS_CONFIG_SMP
/*******************************************************************************
*
* vxCpuStateInit - Initialize the state to start a CPU
*
* The purpose of this function is somewhat similar to the arch-specific
* function taskRegsInit(), except that this function initializes a
* WIND_CPU_STATE structure suitable for the initial state of a processor,
* whereas taskRegsInit only needs to initialize a REG_SET structure
* for the initial state of a vxWorks task.
*
* RETURNS: OK or ERROR if any failure.
*
*/

STATUS vxCpuStateInit
    (
    unsigned int cpu,
    WIND_CPU_STATE *cpuState,
    char *stackBase,
    FUNCPTR entry
    )
    {
    TSS *newTss;
    GDT *newGdt;

    GDTPTR gDtp;
    INT32 nGdtEntries, nGdtBytes;

    if ((cpu < 1) || (cpu > (vxCpuCount - 1)))
        return (ERROR);

    if ((cpuState == NULL) || (entry== NULL) || (stackBase == NULL))
        return (ERROR);

    bzero ((char *)cpuState, sizeof(WIND_CPU_STATE));

    /*
     * establish pointers to global descriptor table & task state segment
     * TSS is assigned a cpu unique structure...
     */

    /* copy the global descriptor table from RAM/ROM to RAM */

    /* Get the Global Data Table Descriptor */

    vxGdtrGet ( (long long int *)&gDtp );

    /* Extract the number of bytes */

    nGdtBytes = (INT32)gDtp.wGdtr[0];

    /* and calculate the number of entries */

    nGdtEntries = (nGdtBytes + 1 ) / sizeof(GDT);

    newGdt = (GDT *) malloc (nGdtEntries * sizeof (GDT));

    if (newGdt == NULL)
      return (ERROR);

    bcopy ((char *)MMU_CVT_PHYS_DIR(&sysGdt), (char *)newGdt,
           (int)(nGdtEntries * sizeof (GDT)));

    _WRS_WIND_VARS_ARCH_SET(cpu,
                            pSysGdt,
                            newGdt);

    newTss = (TSS *) malloc (sizeof (TSS));

    if (newTss == NULL)
      return (ERROR);

    bcopy ((char *)(&sysTss), (char *)newTss, (int)sizeof (TSS));

    _WRS_WIND_VARS_ARCH_SET(cpu,
                            sysTssCurrent,
                            newTss);

    _WRS_WIND_VARS_ARCH_SET(cpu,
                            vxIntStackEnabled,
                            1);

    cpuState->regs.eflags = EFLAGS_BRANDNEW;
    cpuState->regs.rsp = (ULONG) stackBase;
    cpuState->regs.pc = (INSTR *)entry;

    return (OK);
    }


/*******************************************************************************
*
* sysCpuStart - start a CPU
*
* The sysCpuStart() function takes two parameters:
* int cpu;                  /@ core number to start @/
* WIND_CPU_STATE *cpuState; /@ pointer to a WIND_CPU_STATE structure @/
*
* The intent is to implement a function with the basic features of
* vxTaskRegsInit() that sets up the regs structure, then passes the
* cpu number and cpuState pointer to this function, which in turn extracts
* the needed values from the cpuState structure and starts the processor
* running...
*
* RETURNS:  OK - Success
*           ERROR - Fail
*
* \NOMANUAL
*
*/

STATUS sysCpuStart
    (
    int cpu,
    WIND_CPU_STATE *cpuState
    )
    {
    STATUS status = OK;          /* return value */
    INT32  ix;
    unsigned int *tmp_stk;       /* temp stk to pass some env params */
    unsigned int *tmpPtr;        /* temp pointer */
    FUNCPTR * baseAddr;          /* new vector base address */
    UINT8 *mploApicIndexTable;
    PHYS_ADDR physAddr;
    PHYS_ADDR physAddrEnd;
    UINT apDelayCount = SYS_AP_LOOP_COUNT; /* Times to check for AP startup */
    UINT apDelay = SYS_AP_TIMEOUT;         /* Time between AP startup checks */

    /* entry point of AP trampoline code */

    UINT32 entryPoint = (UINT32) CPU_ENTRY_POINT;
    UINT32 scratchPoint = (UINT32) CPU_SCRATCH_POINT;
    UINT32 scratchMem = (UINT32) scratchPoint;

    (void) mpApicLoIndexTableGet ((void *)&mploApicIndexTable);

    /*
     * allow access to page 0, for access to WARM_REST_VECTOR,
     * default state (VM_STATE_VALID | VM_STATE_WRITABLE | VM_STATE_CACHEABLE)
     */

    /* Map these regions for now */

    if (vmStateSet((VM_CONTEXT_ID)NULL,
                   (VIRT_ADDR)0UL,
                   ROUND_UP(&sysTableEnd, VM_PAGE_SIZE),
                   MMU_ATTR_VALID_MSK | MMU_ATTR_PROT_MSK,
                   MMU_ATTR_VALID | MMU_ATTR_SUP_RWX) == ERROR)
        {
        status =  vmPageMap ((VM_CONTEXT_ID) NULL,
                             (VIRT_ADDR) 0UL, (PHYS_ADDR) 0UL,
                             ROUND_UP (&sysTableEnd, VM_PAGE_SIZE),
                             DEFAULT_VM_MAP_ATTR_MASK | MMU_ATTR_PROT_MSK,
                             MMU_ATTR_SUP_RWX | MMU_ATTR_CACHE_DEFAULT |
                             MMU_ATTR_NO_BLOCK | MMU_ATTR_VALID);
        }

    status = vmTranslate((VM_CONTEXT_ID) NULL,
                         (VIRT_ADDR) sysInitCpuStartup,
                         (PHYS_ADDR *) &physAddr);

    status = vmTranslate((VM_CONTEXT_ID) NULL,
                         (VIRT_ADDR) &sysInitCpuStartupEnd,
                         (PHYS_ADDR *) &physAddrEnd);

    if (vmStateSet ((VM_CONTEXT_ID) NULL,
                    (VIRT_ADDR) ROUND_DOWN (physAddr, VM_PAGE_SIZE),
                    ROUND_UP ((physAddrEnd - physAddr), VM_PAGE_SIZE),
                    MMU_ATTR_VALID_MSK | MMU_ATTR_PROT_MSK,
                    MMU_ATTR_VALID | MMU_ATTR_SUP_RWX) == ERROR)
        {
        status =  vmPageMap ((VM_CONTEXT_ID) NULL,
                             (VIRT_ADDR) ROUND_DOWN (physAddr, VM_PAGE_SIZE),
                             (PHYS_ADDR) ROUND_DOWN (physAddr, VM_PAGE_SIZE),
                             ROUND_UP ((physAddrEnd - physAddr), VM_PAGE_SIZE),
                             DEFAULT_VM_MAP_ATTR_MASK | MMU_ATTR_PROT_MSK,
                             MMU_ATTR_SUP_RWX | MMU_ATTR_CACHE_DEFAULT |
                             MMU_ATTR_NO_BLOCK | MMU_ATTR_VALID);
        }

    bcopy ((char *)(MMU_CVT_PHYS_DIR(&sysInitCpuStartup)),
           (char *)((ULONG)entryPoint),
           CPU_STARTUP_SIZE);

    /*
     * Initialization of Temporary AP Scratch Memory:
     *
     *    scratchMem (scratch memory offset)  scratchPoint
     *
     *    Standard GDT Entries:
     *
     *    null descriptor                     scratchMem + 0x04
     *
     *    kernel text descriptor              scratchMem + 0x0C
     *
     *    kernel data descriptor              scratchMem + 0x14
     *
     *    exception text descriptor           scratchMem + 0x1C
     *
     *    interrupt data descriptor           scratchMem + 0x24
     *
     *    gdt limit << 16                     scratchMem + LIM_GDT_OFFSET
     *    gdt base                            scratchMem + BASE_GDT_OFFSET
     *    address of page directory           scratchMem + CR3_OFFSET
     *
     *    idt limit                           scratchMem + LIM_IDT_OFFSET
     *    idt base address                    scratchMem + BASE_IDT_OFFSET
     *
     *    initial AP stack addr               scratchMem + AP_STK_OFFSET
     */

    /* Setup Initial AP... */

    /*
     * scratchPoint saves location of starting point for AP scratch memory,
     * APs offset based on this value for unique memory space...
     * if AP #1, this location doubles as first valid location of AP
     * scratch memory...
     */

    *((int *)((ULONG)scratchPoint)) = (unsigned int)scratchMem;

    /*
     * first valid location of scratch memory,
     * setup offset for AP specific scratch mem
     */

    tmp_stk = (unsigned int *)((ULONG)scratchMem);
    tmp_stk[0] = (unsigned int)scratchMem;

    /* setup intial GDT values */

    tmpPtr = (unsigned int *) (MMU_CVT_PHYS_DIR(&sysGdt32));

    /* place tmp_gdt in memory */

    for (ix = 0; ix < NUM_GDT_ENTRIES; ix++)
        tmp_stk[ix+1] = *tmpPtr++;

    /* (num bytes - 1) << 16 */

    *((int *)((ULONG)(scratchMem + LIM_GDT_OFFSET))) = (int)0x270000;
    *((int *)((ULONG)(scratchMem + BASE_GDT_OFFSET))) =
        (int)(scratchMem + 0x04);

    /* setup initial cr3 */

    *((unsigned int *)((ULONG)(scratchMem + CR3_OFFSET))) = vxCr3Get();

    /* setup initial IDT values */

    baseAddr = intVecBaseGet();

    /* (IDT limit) << 16 */

    *((int *)((ULONG)(scratchMem + LIM_IDT_OFFSET))) = (int)0x0fff0000;

    /* IDT base address */

    *((ULONG *)((ULONG)(scratchMem + BASE_IDT_OFFSET))) = (ULONG)baseAddr;

    /* initial AP Stack Address */

    *((ULONG *)((ULONG)(scratchMem + AP_STK_OFFSET))) =
        (ULONG)cpuState->regs.rsp;

    (void) cacheFlush ((CACHE_TYPE)DATA_CACHE,
                (void *)((ULONG)entryPoint),
                (size_t)0x1000);         /* allow for code & scratch mem */

#ifdef _WRS_VX_IA_IPI_INSTRUMENTATION
    /* Zero IPI instrumentation for this CPU */

    bzero((char *)ipiCounters[cpu], sizeof(struct ipiCounters));
#endif /* _WRS_VX_IA_IPI_INSTRUMENTATION */

    /* BSP sends AP an INIT-IPI and STARTUP-IPI */

    if (ipiStartup ((int) cpu,
                    (UINT32) mploApicIndexTable[cpu],
                    (UINT32) entryPoint,
                    (UINT32) 2) != OK)
        {
        (void) printf ("\nipiStartup failed: %d\n", mploApicIndexTable[cpu]);
        status = ERROR;
        }

    /*
     * Wait for AP start up.
     * We do not always have to delay. When a processor is enabled it is
     * visible via vxCpuEnabled.
     */

    do
        {
        if ((1 << cpu) & vxCpuEnabled)
            break;
        else
            sysUsDelay (apDelay);

        } while (--apDelayCount > 0);

    /*
     * Set ERROR when AP did not start in time.
     * tRootTask will log CPU startup error to the console device.
     */

    if (((1 << cpu) & vxCpuEnabled) == 0)
        status = ERROR;

    /* Un-Map regions used to kick-off AP CPUs that are no longer required */

    (void) vmStateSet((VM_CONTEXT_ID)NULL,
               (VIRT_ADDR)ROUND_DOWN(physAddr, VM_PAGE_SIZE),
               ROUND_UP((physAddrEnd - physAddr), VM_PAGE_SIZE),
               MMU_ATTR_SPL_MSK | MMU_ATTR_VALID_MSK,
               MMU_ATTR_NO_BLOCK | MMU_ATTR_VALID_NOT);

#ifdef INCLUDE_HYPERVISOR
    (void) vmStateSet((VM_CONTEXT_ID)NULL,
               (VIRT_ADDR)0UL,
               ROUND_UP(&sysTableEnd, VM_PAGE_SIZE),
               MMU_ATTR_SPL_MSK | MMU_ATTR_VALID_MSK,
               MMU_ATTR_NO_BLOCK | MMU_ATTR_VALID_NOT);
#else
    (void) vmStateSet((VM_CONTEXT_ID)NULL,
               (VIRT_ADDR)0UL,
               ROUND_DOWN(EBDA_START, VM_PAGE_SIZE),
               MMU_ATTR_SPL_MSK | MMU_ATTR_VALID_MSK,
               MMU_ATTR_NO_BLOCK | MMU_ATTR_VALID_NOT);
#endif

    return (status);
    }


/*******************************************************************************
*
* sysCpuInit - AP CPU Initialization
*
* This routine is the first code executed on a secondary processor after
* startup. It is responsible for ensuring that the registers are set
* appropriately.
*
* In a mapped kernel, it is also responsible for setting up the core's MMU.
*
* Upon completion of these tasks, the core is ready to begin accepting
* kernel work. The address of the initial kernel code (typically
* windCpuEntry) has been placed in sysCpuInitTable[cpuNum].
*
* RETURNS: N/A
*
*/

void sysCpuInit (void)
    {
    unsigned int a = 0;
    int cpuNum;
    FUNCPTR entry;
    STATUS retVal = OK;

    INT64 cpuKernelVars;
    unsigned int cpuIndex;

    sysInitGDT ();

    /* establish dedicated segment register for vxKernelVars */

    cpuIndex = vxCpuIndexGet();

    vxKernelVars[cpuIndex].vars.cpu_archVars.cpu_cpuIndex = cpuIndex;
    cpuKernelVars = (INT64) &vxKernelVars[cpuIndex];
    vxKernelVars[cpuIndex].vars.cpu_archVars.cpu_cpuGsBaseCurrent =
        (ULONG) &vxKernelVars[cpuIndex];

    pentiumMsrSet (IA32_GS_BASE, (long long *) &cpuKernelVars);
    pentiumMsrSet (MSR_IA32_KERNEL_GS_BASE, (long long *) &cpuKernelVars);

    /* enable the MTRR (Memory Type Range Registers) */

    if ((vxCpuId.std.featuresEdx & CPUID_MTRR) == CPUID_MTRR)
        {
        pentiumMtrrDisable ();      /* disable MTRR */
#ifdef INCLUDE_MTRR_GET
        (void) pentiumMtrrGet (&sysMtrr); /* get MTRR initialized by BIOS */
#else
        (void) pentiumMtrrSet (&sysMtrr); /* set your own MTRR */
#endif /* INCLUDE_MTRR_GET */
        pentiumMtrrEnable ();       /* enable MTRR */
        }

#ifdef INCLUDE_HW_FP
    /* enable FP features */

    fppArchCpuInit();
#endif /* INCLUDE_HW_FP */

#ifdef INCLUDE_PMC

    /* enable PMC (Performance Monitoring Counters) */

    pentiumPmcStop ();          /* stop PMC0 and PMC1 */
    pentiumPmcReset ();         /* reset PMC0 and PMC1 */
#endif /* INCLUDE_PMC */

    /* enable the MCA (Machine Check Architecture) */

    pentiumMcaEnable (TRUE);

    cpuNum = vxCpuPhysIndexGet();

    loApicIntrInitAP(cpuNum);

    entry = sysCpuInitTable[cpuNum];

    /*
     * update GDT. LDT, ect. depending on configuration...
     *
     * Note: order here is important, GDT updates for RTPs
     *       must come before OSM support, if included...
     */

#if defined (INCLUDE_RTP)
    retVal = syscallArchInit ();
#endif /* INCLUDE_RTP */

    if (retVal == OK)
        {
#if defined INCLUDE_PROTECT_TASK_STACK ||       \
    defined INCLUDE_PROTECT_INTERRUPT_STACK
#if defined(_WRS_OSM_INIT)
#if defined INCLUDE_PROTECT_TASK_STACK
        retVal = excOsmInit (TASK_USER_EXC_STACK_OVERFLOW_SIZE, VM_PAGE_SIZE);
#else  /* INCLUDE_PROTECT_TASK_STACK */
        retVal = excOsmInit (0, VM_PAGE_SIZE);
#endif /* INCLUDE_PROTECT_TASK_STACK */
#endif /* defined(_WRS_OSM_INIT) */
#endif  /* INCLUDE_PROTECT_TASK_STACK || INCLUDE_PROTECT_INTERRUPT_STACK */

        if ((entry != NULL) && (retVal == OK))
            {
            entry ();

            /* should be no return */

            }
        }

    for (;;)
        {
        if (!(++a % 0x10000))
            {
            sysCpuLoopCount[cpuNum]++;
            }
        }
    }


/*******************************************************************************
*
* sysCpuRe_Enable - re-enable a multi core CPU
*
* This routine brings an AP CPU through a cpu reset cycle and back on line.
*
* RETURNS:  OK - Success
*           ERROR - Fail
*
*/

STATUS sysCpuRe_Enable
    (
    unsigned int cpuNum
    )
    {
    STATUS status = OK;

    if ((cpuNum < 1) || (cpuNum > (vxCpuCount - 1)))
        {
        return (ERROR);
        }

    status = sysCpuStop ((int)cpuNum);
    status = sysCpuReset ((int)cpuNum);
    status = kernelCpuEnable (cpuNum);

    return (status);
    }


/*******************************************************************************
*
* sysCpuDisable - disable a multi core CPU
*
* This routine shuts down the specified core.
*
* RETURNS:  OK - Success
*           ERROR - Fail
*
*/

STATUS sysCpuDisable
    (
    int cpuNum
    )
    {
    if ((cpuNum < 1) || (cpuNum > (vxCpuCount - 1)))
        {
        return (ERROR);
        }

    return (sysCpuStop(cpuNum));
    }


/*******************************************************************************
*
* sysCpuStop - stop a CPU, for now, it is no longer available
*              for use
*
* The sysCpuStop() function takes one parameter:
* int cpu;              /@ core number to stop @/
*
* RETURNS:  OK - Success
*           ERROR - Fail
*
* \NOMANUAL
*
*/

STATUS sysCpuStop
    (
    int cpu
    )
    {
    STATUS status = OK;
    int key;   /* prevent task migration */
    UINT8 *mploApicIndexTable;

    (void) taskCpuLock();       /* if not in interrupt context, taskCpuLock */

    key = intCpuLock ();   /* prevent task migration */

    (void) mpApicLoIndexTableGet ((void *)&mploApicIndexTable);

    /* BSP sends AP the shutdown IPI */

    (void) CPUSET_ATOMICCLR (vxCpuEnabled, cpu);
    status = ipiShutdown (cpu,
                          (UINT32) mploApicIndexTable[cpu],
                          INT_NUM_LOAPIC_IPI + 0);

    intCpuUnlock (key);

    (void) taskCpuUnlock();     /* if not in interrupt context, taskCpuUnlock */

    return (status);
    }


/*******************************************************************************
*
* sysCpuStop_APs - Stop all Application Processors (APs)
*
* Can only run this routine from boot strap processor (BSP)
*
* RETURNS:  OK - Success
*           ERROR - Fail
*
* \NOMANUAL
*
*/

STATUS sysCpuStop_APs (void)
    {
    STATUS status = OK;
    int i;
    int key;   /* prevent task migration */
    UINT8 *mploApicIndexTable;
    int numCpus = vxCpuConfiguredGet();

    (void) taskCpuLock();       /* if not in interrupt context, taskCpuLock */

    key = intCpuLock ();   /* prevent task migration */

    (void) mpApicLoIndexTableGet ((void *)&mploApicIndexTable);

    /* Skip BSP, i == 0 */

    for (i=1; i < numCpus; i++)
        {

        /* BSP sends AP the shutdown IPI */

        (void) CPUSET_ATOMICCLR (vxCpuEnabled, i);
        status = ipiShutdown (i,
                              (UINT32) mploApicIndexTable[i],
                              INT_NUM_LOAPIC_IPI + 0);
        }

    intCpuUnlock (key);

    (void) taskCpuUnlock();     /* if not in interrupt context, taskCpuUnlock */

    return (status);
    }


/*******************************************************************************
*
* sysCpuStop_ABM - Stop all APs but the one I'm running on, (All but me...)
*
* RETURNS:  OK - Success
*           ERROR - Fail
*
* \NOMANUAL
*
*/

STATUS sysCpuStop_ABM (void)
    {
    STATUS status = OK;
    int i, skipProc;
    int key;   /* prevent task migration */
    UINT8 *mploApicIndexTable;
    int numCpus = vxCpuConfiguredGet();

    (void) taskCpuLock();       /* if not in interrupt context, taskCpuLock */

    key = intCpuLock ();   /* prevent task migration */

    (void) mpApicLoIndexTableGet ((void *)&mploApicIndexTable);

    skipProc = vxCpuIndexGet ();

    for (i=1; i < numCpus; i++)
        {
        if (i != skipProc)
            {

            /* sends AP the shutdown IPI */

            (void) CPUSET_ATOMICCLR (vxCpuEnabled, i);
            status = ipiShutdown (i,
                                  (UINT32) mploApicIndexTable[i],
                                  INT_NUM_LOAPIC_IPI + 0);
            }
        }

    intCpuUnlock (key);

    (void) taskCpuUnlock();     /* if not in interrupt context, taskCpuUnlock */

    return (status);
    }


/*******************************************************************************
*
* sysCpuReset - Reset a CPU
*
* Places the specified Application Processor (AP) into the
* INIT reset state, i.e. wait-for-SIPI state.
*
* The sysCpuReset() function takes one parameter:
* int cpu;              /@ core number to stop @/
*
* RETURNS:  OK - Success
*           ERROR - Fail
*
* \NOMANUAL
*
*/

STATUS sysCpuReset
    (
    int cpu
    )
    {
    STATUS status = OK;
    int key;   /* prevent task migration */
    UINT8 *mploApicIndexTable;

    (void) taskCpuLock();       /* if not in interrupt context, taskCpuLock */

    key = intCpuLock ();   /* prevent task migration */

    (void) mpApicLoIndexTableGet ((void *)&mploApicIndexTable);

    /* sends AP the INIT IPI */

    (void) CPUSET_ATOMICCLR (vxCpuEnabled, cpu);
    status = ipiReset (cpu,
                       (UINT32) mploApicIndexTable[cpu]);

    intCpuUnlock (key);

    (void) taskCpuUnlock();     /* if not in interrupt context, taskCpuUnlock */

    return (status);
    }


/*******************************************************************************
*
* sysCpuReset_APs - Reset all Application Processors (APs)
*
* Can only run this routine from BSP.
*
* RETURNS:  OK - Success
*           ERROR - Fail
*
* \NOMANUAL
*
*/

STATUS sysCpuReset_APs (void)
    {
    STATUS status = OK;
    int i;
    int key;   /* prevent task migration */
    UINT8 *mploApicIndexTable;
    int numCpus = vxCpuConfiguredGet();

    (void) taskCpuLock();       /* if not in interrupt context, taskCpuLock */

    key = intCpuLock ();   /* prevent task migration */

    (void) mpApicLoIndexTableGet ((void *)&mploApicIndexTable);

    /* Skip BSP, i == 0 */

    for (i=1; i < numCpus; i++)
        {

        /* BSP sends AP the INIT IPI */

        (void) CPUSET_ATOMICCLR (vxCpuEnabled, i);
        status = ipiReset (i,
                           (UINT32) mploApicIndexTable[i]);
        }

    intCpuUnlock (key);

    (void) taskCpuUnlock();     /* if not in interrupt context, taskCpuUnlock */

    return (status);
    }


/*******************************************************************************
*
* sysCpuReset_ABM - Reset all APs but the one I'm running on, (All but me...)
*
* RETURNS:  OK - Success
*           ERROR - Fail
*
* \NOMANUAL
*
*/

STATUS sysCpuReset_ABM (void)
    {
    STATUS status = OK;
    int i, skipProc;
    int key;   /* prevent task migration */
    UINT8 *mploApicIndexTable;
    int numCpus = vxCpuConfiguredGet();

    (void) taskCpuLock();       /* if not in interrupt context, taskCpuLock */

    key = intCpuLock ();   /* prevent task migration */

    (void) mpApicLoIndexTableGet ((void *)&mploApicIndexTable);

    skipProc = vxCpuIndexGet ();

    for (i=1; i < numCpus; i++)
        {
        if (i != skipProc)
            {

            /* sends AP the INIT IPI */

            (void) CPUSET_ATOMICCLR (vxCpuEnabled, i);
            status = ipiReset (i,
                               (UINT32) mploApicIndexTable[i]);
            }
        }

    intCpuUnlock (key);

    (void) taskCpuUnlock();     /* if not in interrupt context, taskCpuUnlock */

    return (status);
    }
#endif /* _WRS_CONFIG_SMP */

/* Static configuration of mpApic data table */


#ifdef INCLUDE_USR_BOOT_OP

/*
 * Static configuration table for mpApicData structure, which allows
 * for manual configuration of the mpApic driver.
 *
 * One must be very careful when manually configuring APICs,
 * since incorrect data will cause catastrophic error, i.e. BSP
 * failing to boot properly.
 *
 * The data here is dependent on the number of cores involved,
 * interrupt routing tables, and basically the specific configuration
 * of your hardware.
 *
 * Access to this capability is obtained by defining INCLUDE_USR_BOOT_OP
 * in config.h or by adding the INCLUDE_USR_BOOT_OP component.
 */

#include "usrMpApicData.c"


/*******************************************************************************
*
* sysStaticMpApicDataGet - Copies static MpApic data into table
*
* Copies static MpApic data into table pointed to by pMpApicData.
*
* The sysStaticMpApicDataGet() function takes one parameter:
* MP_APIC_DATA *pMpApicData; /@ pointer to the MpApic Table @/
*
* RETURNS: N/A
*
* \NOMANUAL
*
*/

void sysStaticMpApicDataGet
    (
    MP_APIC_DATA * pMpApicData
    )
    {
    INT32 oldLevel = intCpuLock();  /* LOCK INTERRUPTS */

    /* initialize the memory allocated */

    bzero((char *)pMpApicData, sizeof(MP_APIC_DATA));

    /*
     * mpApicData structure - structure holding mpApic details retrieved from
     *                        BIOS/ACPI/or USER defined methods.
     */

    bcopy ((char *) staticMpApicData,
           (char *) pMpApicData,
           (int) sizeof (MP_APIC_DATA));

    /* logical io apic id to address mapping */

    bcopy ((char *) staticMpApicAddrTable,
           (char *) MPAPIC_PHYS_TO_VIRT(mpApicAtLoc),
           (size_t) pMpApicData->mpApicAtSize);

    /* recorded id mapping */

    bcopy ((char *) staticMpApicLogicalTable,
           (char *) MPAPIC_PHYS_TO_VIRT(mpApicLtLoc),
           (size_t) pMpApicData->mpApicLtSize);

    /* interrupt routing table */

    bcopy ((char *) staticMpApicInterruptTable,
           (char *) MPAPIC_PHYS_TO_VIRT(mpApicItLoc),
           (size_t) pMpApicData->mpApicItSize);

    /* bus routing table */

    bcopy ((char *) staticMpApicBusTable,
           (char *) MPAPIC_PHYS_TO_VIRT(mpApicBtLoc),
           (size_t) pMpApicData->mpApicBtSize);

    /* loApic Id translation */

    bcopy ((char *) staticMploApicIndexTable,
           (char *) MPAPIC_PHYS_TO_VIRT(mpApicLaLoc),
           (size_t) pMpApicData->mpApicLaSize);

    intCpuUnlock(oldLevel);         /* UNLOCK INTERRUPTS */
    }

 #endif /* INCLUDE_USR_BOOT_OP */

/*
 * sysPhysRamDesc contains one region of RAM that must be mapped at
 * LOCAL_MEM_LOCAL_ADRS..., more regions can be mapped at the
 * discretion of the kernel. Other entries are only used if the BSP
 * discovers a RAM block during system boot.
 */

LOCAL PHYS_MEM_DESC sysPhysRamDesc [] =
{
    {
    /* Kernel System memory in first 2GB */

    (VIRT_ADDR) LOCAL_MEM_LOCAL_ADRS,
    (PHYS_ADDR) LOCAL_MEM_VIRT_TO_PHYS(LOCAL_MEM_LOCAL_ADRS),
    LOCAL_MEM_SIZE,          /* may be modified at runtime */
    VM_STATE_MASK_FOR_ALL,
    VM_STATE_FOR_MEM_OS
    },

    /* Dummy entries*/

    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,
    (PHYS_ADDR) 0,
    0,
    0,
    0
    },

    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,
    (PHYS_ADDR) 0,
    0,
    0,
    0
    },

    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,
    (PHYS_ADDR) 0,
    0,
    0,
    0
    },

    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,
    (PHYS_ADDR) 0,
    0,
    0,
    0
    },

    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,
    (PHYS_ADDR) 0,
    0,
    0,
    0
    },

    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,
    (PHYS_ADDR) 0,
    0,
    0,
    0
    },

    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,
    (PHYS_ADDR) 0,
    0,
    0,
    0
    },

    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,
    (PHYS_ADDR) 0,
    0,
    0,
    0
    },

    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,
    (PHYS_ADDR) 0,
    0,
    0,
    0
    },

    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,
    (PHYS_ADDR) 0,
    0,
    0,
    0
    },

    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,
    (PHYS_ADDR) 0,
    0,
    0,
    0
    }
};
LOCAL int sysPhysRamDescCount = NELEMENTS(sysPhysRamDesc);

#ifdef INCLUDE_CACHE_DMA32_LIB

/*
 * sysDma32MemDesc is used to provide 32-bit DMA memory
 * if INCLUDE_DMA32_MEM is defined.
 */

LOCAL PHYS_MEM_DESC sysDma32MemDesc[] =
    {

    /* if 32-bit DMA is needed, this entry is used */

    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,  /* kernel assigned */
    (PHYS_ADDR) 0,    /* computed at runtime */
    0,                /* computed at runtime */
    VM_STATE_MASK_FOR_ALL,
#if     ((USER_D_CACHE_MODE & CACHE_SNOOP_ENABLE) == CACHE_SNOOP_ENABLE)
    VM_STATE_FOR_MEM_OS
#else
    VM_STATE_FOR_IO
#endif  /* (USER_D_CACHE_MODE & CACHE_SNOOP_ENABLE) == CACHE_SNOOP_ENABLE */
    }

    };
LOCAL int sysDma32MemDescCount = NELEMENTS(sysDma32MemDesc);
#endif /* INCLUDE_CACHE_DMA32_LIB */

#ifdef INCLUDE_USER_RESERVED_MEMORY

/*
 * sysUserMemDesc is used to provide user reserved memory
 * if INCLUDE_USER_RESERVED_MEMORY is defined.
 */

LOCAL PHYS_MEM_DESC sysUserMemDesc[] =
    {
    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,  /* kernel assigned */
    (PHYS_ADDR) 0,    /* computed at runtime */
    0,                /* computed at runtime */
    VM_STATE_MASK_FOR_ALL,
    VM_STATE_FOR_MEM_OS
    }
    };
LOCAL int sysUserMemDescCount = NELEMENTS(sysUserMemDesc);
#endif /* INCLUDE_USER_RESERVED_MEMORY */

#ifdef INCLUDE_EDR_PM

/*
 * sysPmMemDesc is used to provide persistent memory
 * if INCLUDE_EDR_PM is defined.
 */

LOCAL PHYS_MEM_DESC sysPmMemDesc[] =
    {
    {
    (VIRT_ADDR) MEM_DESC_ADDR_KERNEL_ASSIGNED,  /* kernel assigned */
    (PHYS_ADDR) 0,    /* computed at runtime */
    0,                /* computed at runtime */
    VM_STATE_MASK_FOR_ALL,
    VM_STATE_FOR_MEM_OS
    }
    };
LOCAL int sysPmMemDescCount = NELEMENTS(sysPmMemDesc);
#endif /* INCLUDE_EDR_PM */


/*******************************************************************************
*
* sysMemDescInit - hook for memory auto sizing, used to perform any
*                  adjustemnts/clean up prior to calling sysMemDescGet
*
* RETURNS: N/A
*
*/

void sysMemDescInit (void)
    {
    LOCAL BOOL memFirstTime = TRUE;
#if defined(INCLUDE_MULTIBOOT_MEM_AUTOSIZE) || \
    defined(INCLUDE_BIOS_E820_MEM_AUTOSIZE)
    STATUS multibootStatus = ERROR;
#endif

    /*
     * The first time we're called, we patch the descriptors based
     * upon run-time tests of the various memory segments. The
     * details of how the memory sizing tests are performed is
     * outside the scope of this document.
     */

    if (memFirstTime)
        {
        size_t ramSize;
#if defined(INCLUDE_CACHE_DMA32_LIB) ||      \
    defined(INCLUDE_USER_RESERVED_MEMORY) || \
    defined(INCLUDE_EDR_PM)
        PHYS_ADDR       physicalAddrEnd;
#endif /* INCLUDE_CACHE_DMA32_LIB      || */
       /* INCLUDE_USER_RESERVED_MEMORY || */
       /* INCLUDE_EDR_PM                  */
#ifdef INCLUDE_CACHE_DMA32_LIB
        size_t dma32Size = DMA32_HEAP_SIZE;
#else
        size_t dma32Size = 0;
#endif /* INCLUDE_CACHE_DMA32_LIB */
#ifdef INCLUDE_USER_RESERVED_MEMORY
        size_t userSize = USER_RESERVED_MEM;
#else
        size_t userSize = 0;
#endif /* INCLUDE_USER_RESERVED_MEMORY */
#ifdef INCLUDE_EDR_PM
        size_t pmSize = PM_RESERVED_MEM;
#else
        size_t pmSize = 0;
#endif /* INCLUDE_EDR_PM */

#ifdef INCLUDE_MULTIBOOT_MEM_AUTOSIZE
        multibootStatus = multibootMapGet64(sysPhysRamDesc,sysPhysRamDescCount);
#endif

#ifdef INCLUDE_BIOS_E820_MEM_AUTOSIZE

        /*
         * Populate sysPhysRamDesc using E820 memory map.
         * The routine returns status indicating successful population
         * of the sysPhysRamDesc table. If ERROR is returned, the
         * table was not updated. BSP developers can choose to respond
         * to this condition with user notification, or other actions.
         */

        if (multibootStatus == ERROR &&
            vxBiosE820MapStatusGet () == BIOS_E820_MAP_OK)
            (void) vxBiosE820MapGet64(sysPhysRamDesc,sysPhysRamDescCount);

#endif /* INCLUDE_BIOS_E820_MEM_AUTOSIZE */

        ramSize = sysPhysRamDesc[0].len;

#if defined(INCLUDE_FAST_REBOOT) && defined(INCLUDE_MULTI_STAGE_BOOT)
        memRomSize = ROUND_UP((*(UINT32 *)BOOT_IMAGE_SIZE), VM_PAGE_SIZE);
        memRom = (UINT32) ROUND_DOWN((sysPhysRamDesc[0].physicalAddr +
                                      ramSize - memRomSize),
                                     VM_PAGE_SIZE);
        *(UINT32 *)BOOT_IMAGE_ADRS = memRom;

        /* only capable of doing fast reboots if less than 0xffffffff */

        if ((memRom + memRomSize) > 0xffffffff)
            {
            /* cannot do fast reboot */

            memRom = 0;
            memRomSize = 0;
            *(UINT32 *)BOOT_IMAGE_ADRS = 0;
            *(UINT32 *)BOOT_IMAGE_SIZE = 0;
            }
#else
        memRom = 0;
        memRomSize = 0;
#endif  /* defined(INCLUDE_FAST_REBOOT) && defined(INCLUDE_MULTI_STAGE_BOOT) */

        /* shrink 0th segment by reserved sizes */

        sysPhysRamDesc[0].len = ramSize - dma32Size - userSize - pmSize;
        if (memRomSize > 0)
            {
            sysPhysRamDesc[0].len -= sysPhysRamDesc[0].physicalAddr + ramSize -
                memRom;
            }

#if defined(INCLUDE_CACHE_DMA32_LIB) ||      \
    defined(INCLUDE_USER_RESERVED_MEMORY) || \
    defined(INCLUDE_EDR_PM)

        physicalAddrEnd =
            sysPhysRamDesc[0].physicalAddr + sysPhysRamDesc[0].len;

#endif /* INCLUDE_CACHE_DMA32_LIB      || */
       /* INCLUDE_USER_RESERVED_MEMORY || */
       /* INCLUDE_EDR_PM                  */

#ifdef INCLUDE_CACHE_DMA32_LIB

        /* set physical address and size for 32-bit DMA */

        sysDma32MemDesc[0].len = dma32Size;
        sysDma32MemDesc[0].physicalAddr = physicalAddrEnd;
        physicalAddrEnd += sysDma32MemDesc[0].len;
#endif /* INCLUDE_CACHE_DMA32_LIB */

#ifdef INCLUDE_USER_RESERVED_MEMORY

        /* set physical address and size for user reserved memroy */

        sysUserMemDesc[0].len = userSize;
        sysUserMemDesc[0].physicalAddr = physicalAddrEnd;
        physicalAddrEnd += sysUserMemDesc[0].len;
#endif /* INCLUDE_USER_RESERVED_MEMORY */

#ifdef INCLUDE_EDR_PM

        /* set physical address and size for persistent memroy */

        sysPmMemDesc[0].len = pmSize;
        sysPmMemDesc[0].physicalAddr = physicalAddrEnd;
#endif /* INCLUDE_EDR_PM */

        memFirstTime = FALSE;
        }

    }


/*******************************************************************************
*
* sysMemDescGet - Return memory descriptors describing memory layout
*
* This routine is used during system boot to describe the memory
* layout to the kernel and included components. It supports
* memory autoconfiguration. It also supports carving memory from
* the first contiguous block of RAM for 32-bit DMA and persistent
* memory. This routine assumes that enough memory is provided
* in the 0th descriptor to support all of these segments.
*
* RETURNS: OK, if a valid PHYS_MEM_DESC is copied to the caller.
*
*/

STATUS sysMemDescGet
    (
    MEM_DESC_TYPE memDescType, /* type of memory being queried */
    int index, /* index of memory block for the type */
    PHYS_MEM_DESC *pMemDesc /* to return the description */
    )
    {
    PHYS_MEM_DESC *pDesc;
    int descCount;

    /*
     * Provide the correct array and size parameters based upon the
     * query type provided by the caller.
     */

    if (memDescType == MEM_DESC_RAM)
        {
        pDesc = sysPhysRamDesc;
        descCount = sysPhysRamDescCount;
        }
#ifdef INCLUDE_USER_RESERVED_MEMORY
    else if (memDescType == MEM_DESC_USER_RESERVED_RAM)
        {
        pDesc = sysUserMemDesc;
        descCount = sysUserMemDescCount;
        }
#endif /* INCLUDE_USER_RESERVED_MEMORY */
#ifdef INCLUDE_EDR_PM
    else if (memDescType == MEM_DESC_PM_RAM)
        {
        pDesc = sysPmMemDesc;
        descCount = sysPmMemDescCount;
        }
#endif /* INCLUDE_EDR_PM */
#ifdef INCLUDE_CACHE_DMA32_LIB
    else if (memDescType == MEM_DESC_DMA32_RAM)
        {
        pDesc = sysDma32MemDesc;
        descCount = sysDma32MemDescCount;
        }
#endif /* INCLUDE_CACHE_DMA32_LIB */
    else
        {
        return ERROR; /* no support for MEM_DESC_APPL_RAM in this BSP */
        }

    if (index >= descCount) /* no descriptor at requested index */
        return ERROR;

    /*
     * copy the descriptor. Note that it's OK to return a descriptor that
     * has a length of 0. This allows the BSP to allocate "empty" entries
     * in the descriptor arrays, and fill them in at run-time without having
     * to alter the logic of this routine.
     */

    *pMemDesc = pDesc[index];
    return OK;
    }


/*******************************************************************************
*
* sysMemDescShow - dump descriptors
*
* RETURNS: OK
*
*/

STATUS sysMemDescShow (void)
    {
    MEM_DESC_TYPE mdt;      /* type of memory being queried */
    int i;                  /* index of memory block for the type */
    PHYS_MEM_DESC prm;      /* to return the ram description */
#ifdef INCLUDE_CACHE_DMA32_LIB
    PHYS_MEM_DESC pd32rm;   /* to return the 32-bit DMA description */
#endif /* INCLUDE_CACHE_DMA32_LIB */
#ifdef INCLUDE_USER_RESERVED_MEMORY
    PHYS_MEM_DESC puser;    /* to return the user reserved memory description */
#endif /* INCLUDE_USER_RESERVED_MEMORY */
#ifdef INCLUDE_EDR_PM
    PHYS_MEM_DESC ppm;      /* to return the persistent description */
#endif /* INCLUDE_EDR_PM */

    mdt = MEM_DESC_RAM;

    for (i=0; sysMemDescGet (mdt,i,&prm) != ERROR; i++)
        {
        (void) printf("\nPHYS_MEM_DESC prm : \n");
        (void) printf("  virtualAddr %p\n", (void *) prm.virtualAddr);
        (void) printf("  physicalAddr %p\n", (void *) prm.physicalAddr);
        (void) printf("  len %p\n", (void *) prm.len);
        (void) printf("  initialStateMask 0x%x\n", prm.initialStateMask);
        (void) printf("  initialState 0x%x\n", prm.initialState);
        }

#ifdef INCLUDE_CACHE_DMA32_LIB
    mdt = MEM_DESC_DMA32_RAM;

    for (i=0; sysMemDescGet (mdt,i,&pd32rm) != ERROR; i++)
        {
        (void) printf("\nPHYS_MEM_DESC pd32rm : \n");
        (void) printf("  virtualAddr %p\n", (void *) pd32rm.virtualAddr);
        (void) printf("  physicalAddr %p\n", (void *) pd32rm.physicalAddr);
        (void) printf("  len %p\n", (void *) pd32rm.len);
        (void) printf("  initialStateMask 0x%x\n", pd32rm.initialStateMask);
        (void) printf("  initialState 0x%x\n", pd32rm.initialState);
        }
#endif /* INCLUDE_CACHE_DMA32_LIB */

#ifdef INCLUDE_USER_RESERVED_MEMORY
    mdt = MEM_DESC_USER_RESERVED_RAM;

    for (i=0; sysMemDescGet (mdt,i,&puser) != ERROR; i++)
        {
        (void) printf("\nPHYS_MEM_DESC puser : \n");
        (void) printf("  virtualAddr %p\n", (void *) puser.virtualAddr);
        (void) printf("  physicalAddr %p\n", (void *) puser.physicalAddr);
        (void) printf("  len %p\n", (void *) puser.len);
        (void) printf("  initialStateMask 0x%x\n", puser.initialStateMask);
        (void) printf("  initialState 0x%x\n", puser.initialState);
        }
#endif /* INCLUDE_USER_RESERVED_MEMORY */

#ifdef INCLUDE_EDR_PM
    mdt = MEM_DESC_PM_RAM;

    for (i=0; sysMemDescGet (mdt,i,&ppm) != ERROR; i++)
        {
        (void) printf("\nPHYS_MEM_DESC ppm : \n");
        (void) printf("  virtualAddr %p\n", (void *) ppm.virtualAddr);
        (void) printf("  physicalAddr %p\n", (void *) ppm.physicalAddr);
        (void) printf("  len %p\n", (void *) ppm.len);
        (void) printf("  initialStateMask 0x%x\n", ppm.initialStateMask);
        (void) printf("  initialState 0x%x\n", ppm.initialState);
        }
#endif /* INCLUDE_EDR_PM */

    return (OK);
    }


#ifdef INCLUDE_EARLY_PRINT
/*******************************************************************************
*
* earlyPrint - early serial output service
*
* This routine provides a method to output serial characters to the
* RS232 UART port before printf services are available.
*
* RETURNS: N/A
*
* NOMANUAL
*
*/

void earlyPrint
    (
    char * outString
    )
    {
    if (outString)
        (void) kwriteSerial(outString, strlen(outString));
    }


/*******************************************************************************
*
* kwriteSerial - early serial output to a console UART port
*
* This routine provides a method to output serial characters from a buffer to
* a console serial port before printf services are available and before
* sysHwInit(). No initialization is required. It sends directly to the UART and
* does not need SIO serial driver support.
*
* This function can be used as a user kwrite routine to support kputs or kprintf
* formatted output, and since no spin locks are involved it can be used with
* no restriction in SMP.
*
* kprintf and kputs prototypes are available by including <stdio.h>.
* The _func_kwrite prototype is obtained from <private/kwriteLibP.h>.
* kwriteSerial can be dynamically registered or reregistered as the current
* user kwrite routine with:
*
*    _func_kwrite = &kwriteSerial;
*
* See pkgs/os/core/kernel/src/services/{kprintfLib.c,kputsLib.c} for additional
* information on including kprintf/kputs support and registering this function
* as a user kwrite routine (DEBUG_KWRITE_USR_RTN) in a VxWorks Image Project.
*
* RETURNS: STATUS
*
* NOMANUAL
*
*/

STATUS kwriteSerial
    (
    char * pBuffer,
    size_t len
    )
    {
#define DIV_ROUND(a, b) (((a) + ((b)/2)) / (b))
#define UART_REFCLK_HZ  DRV_SIO_IA_NS16550_INSTANCE_CLK_FREQ
#define UART_BASE       DRV_SIO_IA_NS16550_INSTANCE_0_BASE_ADDR

    int i;
    UINT32  divisor;
    int max_tries = 1000;
    int tries = 0;

    if (!kwriteSerialInitialized)
        {
        /* LDR on, set baud rate, LDR off with 8-bits|no parity|1 stop bit */

        divisor = DIV_ROUND(UART_REFCLK_HZ/16, CONSOLE_BAUD_RATE);
        sysOutByte(UART_BASE + LCR, 0x80);
        sysOutByte(UART_BASE + DLL, (divisor & 0x00FFU));
        sysOutByte(UART_BASE + DLM, ((divisor >> 8) & 0x00FFU));
        sysOutByte(UART_BASE + LCR, 0x3);
        kwriteSerialInitialized = 1;
        }

    if (pBuffer)
        {
        for (i = 0; i < len; i++)
            {
            if (pBuffer[i] == '\n')
                {
                tries = 0;
                while ((sysInByte(UART_BASE + LSR) & LSR_THRE) != LSR_THRE)
                    {
                    tries ++;
                    if (tries >= max_tries)
                        break;
                    }
                sysOutByte(UART_BASE + THR, '\r');
                }
            tries = 0;
            while ((sysInByte(UART_BASE + LSR) & LSR_THRE) != LSR_THRE)
                {
                tries ++;
                if (tries >= max_tries)
                    break;
                }
            sysOutByte(UART_BASE + THR, pBuffer[i]);
            }
        return OK;
        }
    else
        return ERROR;

    }


/*******************************************************************************
*
* vgaPutc - write a character to VGA display buffer
*
* RETURNS: N/A
*
* NOMANUAL
*
*/

void vgaPutc
    (
    UINT offset,
    unsigned char a
    )
    {

    /*
     * A utility function to print a white character on red background
     * at a given offset within the VGA Color Text address range.
     */

    UINT16 * addr = (UINT16 *)vgaBufferVirtAddress;

    UINT16 c = 0;
    c = a|0x1f00;
    addr += offset%(25*80);  /* valid offset is 0..(25*80)-1, otherwise wrap */
    *addr = c;
    }


/*******************************************************************************
*
* kwriteVga - early serial output to a console VGA device
*
* This routine provides a method to output serial characters from a buffer to
* a console VGA device before printf services are available and before
* sysHwInit(). No initialization is required.
* It sends directly to the VGA device memory and does not require PC_CONSOLE
* driver support. kwriteVga provides very fast output without requiring wait
* states and is the least intrusive for use inside interrupt routines.
*
* This function can be used as a user kwrite routine to support kputs or kprintf
* formatted output, and since no spin locks are involved it can be used with
* no restriction in SMP.
*
* When using kwriteVga output it is best to not include PC_CONSOLE or FB_CONSOLE
* support since these consoles do their own independent screen scrolling which
* results in interleaved or overwritten output between kwriteVga and these
* consoles. However, both earlyPrint and earlyPrintVga (and thus kwriteSerial
* and kwriteVga) can be used simultaneously.
*
* kprintf and kputs prototypes are available by including <stdio.h> while
* the kwriteVga function prototype must be added to any file that uses it.
* The _func_kwrite prototype is obtained by including the file
* <private/kwriteLibP.h>. kwriteVga can be dynamically registered or
* reregistered as the current user kwrite routine with:
*
*    _func_kwrite = &kwriteVga;
*
* See pkgs/os/core/kernel/src/services/{kprintfLib.c,kputsLib.c} for
* additional information on including kprintf/kputs support and registering
* this function as a user kwrite routine (DEBUG_KWRITE_USR_RTN) in a VxWorks
* Image Project.
*
* RETURNS: STATUS
*
* NOMANUAL
*
*/

STATUS kwriteVga
    (
    char * pBuffer,
    size_t len
    )
    {

    if (vgaBufferVirtAddress == NULL)
        setVgaBufferVirtAddress();
    
    /*
     * aiop, 04jun18: pgf in brackets below is original but is no longer true
     * with this mod that makes kwriteVga scroll. Instead, when the offset
     * exceeds the 25*80 screen size, scroll lines 1-25 to be lines 0-24, clear
     * line 25, and set the offset to the beginning of line 25.
     * 
     * [Assumes a video mode with 80 columns and 25 rows, but uses only 80x24
     * reserving row 25 as a user controllable status area. The 80 character
     * status area can be written to independently with vgaPutc starting at
     * vgaStatusOffset = 24*80 = 1920.]
     */

    if (pBuffer)
        {
        while (len--)
            {
            if (*pBuffer == '\n')
                {
                vgaScreenOffset -= vgaScreenOffset%80;
                vgaScreenOffset += 80;
       
                if (vgaScreenOffset >= 25*80)           /* aiop, 04Jun 18: scroll change */
                    kwriteVgaScroll();                  /* aiop, 04Jun 18: scroll change */
                
                pBuffer++;                              /* aiop, 05may18: BUG */
                }
            else
                {
                if (vgaScreenOffset == (25*80 - 1))     /* aiop, 04Jun 18: scroll change */
                    kwriteVgaScroll();                  /* aiop, 04Jun 18: scroll change */
                    
                vgaPutc(vgaScreenOffset++, *pBuffer++);
                }
            }

        return OK;
        }
    else
        return ERROR;
}

/* aiop, 04Jun 18: augment start: add scroll and clear capabilities */

/*******************************************************************************
*
* kwriteVgaClearLine - clear the line given by the argument 0-24
* 
* Clear the a line given by the argument 0-24 and leave the cursor at the start
* of that line.
* 
* RETURNS: N/A
*/ 

void kwriteVgaClearLine (int line)
    {
    UINT16 *    addr;
    UINT16      c = 0;
    int         col;

    /* Set the flag byte for each "character" in the line. */
    
    addr = ((UINT16 *)vgaBufferVirtAddress) + line*80;
    
    for (col = 0; col<80; col++)
        {
        c = 0x1f00;
        *addr++ = c;
        }
 
    vgaScreenOffset = line*80;
    
    }
/*******************************************************************************
*
* kwriteVgaClearScreen - clear the VGA screen and set cursor at 0
* 
* RETURNS: N/A
*/ 

void kwriteVgaClearScreen()
    {
    int         line;

    memmove (vgaBufferVirtAddress, vgaBufferVirtAddress + 80*2, 24*80*2);

    /* Clear the lines in reverse order, which leaves the cursor at the start
     * of line 0. */
    
    for (line = 24; line >=0; line--)
        kwriteVgaClearLine (line);
    }

/*******************************************************************************
*
* kwriteVgaPageSizeSet - initialize screen "page" size
* 
* Initialize the "page size" for VGA output from kwriteVga(), with the number of 
* lines on the page given by the argument. After the given number of lines,
* kwriteVgaScroll()( will output message "Type any character to continue: " on 
* the next line and pause for user input. (There is no Q=stop option because 
* kwriteVga() (which calls kwriteVgaScroll()) has no provision to return a 
* "stop" signal to its caller.) Also resets kwriteVgaLineNum (the current line
* number on the page) to 0.
* 
* If the argument is 0, pagination is disabled. Otherwise, any positive number is
* valid. For a 25x80 screen, passing 24 will output up to 24 lines and then 
* write the "continue" message on the last line of the screen. 
* 
* RETURNS: N/A
*/

void kwriteVgaPageSizeSet (UINT lines)
    {
    kwriteVgaPageSizeLines = lines;
    kwriteVgaLineNum = 0;
    }

/*******************************************************************************
*
* kwriteVgaScroll - scroll the VGA and put cursor at start of lowest line
* 
* Scroll the VGA (move lines 2-24 to be 0-23), clear the line 24, and put the 
* cursor at the start of that line. Assumes the VGA is 25*80.
* 
* Page control: after scrolling, if module-static kwriteVgaPageSizeLines is >
* 0, then write a "continue" message and pause for user input. See 
* kwriteVgaPageSizeSet() for details.
* 
* RETURNS: N/A
*/ 

void kwriteVgaScroll (void)
    {
    memmove (vgaBufferVirtAddress, vgaBufferVirtAddress + 80*2, 24*80*2);
    kwriteVgaClearLine (24);

    if ((kwriteVgaPageSizeLines > 0) && (++kwriteVgaLineNum == kwriteVgaPageSizeLines))
        {
        earlyPrintVga("Type any character to continue:");
        vgaScreenOffset = kwriteVgaLineNum*80 + 
               strlen("Type any character to continue:") + 1;
        
        (void) getchar();           /* Discard character--don't care. */
        kwriteVgaLineNum = 0;
        }
    }

/* aiop, 04Jun 18: augment end */

/*******************************************************************************
*
* earlyPrintVga - early VGA output service
*
* This routine provides a method to output serial characters to a
* VGA graphics device before printf services are available.
*
* RETURNS: N/A
*
* NOMANUAL
*
*/

void earlyPrintVga
    (
    char * outString
    )
    {
    if (outString && (vgaBufferVirtAddress != (void *)(-1)))
        kwriteVga(outString, strlen(outString));
    }


#if defined (INCLUDE_ACPI)
/*******************************************************************************
*
* errorPrint - print information and pause when system panic
*
* This routine provides a method to output a string on both serial and VGA,
* then pause, which could be useful for early system initialization failure.
*
* RETURNS: N/A
*
* NOMANUAL
*
*/

void errorPrint
    (
    char * outString
    )
    {
#define ERROR_DELAY (1000 * 3) /* 3 seconds */

    /* print error information */

    if (outString)
        {
        earlyPrint(outString);
        earlyPrintVga(outString);
        }

    /* then pause here */

    sysMsDelay(ERROR_DELAY);

    }
#endif /* INCLUDE_ACPI */
#endif /* INCLUDE_EARLY_PRINT */


#ifdef INCLUDE_CPU_PWR_ARCH
#define MSR_APERF       0xE8
#define MSR_MPERF       0xE7
/*******************************************************************************
*
* sysPerformanceShow - Displays the current system performance setting
*
* This function reads the APERF and MPERF counters and determnes how many ticks
* expire in one second.  The clock rate of the APERF clock is
* tied to the CPU clock rate whereas the MPERF is fixed.  It prints out the
* elapsed time in ticks and percent CPU utilization  as an average over the
* measured second.
*
* This function is provided for test purposes only.
*
* RETURNS: N/A
*
* \NOMANUAL
*
*/

void sysPerformanceShow (void)
    {
    LL_UNION        aperfValue1, aperfValue2;
    LL_UNION        mperfValue1, mperfValue2;
    LL_UNION        regValue;
    long long int   aperfElapsed, mperfElapsed;
    double          pctPerformance;

    taskDelay (1);  /* synchronize to clock tick */
    pentiumMsrGet (MSR_APERF, &aperfValue1.i64);
    pentiumMsrGet (MSR_MPERF, &mperfValue1.i64);
    taskDelay (sysClkRateGet ()); /* wait one second */
    pentiumMsrGet (MSR_APERF, &aperfValue2.i64);
    pentiumMsrGet (MSR_MPERF, &mperfValue2.i64);

    aperfElapsed = aperfValue2.i64 - aperfValue1.i64;

    (void) printf ("APERF clock frequency = %lld\n", aperfElapsed);

    mperfElapsed = mperfValue2.i64 - mperfValue1.i64;
    (void) printf ("MPERF clock frequency = %lld\n", mperfElapsed);

    pctPerformance = ((aperfElapsed - mperfElapsed) * 100)/mperfElapsed + 100.0;
    (void) printf ("system performance rate = %2.2f pct\n", pctPerformance);

    /* read the performance register status value */

    pentiumMsrGet (0x0198, &regValue.i64);
    (void) printf("current performance status register = 0x%llx\n",
                  regValue.i64);
    }


/*******************************************************************************
*
* sysPerformanceGet - Compute the current system performance setting
*
* This function reads the APERF and MPERF counters and determnes how many ticks
* expire in one second.  The clock rate of the APERF clock is
* tied to the CPU clock rate whereas the MPERF is fixed.  It returns the
* percent CPU utilization as an average over the measured second.
*
* This function is provided for test purposes only.
*
* RETURNS: percent utilization
*
* \NOMANUAL
*
*/

UINT32 sysPerformanceGet (void)
    {
    LL_UNION        aperfValue1, aperfValue2;
    LL_UNION        mperfValue1, mperfValue2;
    volatile void   (* cpuPwrIdleEnterFunc) (void);
    long long int   aperfElapsed, mperfElapsed;
    UINT32          pctPerformance;

    taskDelay (1);  /* synchronize to clock tick */
    pentiumMsrGet (MSR_APERF, &aperfValue1.i64);
    pentiumMsrGet (MSR_MPERF, &mperfValue1.i64);
    taskDelay (sysClkRateGet ()); /* wait one second */
    pentiumMsrGet (MSR_APERF, &aperfValue2.i64);
    pentiumMsrGet (MSR_MPERF, &mperfValue2.i64);

    aperfElapsed = aperfValue2.i64 - aperfValue1.i64;

    mperfElapsed = mperfValue2.i64 - mperfValue1.i64;

    pctPerformance = ((aperfElapsed - mperfElapsed) * 100)/mperfElapsed + 100;
    return (pctPerformance);
    }
#endif /* INCLUDE_CPU_PWR_ARCH */
